#include "CaptureDriver.h"
#include "ScrollHook.h"

NTSTATUS __stdcall DispatchDummy(IN  PDEVICE_OBJECT  DeviceObject, IN  PIRP Irp);
VOID __stdcall DispatchUnload(IN  PDRIVER_OBJECT  DriverObject);
NTSTATUS DispatchDeviceControl(IN  PDEVICE_OBJECT  DeviceObject,IN  PIRP Irp);

extern void InstallScrollHook(HANDLE hProcess);
extern void UnInstallScrollHook(void);

extern SYS_SERVICE_TABLE *g_pShadowTable;

extern SCROLL_DATA g_ScrollData;
extern SCROLL_HOOK_TARGET g_ScrollHookTarget;

extern ULONG g_IsScrolled;

NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath)
{
	NTSTATUS  status;

	PDEVICE_OBJECT deviceObject;	
	UNICODE_STRING ntName;
	UNICODE_STRING win32Name;

	int i;

	DbgPrint("OpenCapture ScrollHook Driver Loaded ( Updated : 2009.5.30 )\n");

	RtlInitUnicodeString(&ntName, NT_DEVICE_NAME);
	status = IoCreateDevice(	DriverObject, 
								0, 
								&ntName, 
								CAPTURE_DEV_DRV, 
								0, 
								TRUE, 
								&deviceObject);
	if (FALSE == NT_SUCCESS(status))
	{
		KdPrint( ("%s >> IoCreateDevice fail", __FUNCTION__) );
		return status;
	}

	RtlInitUnicodeString (&win32Name, DOS_DEVICE_NAME);	
	status = IoCreateSymbolicLink ( 
				&win32Name, 
				&ntName);
	if (FALSE == NT_SUCCESS(status))
	{
		KdPrint( ("%s >> IoCreateSymbolicLink() failed", __FUNCTION__) );
		IoDeleteDevice( DriverObject ->DeviceObject );
		return status;
	}

	for (i = 0; i <= IRP_MJ_MAXIMUM_FUNCTION; i++) 
	{
		DriverObject->MajorFunction[i] = DispatchDummy;
	}
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DispatchDeviceControl;
	DriverObject->DriverUnload = DispatchUnload;
	
	return STATUS_SUCCESS;
}

NTSTATUS __stdcall DispatchDummy(
	IN  PDEVICE_OBJECT  DeviceObject,
	IN  PIRP            Irp
	)
{
	PIO_STACK_LOCATION CurIrpStack = IoGetCurrentIrpStackLocation(Irp);	
	PFILE_OBJECT pFileObject;
	UCHAR MajorFunction = CurIrpStack->MajorFunction;	

	Irp ->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	
	return STATUS_SUCCESS;
}

VOID __stdcall DispatchUnload(
	IN  PDRIVER_OBJECT  DriverObject
	)
{
	UNICODE_STRING Win32NameString;

	UnInstallScrollHook();

	RtlInitUnicodeString (&Win32NameString , DOS_DEVICE_NAME);	
	IoDeleteSymbolicLink (&Win32NameString);	
	IoDeleteDevice (DriverObject -> DeviceObject);
}

NTSTATUS DispatchDeviceControl(
	IN  PDEVICE_OBJECT  DeviceObject,
	IN  PIRP            Irp
	)
{
	PIO_STACK_LOCATION irpStack;
	PVOID inputBuffer;
	PVOID outputBuffer;
	ULONG inputBufferLength;
	ULONG outputBufferLength;
	ULONG ioControlCode;
	NTSTATUS ntstatus;

	ntstatus = Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	irpStack = IoGetCurrentIrpStackLocation(Irp);

	inputBuffer = Irp->AssociatedIrp.SystemBuffer;
	inputBufferLength = irpStack->Parameters.DeviceIoControl.InputBufferLength;

	outputBuffer = Irp->AssociatedIrp.SystemBuffer;
	outputBufferLength = irpStack->Parameters.DeviceIoControl.OutputBufferLength;

	ioControlCode = irpStack->Parameters.DeviceIoControl.IoControlCode;
	switch (irpStack->MajorFunction)
	{
		case IRP_MJ_CREATE:
			break;
		
		case IRP_MJ_CLOSE:
			break;

		case IRP_MJ_DEVICE_CONTROL:
			{
				switch (ioControlCode)
				{
					case IOCTL_DRV_INSTALL_SCROLLHOOK:
						{
							if (g_pShadowTable == NULL)
							{
								if (inputBufferLength == sizeof(SCROLL_HOOK_TARGET))
								{
									memcpy(&g_ScrollHookTarget, inputBuffer, sizeof(SCROLL_HOOK_TARGET));
									InstallScrollHook(g_ScrollHookTarget.hSelfProcess);
								}
							}
							else
							{
								KdPrint(("Already InstallScrollHook()"));
							}
						}
						break;
					case IOCTL_DRV_GET_SCROLLDATA:
						 KdPrint(("IOCTL_DRV_SCROLL_HOOK"));

						 if (sizeof(g_ScrollData) == outputBufferLength)
						 {
							// ,  ̶ ũ Ǿ üũմϴ.
							// g_IsScrolled    ( ScrollHook.c ) InterlockedExchange Լ ϹǷ
							// ׳   ڵ Ȱ Lock ɸ ȿ ߻մϴ.
							// , cmp eax, xxx  cmp atomic CPU  ʿ䰡 ٴ  ǹմϴ.
							 if (g_IsScrolled == FALSE)
							 {
								 // ,   ũ  ʾҽϴ.
								 KdPrint(("Not scrolled\n"));
										
								//  ʱȭŵϴ.
								 memset(outputBuffer, 0, sizeof(SCROLL_DATA));
							 }
							 else
							 {
								 // ũ Ǿϴ.
								KdPrint(("%d", g_ScrollData.dy));
								memcpy(outputBuffer, &g_ScrollData, sizeof(g_ScrollData));

								memset(&g_ScrollData, 0, sizeof(g_ScrollData));
							 }
							Irp->IoStatus.Information = sizeof(g_ScrollData);
						 }
						 else
						 {
							 KdPrint(("Invalid Size : g_ScrollData(%d) out(%d)", sizeof(g_ScrollData), outputBufferLength));
						 }
						break;
				}
			}
	}	
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	return ntstatus;
}