﻿unit AccessibleObject;

interface

uses Windows, Types, Variants, ActiveX, OleAcc, Classes, Graphics, SysUtils,
  Dialogs;

procedure AccessibleObjectRectFromWindow(WindowHandle: HWND; lpPoint: TPoint);

implementation

uses uCapture, OleAccDLL;

////////////////////////////////////////////////////////////////////
// 액세서빌리티가 숨겨져 있는지 확인합니다.
////////////////////////////////////////////////////////////////////

function IsHideAccessible(accObject: IAccessible): Boolean; overload;
var
  hr: HRESULT;
  vState, vChild: OleVariant;
  vRole: OleVariant;
  iState: Integer;
  iRole: Integer;
begin
  Result := False;
  if accObject = nil then
    Exit;

  TVarData(vChild).VType := varInteger;
  TVarData(vChild).VInteger := CHILDID_SELF;

  hr := accObject.Get_accState(vChild, vState);
  if Succeeded(hr) = True then
  begin
    iState := TVarData(vState).VInteger;
    if ((iState and STATE_SYSTEM_INVISIBLE) <> 0)
      or ((iState and STATE_SYSTEM_OFFSCREEN) <> 0) then
    begin
      Result := True;
    end;

  end;

end;

////////////////////////////////////////////////////////////////////
// 액세서빌리티가 숨겨져 있는지 확인합니다.
////////////////////////////////////////////////////////////////////

function IsHideAccessible(accObject: IAccessible; vChild: OleVariant): Boolean;
  overload;
var
  hr: HRESULT;
  vState: OleVariant;
  iState: Integer;
begin
  Result := False;
  if accObject = nil then
    Exit;
  if TVarData(vChild).VType = varEmpty then
    Exit;

  hr := accObject.Get_accState(vChild, vState);
  if Succeeded(hr) = True then
  begin

    iState := TVarData(vState).VInteger;
    if ((iState and STATE_SYSTEM_INVISIBLE) <> 0)
      or ((iState and STATE_SYSTEM_OFFSCREEN) <> 0) then
    begin
      Result := True;
    end;

  end;
end;

procedure AccessibleObjectChilds(Accessible: IAccessible);

  procedure AddObjectRectList(accObject: IAccessible; vChild: OleVariant);
  var
    pxLeft, pxTop, pxWidth, pxHeight: Integer;
    ARect: TRect;
  begin
    if accObject = nil then
      Exit;

    accObject.accLocation(pxLeft, pxTop, pxWidth, pxHeight, vChild);

    ARect := Rect(pxLeft, pxTop, pxLeft + pxWidth, pxTop + pxHeight);

    SetLength(ObjectRectList, High(ObjectRectList) + 2);
    ObjectRectList[High(ObjectRectList)] := ARect;
  end;

var
  i: Integer;

  hr: HRESULT;

  vChilds: array of Variant;
  vChild, vChildSelf: OleVariant;

  iChildCount: Integer;
  iNewChildCount: Integer;

  cObtained: Integer;

  NewAccessible: IAccessible;
begin
  if Accessible = nil then
    Exit;

  hr := Accessible.Get_accChildCount(iChildCount);
  if Failed(hr) = True then
    Exit;

  NewAccessible := nil;

  if iChildCount > 0 then
  begin

    SetLength(vChilds, iChildCount);
    if Succeeded(AccessibleChildren(Accessible, 0, iChildCount, vChilds[0],
      cObtained)) = True then
    begin

      if iChildCount = cObtained then
      begin

        for i := 0 to iChildCount - 1 do
        begin

          vChild := vChilds[i];

          if TVarData(vChild).VType = varDispatch then
          begin
            if
              Succeeded(IDispatch(TVarData(vChild).VDispatch).QueryInterface(IID_IAccessible, NewAccessible)) = True then
            begin

              if NewAccessible = nil then
                continue;

              if IsHideAccessible(NewAccessible) = True then
                continue;

              TVarData(vChildSelf).VType := varInteger;
              TVarData(vChildSelf).VInteger := CHILDID_SELF;

              AddObjectRectList(NewAccessible, vChildSelf);

              //if NewAccessible.accChildCount > 0 then begin
              // AccessibleObjectChilds(NewAccessible);
              //end;
              hr := NewAccessible.Get_accChildCount(iNewChildCount);
              if Failed(hr) = True then
                continue;

              if iNewChildCount > 0 then
                AccessibleObjectChilds(NewAccessible);

            end;
          end
          else if TVarData(vChild).VType = varInteger then
          begin
            if IsHideAccessible(Accessible, vChild) = True then
              Continue;

            AddObjectRectList(Accessible, vChild);
          end;
        end; // for i := 0 to iChildCount - 1 do begin
      end; // if iChildCount = cObtained then begin
    end; // if Succeeded(AccessibleChildren(Acc, 0, iChildCount, vChilds[0], cObtained)) = True then begin

    vChilds := nil;
  end;

end;

procedure AccessibleObjectRectFromWindow(WindowHandle: HWND; lpPoint: TPoint);
var
  Acc: IAccessible;
begin
  ObjectRectList := nil;
  SetLength(ObjectRectList, 1);

  ObjectRectList[High(ObjectRectList)] := Rect(0, 0, 0, 0);

  if Succeeded(AccessibleObjectFromWindow(WindowHandle, OBJID_WINDOW,
    IID_IAccessible, Acc)) = True then
    AccessibleObjectChilds(Acc);
end;

end.
