﻿unit CaptureDriver;

interface
uses Windows, WinSvc, Dialogs, SysUtils;

const
  SVC_DELETE = $00010000;
  SERVICENAME = 'OpenCapture_Driver';

  CAPTURE_DEV_DRV = $00002A7B;

type
  TScrollData = packed record
    dy: Integer;
    rcScroll: TRect;
  end;

type
  TScrollHookTarget = packed record
    hTargetWindow: HWND;
    hSelfProcess: THandle;
    hTargetProcess: THandle;
  end;

function LoadDriver(strDriverPath: string): cardinal;
procedure UnloadDriver;

function SendDataToDriver(hDevice: THandle; dwCode: DWORD; lpInBuffer: Pointer;
  nInBufferSize: DWORD; lpOutBuffer: Pointer; nOutBufferSize: DWORD): Boolean;

implementation
uses WinDriver;

function LoadDriver(strDriverPath: string): cardinal;
var
  hSCManager: SC_HANDLE;
  hService: SC_HANDLE;
  ss: SERVICE_STATUS;
  path: array[0..MAX_PATH - 1] of Char;
  status: cardinal;
  temp: PChar;
begin
  FillMemory(@path[0], MAX_PATH - 1, 0);
  CopyMemory(@path[0], @strDriverPath[1], Length(strDriverPath) *
    sizeof(WCHAR));

  hSCManager := OpenSCManager(nil, nil, SC_MANAGER_CREATE_SERVICE);
  if hSCManager <> 0 then
  begin
    hService := CreateService(hSCManager, SERVICENAME,
      'This is Driver OpenCapture',
      SERVICE_START or SVC_DELETE or SERVICE_STOP, SERVICE_KERNEL_DRIVER,
      SERVICE_DEMAND_START, SERVICE_ERROR_IGNORE, path, nil, nil, nil, nil,
      nil);

    if hService = 0 then
    begin
      status := GetLastError();
      hService := OpenService(hSCManager, SERVICENAME, SERVICE_START or
        SVC_DELETE or SERVICE_STOP);
      ControlService(hService, SERVICE_CONTROL_STOP, ss);
      DeleteService(hService);

    end
    else
    begin
      temp := nil;

      if not StartService(hService, 0, temp) then
      begin
        status := GetLastError();
        hService := OpenService(hSCManager, SERVICENAME, SERVICE_START or
          SVC_DELETE or SERVICE_STOP);
        ControlService(hService, SERVICE_CONTROL_STOP, ss);
        DeleteService(hService);
      end
      else
      begin
        status := 0;
      end;

    end;
  end
  else
  begin
    status := GetLastError();
  end;

  CloseServiceHandle(hService);
  CloseServiceHandle(hSCManager);
  Result := status;
end;

procedure UnloadDriver;
var
  hSCManager: SC_HANDLE;
  hService: SC_HANDLE;
  ss: SERVICE_STATUS;
begin
  hSCManager := OpenSCManager(nil, nil, SC_MANAGER_CREATE_SERVICE);
  hService := OpenService(hSCManager, SERVICENAME, SERVICE_START or SVC_DELETE
    or
    SERVICE_STOP);

  ControlService(hService, SERVICE_CONTROL_STOP, ss);

  DeleteService(hService);

  CloseServiceHandle(hService);
  CloseServiceHandle(hSCManager);
end;

function SendDataToDriver(hDevice: THandle; dwCode: DWORD; lpInBuffer: Pointer;
  nInBufferSize: DWORD; lpOutBuffer: Pointer; nOutBufferSize: DWORD): Boolean;
var
  dwBytesReturned: DWORD;
begin
  {
  #define FILE_READ_ACCESS	0x00000001
  #define FILE_WRITE_ACCESS	0x00000002
  }
  Result := DeviceIoControl(hDevice, CTL_CODE(CAPTURE_DEV_DRV, dwCode, 0, 2)
    , lpInBuffer, nInBufferSize,
    lpOutBuffer, nOutBufferSize, dwBytesReturned, nil);
end;

end.
