﻿unit uCapture;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, Buttons, GR32_Image, ufrmInherited,
  OpenThreadTimer;

type
 TWinInfo = Record
  Rgn: HRGN;
  lpClassName: Array[0..255] of Char;
  FHandle: THandle;
  FScrollBar: Boolean;
end;

  Const dtLine = 0;
        dtRectangle = 1;
        dtEllipse = 2;
        dtRoundRect = 3;
        dtTriangle = 4;

        ctNull = 0;
        ctUserCapture = 1;
        ctWindow = 2;
        ctControl = 3;
        ctIcon = 4;
        ctFixed = 5;
        ctColor = 6;
        ctPolygon = 7;
        ctObject = 8;

 type
  TfrmCapture = class(TfrmNormal)
    pnlViewer: TPanel;
    imgBackBuffer: TImage;
    pnlZoomViewer: TPanel;
    lblExam: TLabel;
    lblMousePos: TLabel;
    lblCaptureType: TLabel;
    pnlWinInfo: TPanel;
    lblWinInfo: TLabel;
    lblSelectSize: TStaticText;
    shpEdgeViewer: TShape;
    imgColorViewer: TImage;
    procedure imgBackBufferMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgBackBufferMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure imgBackBufferMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ZoomImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure imgBackBufferDblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FImgZoom: TImage32;
    FShape1: TShape;
    FDesktopRect: TRect;

    OldHandle: HWND;
    FDown:Boolean;
    First:Boolean;
    // ** 돋보기 위치가 변경되었나 ** //
    tmrZoomPt: Integer;
    // ** 마우스 좌표가 변경되었나 ** //
    tmrPt:TPoint;
    PolyWork:Boolean;
    // ** 스레드 타이머 ** //
    tmrThreadZoom: TOpenThreadTimer;
    tmrThreadFromPoint: TOpenThreadTimer;
    procedure tmrZoomThreadTimer(Sender: TObject);
    procedure tmrFromPointThreadTimer(Sender: TObject);
    // ******************* //
    procedure WMEraseBkGnd(var Message:TWMEraseBkGnd); Message WM_ERASEBKGND;
    procedure EnumWinInfo;
    procedure ResizeForm;
    procedure ResizePolygon;
    procedure ChangeFixedPoint(X,Y: Integer);
    procedure SetWinControlPosition(AWinControl:TWinControl);
    procedure UpdateColorView(X,Y: Integer);
    function CaptureWork:Boolean;
  public
    { Public declarations }
    UserCaptureType: Integer;
    ShapeType: Integer;
    LastRect:TRect;

    AutoScroll:Boolean;

    FixedW,FixedH: Integer;

    OldFixedPt,nFixedPt:TPoint;
    FixedMove:Boolean;

    WindowName:String;
    SelectHandle:THandle;
    OldRgn,LastRgn: HRGN;
    // ** HRGN 자원을 사용후 제대로 제거하기 위한 변수 ** //
    IsDeleteRGN:Boolean;


    // 컬러 캡쳐시 저장할 변수
    SelectColor:TColor;

    procedure CaptureSetup;
  end;

var
  frmCapture: TfrmCapture;
  tmpWinInfo: Array of TWinInfo = nil;
  WinInfo: Array of TWinInfo = nil;
  ObjectRectList: Array of TRect = nil;
  
implementation

uses CaptureEngine, Imm, SHDocVw, OpenInterface, AccessibleObject;

{$R curCapture.res}
{$R *.dfm}

procedure TfrmCapture.CaptureSetup;

 procedure CreatePoly;
 begin
  PolyWork := False;

  PolyRect.Left := Screen.DesktopWidth;
  PolyRect.Top := Screen.DesktopHeight;
  PolyRect.Right := 0;
  PolyRect.Bottom := 0;
 end;

 procedure LoadCaptureCursor;
 begin
  Case UserCaptureType of
   ctUserCapture,ctPolygon,ctColor : Screen.Cursors[UserCaptureType]  :=  LoadCursor(HInstance,'CUR_SHAPE');
   ctWindow..ctFixed, ctObject : Screen.Cursors[UserCaptureType]  :=  LoadCursor(HInstance,'CUR_SELECT');
  end;

  imgBackBuffer.Cursor := UserCaptureType;

  if (AutoScroll = True) and
     (UserCaptureType = ctControl) then
       Screen.Cursors[8]  :=  LoadCursor(HInstance,'CUR_SCROLL');
 end;

 procedure INIZoomViewer;
 begin
  tmrPt := Point(-1,-1);
  tmrZoomPt := 0;

  tmrThreadZoom := TOpenThreadTimer.Create;
  tmrThreadZoom.OnThreadTimer := tmrZoomThreadTimer;
  tmrThreadZoom.Enabled := True;

  with FImgZoom do begin
   Bitmap.SetSize(Width div 3,Height div 3);
   Bitmap.Clear(clWhite);
  end;

  SetFontAntiAlias(lblSelectSize.Font);
  imgBackBuffer.Canvas.Font := lblSelectSize.Font;
  pnlZoomViewer.DoubleBuffered := True;
 end;

begin
 FDesktopRect := Screen.DesktopRect;

 ResizeForm;
 CreatePoly;
 NewPoint := Point(0, 0);
 EndPoint := Point(0, 0);

 imgBackBuffer.Picture.Bitmap.Assign(FCaptureEngine.FBackBuffer);
 with imgBackBuffer.Canvas do begin
   Pen.Color := RGB(76, 76, 165);
   Pen.Mode := pmNotXor;

   Brush.Style := bsClear;
 end;

 First := True;
 if UserCaptureType <> ctFixed then
  pnlViewer.Visible := True;

 case UserCaptureType of
  ctUserCapture : begin
                   INIZoomViewer;
                   lblSelectSize.Visible := True;

                   case ShapeType of
                    stRect : lblCaptureType.Caption := '옵션 : 사각형';
                    stRoundRect : lblCaptureType.Caption := '옵션 : 둥근 사각형';
                    stCircle : lblCaptureType.Caption := '옵션 : 타원형';
                    stTriangle : lblCaptureType.Caption := '옵션 : 삼각형';
                    stPolygon : begin
                                 lblExam.Caption := '더블 클릭은 완료 ,' + #13 + '오른쪽 버튼은 취소';
                                 imgBackBuffer.Canvas.Pen.Style := psDot;
                                 lblCaptureType.Caption := '옵션 : 다각형';
                                 lblSelectSize.Visible := False;
                                end;
                    stPolyLine : lblCaptureType.Caption := '옵션 : 폐곡선';
                   end;
                  end;

  ctColor : begin
             INIZoomViewer;
             with imgColorViewer do begin
              Visible := True;

              with Picture do begin
               Bitmap.Width := imgColorViewer.Width;
               Bitmap.Height := imgColorViewer.Height;
              end;
             end;

             UpdateColorView(Mouse.CursorPos.X,Mouse.CursorPos.Y);
             lblCaptureType.Caption := '옵션 : 컬러 캡쳐';
            end;
  ctWindow,
  ctControl,
  ctIcon,
  ctObject : begin
              if UserCaptureType = ctObject then
               OldHandle := 0;

              IsDeleteRGN := False;
              EnumWinInfo;

              pnlViewer.Width := 255;
              pnlViewer.Height := 125;
              SetWinControlPosition(pnlViewer);
              pnlWinInfo.Visible := True;

              tmrThreadFromPoint := TOpenThreadTimer.Create;
              tmrThreadFromPoint.OnThreadTimer := tmrFromPointThreadTimer;
              tmrThreadFromPoint.Enabled := True;
             end;
  ctFixed : begin
             FixedMove := False;
             imgBackBuffer.CanVas.Pen.Width := 2;
            end;
 end;

 if UserCaptureType <> ctFixed then
  SetWinControlPosition(pnlViewer);

 LoadCaptureCursor;
end;

function TfrmCapture.CaptureWork:Boolean;
begin
   if ShapeType = stPolyline then
    if (PolyRect.Right - PolyRect.Left <= 0) and (PolyRect.Bottom - PolyRect.Top <= 0) then
     Result := False
      else Result := True
    else if (EndPoint.X - NewPoint.X = 0) and (EndPoint.Y - NewPoint.Y = 0) then
          Result := False
           else Result := True;

end;

procedure TfrmCapture.UpdateColorView(X,Y: Integer);
var
AColor:DWord;
begin

 with imgColorViewer do begin
  Left := X + 20;
  Top := Y + 20;
  if Left + Width > FDesktopRect.Right then
   Left := X - 20 - Width;
  if Top + Height > FDesktopRect.Bottom then
    Top := Y - 20 - Height;
  if SelectColor <> imgBackBuffer.Canvas.Pixels[X,Y] then
   with Canvas do begin
    Brush.Style := bsSolid;
    Brush.Color := clWhite;
    Rectangle(0,0,Width,Height);

    Brush.Color := imgBackBuffer.Canvas.Pixels[X,Y];
    SelectColor := Brush.Color;
    AColor := ColorToRGB(SelectColor);
    Rectangle(7,17,22,32);
    Brush.Style := bsClear;

    Font.Name := 'Arial';
    Font.Size := 9;
    Font.Style := [fsBold];
    Font.Charset := ANSI_CHARSET;
    SetFontAntiAlias(Font);
    TextOut(28,5,Format('RGB(%d,%d,%d)',[GetRValue(AColor),GetGValue(AColor),GetBValue(AColor)]));
    TextOut(28,18,Format('Web Color : #%.2x%.2x%.2x',[GetRValue(AColor), GetGValue(AColor), GetBValue(AColor)]));
    TextOut(28,31,Format('Delphi Color : %s', [ColorToString(SelectColor)]));
   end;

 end;

end;

procedure TfrmCapture.SetWinControlPosition(AWinControl: TWinControl);
const Margin = 300;
var
 CursorPos:TPoint;
 ScreenRect: TRect;
begin
 GetCursorPos(CursorPos);
 ScreenRect := Screen.DesktopRect;

 if (CursorPos.X < ScreenRect.Left + Margin) and (CursorPos.Y < ScreenRect.Top + Margin)
     and (AWinControl.Left = 0) then
  with AWinControl do begin
   Visible := False;
   Left := ScreenRect.Right - Width;
   Top := ScreenRect.Bottom - Height;
   Visible := True;
   Exit;
  end;

 if (CursorPos.X > (ScreenRect.Right - Margin)) and
    (CursorPos.Y > (ScreenRect.Bottom - Margin))
    and (AWinControl.Left > 0) then
     with AWinControl do begin
      Visible := False;
      Left := 0;
      Top := 0;
      Visible := True;
     end;

end;

procedure TfrmCapture.ChangeFixedPoint(X, Y: Integer);
var
 DesktopWidth: Integer;
 DesktopHeight: Integer;
begin;
  DesktopWidth := Screen.DesktopWidth;
  DesktopHeight := Screen.DesktopHeight;

  NewPoint.X := X - Round(FixedW / 2);
  NewPoint.Y := Y - Round(FixedH / 2);
  EndPoint.X := X + Round(FixedW / 2);
  EndPoint.Y := Y + Round(FixedH / 2);

  if NewPoint.X < 0 then begin
   NewPoint.X := 0;
   EndPoint.X := FixedW;
  end;
  if NewPoint.Y < 0 then begin
   NewPoint.Y := 0;
   EndPoint.Y := FixedH;
  end;
  if EndPoint.Y > DesktopHeight then begin
   NewPoint.Y := DesktopHeight - FixedH;
   EndPoint.Y := DesktopHeight;
  end;
  if EndPoint.X > DesktopWidth then begin
   EndPoint.X := DesktopWidth;
   NewPoint.X := DesktopWidth - FixedW;
  end;

end;

procedure TfrmCapture.ResizePolygon;
begin
 // ** 다각형 캡쳐 시 이미지의 크기를 설정하는 루틴 ** //
 if NewPoint.X < PolyRect.Left then
  PolyRect.Left := NewPoint.X;

 if NewPoint.Y < PolyRect.Top then
  PolyRect.Top := NewPoint.Y;

 if NewPoint.X > PolyRect.Right then
  PolyRect.Right := NewPoint.X;

 if NewPoint.Y > PolyRect.Bottom then
  PolyRect.Bottom := NewPoint.Y;
 // ************************************************** //

 // **  - 1 이라면 변수가 nil 인 것이다. ** //
  if High(PolygonPt) = - 1 then
   SetLength(PolygonPt, 1)
  else
   SetLength(PolygonPt, High(PolygonPt) + 2);

 PolygonPt[High(PolygonPt)].X := NewPoint.X;
 PolygonPt[High(PolygonPt)].Y := NewPoint.Y;
end;

procedure TfrmCapture.ResizeForm;
var
 DC: HDC;
 ARect: TRect;
begin
 DC := GetDC(0);

 GetClipBox(DC, ARect);
 MoveWindow(Self.Handle, ARect.left, ARect.Top, ARect.Right - ARect.Left, ARect.Bottom - ARect.Top, True);

 ReleaseDC(0, DC);
end;

// ** 윈도우의 목록을 가져온다.. ** //
// ...... 여기서부터 .......... //

procedure AddWinInfo;
begin
 // **  - 1 이라면 변수가 nil 인 것이다. ** //
 if High(WinInfo) =  - 1 then
  SetLength(WinInfo,1)
   else SetLength(WinInfo,High(WinInfo) + 2);
end;

procedure AddtmpWinInfo;
begin
 // **  - 1 이라면 변수가 nil 인 것이다. ** //
 if High(tmpWinInfo) =  - 1 then
  SetLength(tmpWinInfo,1)
   else SetLength(tmpWinInfo,High(tmpWinInfo) + 2);
end;

function EnumWindowsProc(Handle:Hwnd;lParam:lParam): BOOL; stdcall;
begin
  Result  :=  True;
  if not IsWindowVisible(Handle) then
   Exit;
  if IsIconic(Handle) then
   Exit;

  AddtmpWinInfo;

  with tmpWinInfo[High(tmpWinInfo)] do begin
  
   GetWindowRealRgn(Handle, Rgn, True);
   GetClassName(Handle,lpClassName,SizeOf(lpClassName));
   FHandle := Handle;

   if frmCapture.UserCaptureType = ctControl then
    if Boolean(lParam) = True then
     FScrollBar := IsScrollBar(Handle)
      else FScrollbar := False;

  end;

end;

procedure TfrmCapture.EnumWinInfo;
var
i: Integer;
begin

 EnumWindows(@EnumWindowsProc,Integer(AutoScroll));


 for i := High(tmpWinInfo) Downto Low(tmpWinInfo) do begin

  AddWinInfo;
  WinInfo[High(WinInfo)] := tmpWinInfo[i];

 end;

 tmpWinInfo := nil;

end;

procedure TfrmCapture.WMEraseBkGnd(var Message:TWMEraseBkGnd);
begin
  if First = True then with FCaptureEngine.FBackBuffer do begin
    BitBlt(Message.DC, 0, 0, Width, Height, Canvas.Handle, 0, 0, SRCCOPY);

    // ** 캡쳐 작업 영역이 뒤로 가는 수가 있다. ** //
    // ** 예 : 프로그램 실행  -  > 곧바로 내 컴퓨터 실행  -  > 캡쳐 진행 ** //
    SetForegroundWindow(Application.Handle);
  end;

  // ** IME 창을 잠시 사라지게 한다 ** //
  DestroyWindow(ImmGetDefaultIMEWnd(Application.Handle));
  First := false;
end;

procedure TfrmCapture.imgBackBufferMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

  function Drawing:Boolean;
  begin
   Result := False;
   if UserCaptureType = ctUserCapture then
    if (FDown = True) and (ShapeType <> stPolygon) then
     Result := True;
  end;

begin

  if (Button in [mbLeft]) and (UserCaptureType <> ctFixed)
     // ** [ 엔터  -  > 드래그  -  > 엔터  -  > 마우스 다운 ] 버그 수정 ** //
     and (Drawing = False) then begin

   FDown := True;
   NewPoint := Point(X,Y);
   EndPoint := NewPoint;

   if ShapeType = stPolygon then begin

    if PolyWork = True then
     if EqualPoint(PolygonPt[High(PolygonPt)],Point(X,Y)) = True then
      Exit;

    PolyWork := True;
    ResizePolygon;
   end;

    if (UserCaptureType = ctUserCapture) and (ShapeType <> stPolygon) and (ShapeType <> stPolyline) then
    begin
      with imgBackBuffer.Canvas do
      begin
       //캡쳐 화면에서 이전 위치의 십자선을 지운다. (XOR 연산의 특성을 이용)
       MoveTo(tmrPt.X, 0);
       LineTo(tmrPt.X, Screen.DesktopHeight);
       MoveTo(0, tmrPt.Y);
       LineTo(Screen.DesktopWidth, tmrPt.Y);
      end;
    end;
  end;
end;

procedure TfrmCapture.imgBackBufferMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);

 procedure MoveLabel(X,Y: Integer;DrawPoint:Boolean);
 var
  OldPt, nPt: TPoint;
  OldLeft, OldTop: Integer;
  ScrWidth, ScrHeight: Integer;
  CaptureWidth, CaptureHeight: Integer;
 begin
  with lblSelectSize do begin

   if ShapeType <> stPolyline then begin
    OldPt := NewPoint;
    nPt := EndPoint;
    end else begin
              OldPt := Point(PolyRect.Left, PolyRect.Top);
              nPt := Point(PolyRect.Right, PolyRect.Bottom);
             end;

   NormalizePoint(OldPt, nPt);

   ScrWidth := Screen.DesktopWidth;
   ScrHeight := Screen.DesktopHeight;

   OldLeft := X + 20;
   OldTop := Y + 20;

   if OldLeft + Width * 2 > ScrWidth then
    OldLeft := ScrWidth - (Width * 2);
   if OldTop + Height * 2 > ScrHeight then
    OldTop := ScrHeight - (Height * 5);

   Left := OldLeft;
   Top := OldTop;

   if (DrawPoint = False) or ( (High(PolygonPt) =  - 1) and (ShapeType = stPolyline)) then begin
    nPt := Point( - 1, - 1);
    OldPt := Point(0,0);
   end;

   CaptureWidth := (nPt.X - OldPt.X) + 1;
   CaptureHeight := (nPt.Y - OldPt.Y) + 1;

   Width := imgBackBuffer.Canvas.TextWidth(IntToStr(CaptureWidth) + ' x '
    + IntToStr(CaptureHeight)) + 1;

   Caption := IntToStr(CaptureWidth) + ' x '
    + IntToStr(CaptureHeight);

  end;

 end;


 procedure DrawShape(flags: Integer);
 var
 OldPt,nPt:TPoint;
 begin
  Case flags of
  dtLine : with imgBackBuffer.Canvas do begin
            MoveTo(NewPoint.X, NewPoint.Y);
            LineTo(EndPoint.X, EndPoint.Y);
            MoveTo(NewPoint.X, NewPoint.Y);
            LineTo(X, Y);
           end;
  dtRectangle : with imgBackBuffer.Canvas do begin

                 if UserCaptureType = ctFixed then begin
                  if FixedMove = False then
                    Rectangle(NewPoint.X, NewPoint.Y, EndPoint.X + 1, EndPoint.Y + 1)
                   else begin
                    Rectangle(OldFixedPt.X, OldFixedPt.Y, nFixedPt.X + 1,nFixedPt.Y + 1);
                    Rectangle(NewPoint.X, NewPoint.Y, EndPoint.X + 1, EndPoint.Y + 1);
                   end;

                 end else begin
                  OldPt := NewPoint;
                  nPt := EndPoint;

                  NormalizePoint(OldPt, nPt);

                  imgBackBuffer.Picture.Bitmap.Canvas.Rectangle(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1);

                  OldPt := NewPoint;
                  nPt := Point(X, Y);
                  NormalizePoint(OldPt, nPt);

                  imgBackBuffer.Picture.Bitmap.Canvas.Rectangle(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1);
                 end;
                end;
  dtEllipse : with imgBackBuffer.Canvas do begin
               OldPt := NewPoint;
               nPt := EndPoint;
               NormalizePoint(OldPt, nPt);

               Ellipse(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1);

               OldPt := NewPoint;
               nPt := Point(X, Y);
               NormalizePoint(OldPt, nPt);

               Ellipse(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1);
              end;
  dtRoundRect : with imgBackBuffer.Canvas do begin
                 OldPt := NewPoint;
                 nPt := EndPoint;
                 NormalizePoint(OldPt, nPt);

                 RoundRect(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1, (OldPt.X - (nPt.X + 1)) div 2, (OldPt.Y - (nPt.Y + 1)) div 2);

                 OldPt := NewPoint;
                 nPt := Point(X, Y);
                 NormalizePoint(OldPt, nPt);

                 RoundRect(OldPt.X, OldPt.Y, nPt.X + 1, nPt.Y + 1, (OldPt.X - (nPt.X + 1)) div 2, (OldPt.Y - (nPt.Y + 1)) div 2);
                end;
  dtTriangle : with imgBackBuffer do begin
                DrawTriangle(Canvas, NewPoint.X, NewPoint.Y, EndPoint.X, EndPoint.Y);
                DrawTriangle(Canvas, NewPoint.X, NewPoint.Y, X, Y);
               end;
  end;
 end;

begin
  if UserCaptureType <> ctFixed then
   SetWinControlPosition(pnlViewer);

  if UserCaptureType in [ctUserCapture,ctPolygon,ctColor] then
   lblMousePos.Caption := '좌표 :' + IntToStr(X) + ' , ' + IntToStr(Y);

  Case UserCaptureType of
  ctUserCapture : begin
                   if FDown then begin

                    case ShapeType of
                     stRect : DrawShape(dtRectangle);
                     stRoundRect : DrawShape(dtRoundRect);
                     stCircle : DrawShape(dtEllipse);
                     stTriangle : DrawShape(dtTriangle);
                     stPolygon : DrawShape(dtLine);
                     stPolyline : begin
                                   imgBackBuffer.Canvas.MoveTo(EndPoint.X, EndPoint.Y);
                                   imgBackBuffer.Canvas.LineTo(X, Y);
                                   if EqualPoint(NewPoint, Point(X, Y)) = False then
                                    ResizePolygon;

                                   NewPoint := Point(X, Y);
                                   EndPoint := Point(X, Y);

                                   MoveLabel(X, Y, True);

                                   Exit;
                                  end;
                    end;

                    EndPoint := Point(X, Y);

                   end;
                   MoveLabel(X, Y, not EqualPoint(NewPoint, EndPoint));

                  end;

  ctFixed : begin
             ChangeFixedPoint(X, Y);

             DrawShape(dtRectangle);

             OldFixedPt := NewPoint;
             nFixedPt := EndPoint;
             FixedMove := True;
            end;
  ctColor : begin
             UpdateColorView(X,Y);
            end;
  end; // Case UsercaptureType of

end;


procedure TfrmCapture.imgBackBufferMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

  function SetFileName(Const FileName:String):String;
  Const
   dName:Array[0..7] of Char = ('\','/',':','*','"','<','>','|');
   var
    i: Integer;
   begin
    Result := FileName;

     for i := 0 to 7 do
      Result := StringReplace(Result,dName[i],'',[rfreplaceAll,rfIgnoreCase]);

    Result := StringReplace(Result,#13 + #10,' ',[rfreplaceAll,rfIgnoreCase]);

   end;

  var
   WindowText: Array[0..255] of Char;
  begin

  case UserCaptureType of
  ctUserCapture : begin
                   // ******************** //
                   case ShapeType of

                    // ******************** //
                    stPolygon : case Button of
                                  mbLeft : begin
                                            if Low(PolygonPt) < High(PolygonPt) then
                                             if EqualPoint(PolygonPt[High(PolygonPt)],PolygonPt[High(PolygonPt) - 1]) = True then begin
                                              FDown := False;
                                              PolyWork := False;
                                              ModalResult  :=  mrOk;
                                             end;
                                            end;
                                  mbRight : begin
                                             FDown := False;
                                             PolyWork := False;
                                             ModalResult := mrCancel;
                                            end;
                                 end;
                    // ******************** // case Button of
                                 else begin
                                       // ** NewPoint 와 EndPoint 는 폴리곤캡쳐 시 이미지의 크기를 ** //
                                       // **저장하는 역할도 함으로 ** //
                                       if Button in [mbLeft] then begin

                                        //if (EndPoint.X - NewPoint.X = 0) and (EndPoint.Y - NewPoint.Y = 0) and (ShapeType <> stPolyline) then
                                        // Exit;
                                        //if (PolyRect.Right - PolyRect.Left < 0) and (PolyRect.Bottom - PolyRect.Top < 0) and (ShapeType = stPolyline) then
                                        // Exit;

                                        if CaptureWork = True then
                                         ModalResult  :=  mrOK;
                                       end else
                                        ModalResult := mrCancel;
                                      end;
                     end; // ******************** // ShapeType of

                  end;  // ******************** // ctUserCapture : begin


  ctWindow, ctControl, ctIcon, ctObject : begin
                              tmrThreadFromPoint.Free;

                              tmrFromPointThreadTimer(Sender);

                              if Button in [mbLeft] then begin
                                 WindowName := '';
                                 if Shift = [ssCtrl] then begin

                                    GetWindowText(SelectHandle, WindowText, SizeOf(WindowText) div SizeOf(WindowText));

                                     if not (WindowText='') then begin
                                      WindowName := WindowText;

                                      if Length(WindowName) > 35 then
                                       SetLength(WindowName, 35);

                                      WindowName := SetFileName(WindowName);

                                     end;
                                 end;

                               ModalResult := mrOK;

                                end else ModalResult  :=  mrCancel;
                             end;

  ctFixed : if Button in [mbLeft] then
             ModalResult := mrOK
              else ModalResult := mrCancel;
  ctColor : begin
             if Button in [mbLeft] then begin
              SelectColor := FCaptureEngine.FBackBuffer.Canvas.Pixels[X,Y];
              ModalResult  :=  mrOK;
              end else ModalResult  :=  mrCancel;
            end;

  end;

end;

procedure TfrmCapture.ZoomImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
 if (pnlViewer.Left = FDesktopRect.Left) and (pnlViewer.Top = FDesktopRect.Top) then with pnlViewer do begin
  Visible := false;
  Left := FDesktopRect.Right - Width;
  Top := FDesktopRect.Bottom - Height;
  Visible := True;
  end else with pnlViewer do begin
            Visible := false;
            Left := FDesktopRect.Left;
            Top := FDesktopRect.Top;
            Visible := True;
           end;

end;

procedure TfrmCapture.FormDestroy(Sender: TObject);
var
 i: Integer;
begin
Case UserCaptureType of
 ctWindow, ctControl : begin
                        WinInfo := nil;
                        tmpWinInfo := nil;
                        DeleteObject(OldRgn);
                        DeleteObject(LastRgn);
                        for i := High(WinInfo) downto Low(WinInfo) do
                         DeleteObject(WinInfo[i].Rgn);
                       end;

 end;

 tmrThreadZoom.Free;
 // ** 없으면 에러 남 ** //
 //tmrZoom.Enabled := False;

 frmCapture := nil;

 UserCaptureType := ctNull;
 FDown := False;

 FShape1.Free;
 FImgZoom.Free;
end;

procedure TfrmCapture.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
 lpPoint:TPoint;
begin

 if Key = VK_ESCAPE then
  ModalResult := mrCancel;

 GetCursorPos(lpPoint);

 if GetAsyncKeyState(VK_LEFT) < 0 then
  SetCursorPos(lpPoint.X - 1,lpPoint.Y);
 if GetAsyncKeyState(VK_RIGHT) < 0 then
  SetCursorPos(lpPoint.X + 1,lpPoint.Y);
 if GetAsyncKeyState(VK_UP) < 0 then
  SetCursorPos(lpPoint.X,lpPoint.Y - 1);
 if GetAsyncKeyState(VK_DOWN) < 0 then
  SetCursorPos(lpPoint.X,lpPoint.Y + 1);

end;

procedure TfrmCapture.imgBackBufferDblClick(Sender: TObject);
begin
 if ShapeType = stPolygon then begin
  if High(PolygonPt) = 0 then
   Exit;
  FDown := False;
  PolyWork := False;
  ModalResult  :=  mrOk;
 end;
end;

procedure TfrmCapture.FormCreate(Sender: TObject);
begin
 FImgZoom := TImage32.Create(Self);
 FShape1 := TShape.Create(Self);

 FImgZoom.Name := 'imgZoom';
 FShape1.Name := 'Shape1';

 FImgZoom.SetBounds(4, 4, 151, 151);

 FImgZoom.ScaleMode := smStretch;

 FImgZoom.Parent := pnlZoomViewer;
 FShape1.Parent := FImgZoom;
 FShape1.Align := alClient;

 FShape1.Brush.Style := bsClear;

 FShape1.Pen.Color := clGray;
 FShape1.Pen.Style := psDot;

 frmCapture := Self;
end;

procedure TfrmCapture.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
 lpPoint:TPoint;
begin
 GetCursorPos(lpPoint);

 if Key = VK_RETURN then
  with imgBackBuffer do
   if UserCaptureType = ctUserCapture then begin
    if FDown = False then
     OnMouseDown(Sender,mbLeft,[],lpPoint.X,lpPoint.Y)
      else OnMouseUp(Sender,mbLeft,[],lpPoint.X,lpPoint.Y)
   end else OnMouseUp(Sender,mbLeft,[],lpPoint.X,lpPoint.Y);

end;

procedure TfrmCapture.tmrZoomThreadTimer(Sender: TObject);
Const
 CenterPoint = 25;
var
 MousePoint: TPoint;
 BitmapWidth, BitmapHeight: Integer;

 PolyPoint: Array [0..1] of TPoint;
begin
 MousePoint := DesktopCoordinateToOriginCoordinate(Mouse.CursorPos);

 if EqualPoint(MousePoint, tmrPt) = False then begin

  BitmapWidth := (MousePoint.X + 24) - (MousePoint.X - 25);
  BitmapHeight := BitmapWidth;

  with FImgZoom do begin
   BeginUpdate;

   Bitmap.Clear(clWhite);

   if (UserCaptureType = ctUserCapture) and (ShapeType <> stPolygon) and (ShapeType <> stPolyline) then
   begin
    if FDown = False then
    with imgBackBuffer.Canvas do
    begin
     if (tmrPt.X <> -1) and (tmrPt.Y <> -1) then begin
      //캡쳐 화면에서 이전 위치의 십자선을 지운다. (XOR 연산의 특성을 이용)
      MoveTo(tmrPt.X, 0);
      LineTo(tmrPt.X, Screen.DesktopHeight);
      MoveTo(0, tmrPt.Y);
      LineTo(Screen.DesktopWidth, tmrPt.Y);
     end;

     //캡쳐 화면에 십자선을 그린다.  (현재 PenMode = pmNotXor)
     MoveTo(MousePoint.X, 0);
     LineTo(MousePoint.X, Screen.DesktopHeight);
     MoveTo(0, MousePoint.Y);
     LineTo(Screen.DesktopWidth, MousePoint.Y);
    end;
   end;

   BitBlt(Bitmap.Canvas.Handle, 0, 0, BitmapWidth, BitmapHeight, imgBackBuffer.Canvas.Handle,
          MousePoint.X - 25, MousePoint.Y - 25,SRCCOPY);

   if (ShapeType = stPolygon) or (ShapeType = stPolyline) or (UserCaptureType = ctColor) then
   begin
     Bitmap.Canvas.Pen.Color := clBlack;
     Bitmap.Canvas.Pen.Mode := pmNotXor;

     PolyPoint[0].X := CenterPoint - 20;
     PolyPoint[0].Y := CenterPoint;
     PolyPoint[1].X := CenterPoint;
     PolyPoint[1].Y := CenterPoint;

     Bitmap.Canvas.Polyline(PolyPoint);     //Zoomviewer 에 십자선을 그려준다.

     PolyPoint[0].X := CenterPoint;
     PolyPoint[0].Y := CenterPoint - 20;
     PolyPoint[1].X := CenterPoint;
     PolyPoint[1].Y := CenterPoint;

     Bitmap.Canvas.Polyline(PolyPoint);

     PolyPoint[0].X := CenterPoint + 1;
     PolyPoint[0].Y := CenterPoint;
     PolyPoint[1].X := BitmapWidth - 5;
     PolyPoint[1].Y := CenterPoint;

     Bitmap.Canvas.Polyline(PolyPoint);

     PolyPoint[0].X := CenterPoint;
     PolyPoint[0].Y := CenterPoint + 1;
     PolyPoint[1].X := CenterPoint;
     PolyPoint[1].Y := BitmapHeight - 5;

     Bitmap.Canvas.Polyline(PolyPoint);
   end;

   EndUpdate;

   Repaint;

  end;


  if tmrZoomPt <> pnlViewer.Left then
   pnlZoomViewer.Repaint;

  tmrZoomPt := pnlViewer.Left;
  tmrPt := MousePoint;
 end;

end;

procedure TfrmCapture.tmrFromPointThreadTimer(Sender: TObject);
var
 i, j: Integer;
 WindowText: Array[0..255] of Char;
 rcClient: TRect;
 FWinInfo: TWinInfo;
 ChildHandle: THandle;
 lpPoint: TPoint;
 FCreateChildRgn: Boolean;
begin
   GetCursorPos(lpPoint);

   FCreateChildRgn := False;

   imgBackBuffer.Canvas.Brush.Style := bsSolid;
   imgBackBuffer.Canvas.Brush.Color := RGB(76, 76, 165);

   for i := High(WinInfo) downto Low(WinInfo) do begin

    FWinInfo := WinInfo[i];

    if PtInRegion(WinInfo[i].Rgn, lpPoint.X, lpPoint.Y) = True then begin

      if UserCaptureType in [ctControl, ctObject] then begin
       ChildHandle := MyChildWindowFromPoint(WinInfo[i].FHandle, lpPoint);

       if ChildHandle <> 0 then
        with FWinInfo do begin

         FHandle := ChildHandle;
         //DeleteObject(Rgn);
         // ** DeleteObject(Rgn); 를 하면 Rgn 이 사라지므로..
         // 새로운 Rgn 을 만든다 ** //
         Rgn := 0;
         GetChildWindowRealRgn(WinInfo[i].FHandle, FHandle, Rgn, True);

         if UserCaptureType = ctObject then begin
          if OldHandle <> ChildHandle then begin
            AccessibleObjectRectFromWindow(ChildHandle, lpPoint);
            OldHandle := ChildHandle;
           end;

          //GetWindowRect(ChildHandle, lpRect);
          for j := High(ObjectRectList) downto 0 do
           //if EqualRect(lpRect, ObjectRectList[j]) = False then
             if PtInRect(ObjectRectList[j], lpPoint) = True then begin
              DeleteObject(Rgn);

              SetRectInParent(ChildHandle, ObjectRectList[j]);

              Rgn := CreateRectRgnIndirect(ObjectRectList[j]);
              Break;
          end;
         end; // if UserCaptureType = ctObject then begin

         FCreateChildRgn := True;

         GetClassName(FHandle,lpClassName,SizeOf(lpClassName));

         FScrollBar := IsScrollBar(FHandle) or IsScrollnBar(WinInfo[i].FHandle, FHandle, SB_CTL);
        end; // with FWinInfo do begin
      end; // if UserCaptureType in [ctControl, ctObject] then begin

     LastRgn := FWinInfo.Rgn;
     GetRgnBox(LastRgn, LastRect);
     SelectHandle := FWinInfo.FHandle;
     GetWindowText(FWinInfo.FHandle, WindowText, SizeOf(WindowText) div SizeOf(Char));
     lblWinInfo.Caption := '텍스트: "' + WindowText + '"'
                          + #13 + '사각형 [' + IntToStr(LastRect.Left) + ',' + IntToStr(LastRect.Top) + ',' + IntToStr(LastRect.Right) + ',' + IntToStr(LastRect.Bottom) + ']'
                          + #13 + '핸들: ' + IntToStr(FWinInfo.FHandle)
                          + #13 + '클래스 이름: ' + FWinInfo.lpClassName
                          + #13 + #13 + '왼쪽버튼은 선택 ,'
                          + #13 + '오른쪽버튼 또는 ESC 는 취소'
                          + #13 + #13 + 'Ctrl + 왼쪽버튼 클릭 시 텍스트로 이름지정';

     if UserCaptureType = ctObject then
      Break;

     if (UserCaptureType = ctControl) and (AutoScroll = True) then begin
          GetWorkRect(FWinInfo.FHandle,rcClient);
          if (PtInRect(rcClient,lpPoint) = True) and
             // ** 기본 윈도우 스크롤 바.. ** //
             ((FWinInfo.FScrollBar = True)
             // ** 웹 스크롤 바 ** //
             or (IsWebScrollBar(FWinInfo.FHandle) = True)
             // ** 오피스 스크롤 바 ** //
             or (IsOfficeScrollBar(FWinInfo.FHandle) = True)) then begin
           LastRect := rcClient;

           if FCreateChildRgn = True then
            DeleteObject(LastRgn);
           LastRgn := 0;
           LastRgn := CreateRectRgnIndirect(rcClient);

           imgBackBuffer.Cursor := 8
            end else imgBackBuffer.Cursor := 3; // if PtInRect(WindowInfo.rcClient,Mouse.CursorPos) then begin
      // Add ..if (UserCaptureType = ctControl) and (frmM ......
     end;
     break;
    end;
    // Add ..for i := High(WinInfo) downto Low(WinInfo) do begin
   end;

   if EqualRgn(OldRgn, LastRgn) = True then
    Exit
   else begin
    DesktopCoordinateToOriginCoordinate(OldRgn);
    FrameRgn(imgBackBuffer.Canvas.Handle, OldRgn, imgBackBuffer.Canvas.Brush.Handle, 3, 3);
    OriginCoordinateToDesktopCoordinate(OldRgn);
   end;

   DesktopCoordinateToOriginCoordinate(LastRgn);
   FrameRgn(imgBackBuffer.Canvas.Handle, LastRgn, imgBackBuffer.Canvas.Brush.Handle, 3, 3);
   OriginCoordinateToDesktopCoordinate(LastRgn);

   imgBackBuffer.Refresh;

   // ** 이전의 차일드 윈도우가 있어서 Rgn 을 새로 생성 한 경우..
   // 삭제해준다 ** //
   if IsDeleteRGN = True then begin
    DeleteObject(OldRgn);
    IsDeleteRgn := False;
   end;

   OldRgn := LastRgn;

   if FCreateChildRgn = True then
    IsDeleteRGN := True;

end;

end.
