unit HSLUnit;

interface

uses
 Windows, Graphics;

type
 THSLColor = record
  H : 0..239;
  S : 0..240;
  L : 0..240;
end;

function ColorToHSL(Const Color:TColor):THSLColor;
function HSLToColor(HSL:THSLColor):TColor;
function HSL(Const H,S,L:Integer):THSLColor;

implementation

function HSL(Const H,S,L:Integer):THSLColor;
begin
if H > 239 then
   Result.H:= 239 else
   Result.H:= H;
if S > 240 then
   Result.S:= 240 else
   Result.S:= S;
if L > 240 then
   Result.L:= 240 else
   Result.L:= L;
end;

Procedure HSLtoRGB (Const H,S,L: Double;Var R,G,B:Byte);
var
 M1,M2: double;
 function HueToColourValue (Hue: Double) : byte;
 var
   V : double;
 begin
   if Hue < 0 then
      Hue := Hue + 1 else
      if Hue > 1 then Hue := Hue - 1;
   if 6 * Hue < 1 then
      V := M1 + (M2 - M1) * Hue * 6 else
      if 2 * Hue < 1 then
         V := M2 else
         if 3 * Hue < 2 then
            V := M1 + (M2 - M1) * (2/3 - Hue) * 6 else
            V := M1;
   Result := Round (255 * V)
 end;
begin
 if S = 0 then
 begin
   R := Round (255 * L);
   G := R;
   B := R
 end else begin
   if L <= 0.5 then
    M2 := L * (1 + S) else
    M2 := L + S - L * S;
   M1 := 2 * L - M2;
   R := HueToColourValue (H + 1/3);
   G := HueToColourValue (H);
   B := HueToColourValue (H - 1/3)
 end;
end;


procedure RGBtoHSL (Const R,G,B:Double;Var H,S,L:Double);
 function MaxMin (Const A,B:Double;Const C:Boolean=True):Double;
 begin
  if C then
   if A > B then
      Result := A else
      Result := B
  else
   if A < B then
      Result := A else
      Result := B;
  end;
var
 D,MX,MN: Double;
begin
 MX := MaxMin (R, MaxMin (G, B));
 MN := MaxMin (R, MaxMin (G, B, False), False);
 L := (MX + MN) / 2;
 if MX = MN then
 begin
   H := 0;
   S := 0;
 end else begin
   D := MX - MN;
   if L < 0.5 then S := D / (MX + MN)
   else S := D / (2 - MX - MN);
   if R=MX then H := (G-B)/D
   else
    if G=MX then H := 2+(B-R)/D
    else H := 4+(R-G)/D;
   H := H/6;
   if H < 0 then H := H+1;
 end
end;

function HSLToColor(HSL:THSLColor):TColor;
var
R,G,B : Byte;
begin
 HSLToRGB(HSL.H/239,HSL.S/240,HSL.L/240,R,G,B);
 Result:=RGB(R,G,B);
end;

function ColorToHSL(Const Color:TColor):THSLColor;
var
 HD,SD,LD:Double;
begin
 RGBtoHSL (GetRValue(ColorToRGB(Color))/255,
           GetGValue(ColorToRGB(Color))/255,
           GetBValue(ColorToRGB(Color))/255,HD,SD,LD);

 with Result do begin
  H := Round(HD * 239);
  S := Round(SD * 240);
  L := Round(LD * 240);
  if H > 239 then H := 239;
  if S > 240 then S := 240;
  if L > 240 then L := 240;
 end;
end;

end.
 