﻿unit ColorPicker;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, StdCtrls, ExtCtrls, AppEvnts, GR32_RangeBars;

Const
 CM_ColorChange = WM_USER + $1005;
 CM_NotColorChange = WM_USER + $1006;
 COLOR_Frame = $007A8689;
 COLOR_Border = $007A8689;
 COLOR_Bk = $00EEF4F3;

 ConstColor:Array[0..39] of TColor =
 (clRed, clYellow, $0000FF80, $0040FF00, clAqua, $00C08000, $00C08080, clFuchsia
 , clMaroon, $000080FF, clGreen, $00408000, clBlue, $00A00000, clPurple, $00FF0080
 , clBlack, clOlive, $00408080, clGray, $00808040, clSilver, $00400040, clWhite
 , clWhite, clWhite, clWhite, clWhite, clWhite, clWhite, clWhite, clWhite
 , clWhite, clWhite, clWhite, clWhite, clWhite, clWhite, clWhite, clWhite);

type
  TfrmOpenColorPicker = class(TForm)
    ColorDialog: TColorDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, 
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X,  Y: Integer);
    procedure FormDeactivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    ColorRect:Array[0..39] of TRect;
    OldRect: TRect;
    CaptureButtonRect: TRect;
    CustomButtonRect: TRect;
    ConvertButtonRect: TRect;
    FDestory: Boolean;
    procedure ColorChange;
  public
    { Public declarations }
    AColor:Array[0..39] of TColor;
    SelectColor: TColor;
    SelectRect: TRect;
    ParentHandle: THandle;
    procedure AddColorRect;
  end;

implementation

 uses CaptureEngine, OpenPalette, OpenDrawFrame, ColorConvert;

{$R *.dfm}

procedure TfrmOpenColorPicker.AddColorRect;
var
 i:Integer;
 ChangeColor:Boolean;
begin
 ChangeColor := False;

 ColorRect[0] := Rect(7,  12,  7+12,  12+12);
 ColorRect[8] := Rect(7,  32,  7+12,  32+12);
 ColorRect[16] := Rect(7,  52,  7+12,  52+12);
 ColorRect[24] := Rect(7,  75,  7+12,  75+12);
 ColorRect[32] := Rect(7,  95,  7+12,  95+12);

 if SelectColor = AColor[0] then begin
  SelectRect := ColorRect[0];
  ChangeColor := True;
 end;

 for i := 1 to High(AColor) do begin
  if (ChangeColor = False) and (i mod 8 = 0) then
   if SelectColor = AColor[i] then begin
    SelectRect := ColorRect[i];
    ChangeColor := True;
   end;

  if i mod 8 = 0 then
   Continue;

  with ColorRect[i-1] do
   ColorRect[i] := Rect(Right+7,  Top,  (Right+12)+7,  Top+12);
  if i in [9..15] then
   ColorRect[i].Top := 32;
  if i in [17..23] then
   ColorRect[i].Top := 52;
  if i in [25..31] then
   ColorRect[i].Top := 75;
  if i in [33..39] then
   ColorRect[i].Top := 95;

  if ChangeColor = False then
   if SelectColor = AColor[i] then begin
    SelectRect := ColorRect[i];
    ChangeColor := True;
   end;

 end;

 if ChangeColor = False then
  SelectRect := CustomButtonRect;

end;

procedure TfrmOpenColorPicker.ColorChange;
begin
 // ** 메세지가 엉켜서 발생하는 메모리 참조 오류 방지 ** //
 FDestory := True;
 SendMessage(ParentHandle,  CM_ColorChange,  0,  0);
end;

procedure TfrmOpenColorPicker.FormCreate(Sender: TObject);
begin
 FDestory := False;

 SelectColor := clWhite;

 OldRect := Rect(0, 0, 0, 0);
 SelectRect := Rect(0, 0, 0, 0);
 CaptureButtonRect := Rect(40, 120, 127, 142);
 ConvertButtonRect := Rect(40, 180, 127, 202);
 CustomButtonRect := Rect(40, 150, 127, 172);

 ParentHandle := 0;

 Self.Font := Screen.MenuFont;
 Canvas.Font := Screen.MenuFont;

 DoubleBuffered := True;
end;

procedure TfrmOpenColorPicker.FormPaint(Sender: TObject);
var
 i:Integer;
 OtherColor:Boolean;
begin

 OtherColor := True;

 with Canvas do begin
  Pen.Color := COLOR_Border;
  Pen.Style := psDot;
  MoveTo(5,  69);
  LineTo(width-5,  69);
  MoveTo(5,  112);
  LineTo(width-5,  112);
  Pen.Style := psSolid;
 end;


 for i := 0 to High(AColor) do with Canvas do begin

  if OtherColor = True then
   if EqualRect(SelectRect,  ColorRect[i]) = True then begin
    DrawFrame(Canvas,   SelectRect,  COLOR_Border,  COLOR_Bk,  True);
    OtherColor := False;
   end;

  Pen.Color := clGray;
  Brush.Color := AColor[i];
  Rectangle(ColorRect[i]);

 end;

  with Canvas do begin

   if OtherColor = True then
    DrawFrame(Canvas, CustomButtonRect, COLOR_Border, COLOR_Bk, True);

   Brush.Style := bsClear;

   TextOut(CaptureButtonRect.Left + 3, CaptureButtonRect.Top + 3,
           ' 컬러 캡쳐 ...');
   TextOut(CustomButtonRect.Left + 3, CustomButtonRect.Top + 3,
           ' 기타 색상 ...');
   TextOut(ConvertButtonRect.Left + 3, ConvertButtonRect.Top + 3,
           ' 색상 변환 ...');
  end;

 DrawClientFrame(Canvas,  Self.ClientRect,  COLOR_Frame);

end;

procedure TfrmOpenColorPicker.FormMouseMove(Sender: TObject;
  Shift: TShiftState; X,   Y: Integer);
var
 i:Integer;
begin

 RePaint;

 for i := 0 to High(AColor) do with ColorRect[i] do
  if PtInRect(Rect(Left-3, Top-3, Right+3, Bottom+3),  Point(X, Y)) = True then
   if EqualRect(ColorRect[i],  SelectRect) = False then begin
    DrawFrame(Canvas,  ColorRect[i], COLOR_Border, COLOR_Bk, False);
    break;
   end;

 if PtInRect(CaptureButtonRect, Point(X, Y)) = True then
   DrawFrame(Canvas, CaptureButtonRect, COLOR_Border, COLOR_Bk, False);

 if PtInRect(CustomButtonRect, Point(X, Y)) = True then
  if EqualRect(CustomButtonRect, SelectRect) = False then
   DrawFrame(Canvas,  CustomButtonRect, COLOR_Border, COLOR_Bk, False);

 if PtInRect(ConvertButtonRect, Point(X, Y)) = True then
  if EqualRect(ConvertButtonRect, SelectRect) = False then
   DrawFrame(Canvas, ConvertButtonRect, COLOR_Border, COLOR_Bk, False);

end;

procedure TfrmOpenColorPicker.FormMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X,   Y: Integer);
var
 i:Integer;
begin

 if Button in [mbLeft] then
  with CaptureButtonRect do
   if PtInRect(Rect(Left - 3,  Top - 3,  Right + 3,  Bottom + 3),  Point(X,  Y)) = True then
    with TCaptureEngine.Create(nil) do
     try
      CaptureSetup(False, False, False, True, 500, False, '');

      FDestory := True;
      Self.Hide;

      CaptureColor(SelectColor);

      ColorChange;

      Exit;
     finally
      Free;
     end;

 if Button in [mbLeft] then
  with CustomButtonRect do
   if PtInRect(Rect(Left - 3, Top - 3, Right + 3, Bottom + 3), Point(X,  Y)) = True then begin
    ColorDialog.Color := SelectColor;

    if ColorDialog.Execute = True then
     SelectColor := ColorDialog.Color;

    ColorChange;
    Exit;
   end;

 if Button in [mbLeft] then
  with ConvertButtonRect do
   if PtInRect(Rect(Left - 3, Top - 3, Right + 3, Bottom + 3), Point(X,  Y)) = True then
    with TfrmColorConvert.Create(nil) do
     try
      shpPrevColor.Brush.Color := SelectColor;
      ShowModal;

      SelectColor := shpNextColor.Brush.Color;
      ColorChange;

      Exit;
     finally
      Free;
     end;

  if Button in [mbLeft] then
   for i := 0 to High(AColor) do with ColorRect[i] do
    if PtInRect(Rect(Left-3,  Top-3, Right+3, Bottom+3), Point(X,  Y)) = True then begin
     SelectColor := AColor[i];
     ColorChange;
     Exit;
    end;

 FDestory := True;
 SendMessage(ParentHandle, CM_NotColorChange, 0, 0);

end;

procedure TfrmOpenColorPicker.FormDeactivate(Sender: TObject);
begin

 if FDestory = False then
  SendMessage(ParentHandle, CM_NotColorChange, 0, 0);

end;

procedure TfrmOpenColorPicker.FormDestroy(Sender: TObject);
begin
 Sleep(100);
end;

end.
