﻿// ** 오픈 캡쳐를 위한 ColorCombo.pas 수정판 ** //
{
  수정 된 부분 : 색상 이름 한글화
  추가 된 부분 : 목록에 없는 색상도 선택 가능(사용자 지정)
}

unit OpenColorCbx;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TOpenColorCbx = class(TCustomComboBox)
  private
    FCustomColor:TColor;
    function GetCustomColor:TColor;
    procedure SetCustomColor(Value:TColor);
    function GetSelectColor:TColor;
    procedure SetSelectColor(Value:TColor);
    procedure ComboBoxCloseUp(Sender:TObject);
  protected
    procedure SetStyle(Value:TComboBoxStyle); override;
    procedure DrawItem(Index:Integer;Rect:TRect;
      State: TOwnerDrawState); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner:TComponent); override;
    procedure AddColorList;
    property SelectColor:TColor read GetSelectColor write SetSelectColor;
  published
    property CustomColor:TColor read GetCustomColor write SetCustomColor;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
    property Items;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
  end;

procedure Register;

implementation

const
  ColorArray: array[0..15] of TColor =
  (
    clBlack, clMaroon, clGreen, clOlive, clNavy, clPurple, clTeal, clGray,
    clSilver, clRed, clLime, clYellow, clBlue, clFuchsia, clAqua, clWhite
  );
  ColorNameArray: array[0..16] of String =
  (
    '사용자 지정...','검정', '적갈색', '녹색', '올리브', '짙은 파랑', '자주', '청록', '회색',
    '은색', '빨강', '라임', '노랑', '파랑', '밝은 자홍', '바다색', '흰색'
  );

procedure Register;
begin
 RegisterComponents('OpenCanvas', [TOpenColorCbx]);
end;

function TOpenColorCbx.GetCustomColor:TColor;
begin
 Result := FCustomColor;
end;

procedure TOpenColorCbx.SetCustomColor(Value:TColor);
begin
 FCustomColor := Value;
 RePaint;
end;

function TOpenColorCbx.GetSelectColor:TColor;
begin
 if ItemIndex = 0 then
  Result := FCustomColor
   else Result := ColorArray[ItemIndex - 1];
end;

procedure TOpenColorCbx.SetSelectColor(Value:TColor);
var
 i:Integer;
 FEqual:Boolean;
begin
 FEqual := False;
 for i := Low(ColorArray) to High(ColorArray) do
  if ColorArray[i] = Value then begin
   ItemIndex := i+1;
   FEqual := True;

   Break;
  end;

  if FEqual = False then begin
   FCustomColor := Value;
   ItemIndex := 0;
  end;

end;

procedure TOpenColorCbx.ComboBoxCloseUp(Sender:TObject);
Const
 Drop_Height = 16;
var
 DropRect:TRect;
begin
 if ItemIndex = 0 then begin
  GetWindowRect(FindWindow('ComBoLBox', nil), DropRect);

  // ** 드롭 박스가 아래로 펼쳐졌다.
  DropRect.Bottom := DropRect.Top + Drop_Height;

  if PtInRect(DropRect, Mouse.CursorPos) = True then
   with TColorDialog.Create(Self) do
    try
     Color := FCustomColor;
     Options := [cdFullOpen];

     if Execute = True then
     // ** property CustomColor:TColor 를 써야 됨.. RePaint 하기 위해서 ** //
      CustomColor := Color;
    finally
     Free;
    end;

  end;

end;

procedure TOpenColorCbx.SetStyle(Value:TComboBoxStyle);
begin
 inherited SetStyle(csOwnerDrawFixed);
end;

procedure TOpenColorCbx.DrawItem(Index:Integer;Rect:TRect;State:TOwnerDrawState);
var
 DrawRect:TRect;
 AColor:TColor;
begin
 DrawRect.Top := Rect.Top+3;
 DrawRect.Bottom := Rect.Bottom-3;
 DrawRect.Left := Rect.Left+3;
 DrawRect.Right := DrawRect.Left+20;

 with Canvas do begin
  FillRect(Rect);
  AColor := Brush.Color;

  if Index = 0 then
   Brush.Color := FCustomColor
    else
     Brush.Color := ColorArray[Index-1];


  FillRect(DrawRect);
  Brush.Color := clBlack;
  FrameRect(DrawRect);
  Brush.Color := AColor;
 end;
 DrawRect := Rect;
 DrawRect.Left := DrawRect.Left+24;
 Inc(DrawRect.Top, 2);

 inherited DrawItem(Index, DrawRect, State);
end;

procedure TOpenColorCbx.CreateWnd;
begin
 inherited CreateWnd;

 AddColorList;
 ItemIndex := 16;

 OnCloseUp := ComboBoxCloseUp;

 DropDownCount := Items.Count;
end;

constructor TOpenColorCbx.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);

 FCustomColor := clWhite;
 Style := csOwnerDrawFixed;
end;

procedure TOpenColorCbx.AddColorList;
var
 i:Integer;
begin
 for i := Low(ColorNameArray) to High(ColorNameArray) do
  Items.Add(ColorNameArray[i]);
end;

end.
