﻿unit OpenPalette;

interface

uses
  Windows , Messages , SysUtils, Classes, Controls , Graphics ,
  ColorPicker , Dialogs;

Const
 OpenPaletteVersion = 'OpenPaletteVersion v1.1.0';
 SectionName = 'OpenColorPicker';


type
  TOpenPalette = class(TCustomControl)
  private
    { Private declarations }
    FColor:TColor;
    FFileName:String;
    MouseDown:Boolean;
    procedure SetColor(Value:TColor);
    procedure SetFileName(Value:String);
    procedure LoadCustomColors;
    procedure SaveCustomColors;
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure WMLButtonDown(var Message: TWMLButtonDown);
      message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp);
      message WM_LBUTTONUp;
    procedure CMColorChange(var Message: TMessage);
      message CM_ColorChange;
    procedure CMNotColorChange(var Message: TMessage);
      message CM_NotColorChange;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FreeColorPicker;
  published
    { Published declarations }
    property Color:TColor read FColor write SetColor;
    property FileName:String read FFileName write SetFileName;
  end;

  var
   frmOpenColorPicker: TfrmOpenColorPicker;

procedure Register;

implementation

 uses IniFiles;

constructor TOpenPalette.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);

 FColor := clWhite;
 FFileName := '';
 MouseDown := False;
 frmOpenColorPicker := nil;
end;

destructor TOpenPalette.Destroy;
begin
 inherited;
 // ** 혹시나 -_- ** //
 FreeColorPicker;
end;

procedure TOpenPalette.FreeColorPicker;
begin
 if frmOpenColorPicker <> nil then begin
  frmOpenColorPicker.Free;
  frmOpenColorPicker := nil;
 end;

end;

procedure TOpenPalette.SetColor(Value:TColor);
begin
 FColor := Value;
 Paint;
end;

procedure TOpenPalette.SetFileName(Value:String);
begin
 FFileName := Value;
end;

procedure TOpenPalette.LoadCustomColors;
Const
 ColorPrefix = 'Color';
var
 IniFile:TIniFile;
 AStream:TMemoryStream;
 i:Integer;
 Value:String;
begin
 if frmOpenColorPicker <> nil then
  if FileName <> '' then begin
   IniFile := TIniFile.Create(FileName);
    try
     AStream := TMemoryStream.Create;
      try
       IniFile.ReadBinaryStream(SectionName,'ColorStream',AStream);
       AStream.Position := 0;
       frmOpenColorPicker.ColorDialog.CustomColors.LoadFromStream(AStream);

       with frmOpenColorPicker do
        for i := 0 to High(AColor) do
         case i of
          0..23 : AColor[i] := ConstColor[i];
          24..39 : begin
                    Value := ColorDialog.CustomColors.Values[ColorPrefix + Char(Ord('A') + i-24)];
                    if Value <> ''then
                     AColor[i] := StrToInt('$'+Value)
                      else begin
                            AColor[i] := clWhite;
                            ColorDialog.CustomColors.Values[ColorPrefix + Char(Ord('A') + i-24)] := 'FFFFFF';
                           end;
                   end;
         end;

      finally
       AStream.Free;
      end;
    finally
     IniFile.Free;
    end;

  end else with frmOpenColorPicker do
            for i := 0 to High(AColor) do
             case i of
              0..23 : AColor[i] := ConstColor[i];
              24..39 : AColor[i] := clWhite;
            end;

end;

procedure TOpenPalette.SaveCustomColors;
var
 IniFile:TIniFile;
 AStream:TMemoryStream;
begin
 if (FileName <> '') and (frmOpenColorPicker <> nil) then begin
  IniFile := TIniFile.Create(FileName);
   try
    AStream := TMemoryStream.Create;
     try
      frmOpenColorPicker.ColorDialog.CustomColors.SaveToStream(AStream);
      AStream.Position := 0;
      IniFile.WriteBinaryStream(SectionName,'ColorStream',AStream);
     finally
      AStream.Free;
     end;
   finally
    IniFile.Free;
   end;

 end;

end;

procedure TOpenPalette.Paint;
begin

 with Canvas do begin
  Brush.Style := bsSolid;
  Brush.Color := FColor;
  Rectangle(Rect(0,0,Width,Height));

  Brush.Style := bsClear;
  Pen.Style := psSolid;

  Pen.Color := clGray;
  Rectangle(Rect(0,0,Width,Height));

  Pen.Color := clWhite;
  Rectangle(Rect(1,1,Width-1,Height-1));
 end;

end;

procedure TOpenPalette.WMLButtonDown(var Message: TWMLButtonDown);
begin
 SetCapture(Handle);
 MouseDown := True;
 Paint;
end;

procedure TOpenPalette.WMLButtonUp(var Message: TWMLButtonUp);
var
 FormPoint:TPoint;
begin
 ReleaseCapture;
 MouseDown := False;
 Paint;

 if frmOpenColorPicker = nil then begin
  frmOpenColorPicker := TfrmOpenColorPicker.Create(Self);

   with frmOpenColorPicker do begin
    LoadCustomColors;
    SelectColor := FColor;
    AddColorRect;
    ParentHandle := Self.Handle;

    FormPoint := Point(Message.XPos, Message.YPos);

    Windows.ClientToScreen(Self.Handle, FormPoint);

    Left := FormPoint.X; Top := FormPoint.Y;

    Show;
   end;
 end;

end;

procedure TOpenPalette.CMColorChange(var Message: TMessage);
begin

 if frmOpenColorPicker <> nil then begin
  SetColor(frmOpenColorPicker.SelectColor);
  SaveCustomColors;
  FreeColorPicker;
 end;

end;

procedure TOpenPalette.CMNotColorChange(var Message: TMessage);
begin
 FreeColorPicker;
end;

procedure Register;
begin
  RegisterComponents('OpenCanvas', [TOpenPalette]);
end;

end.
