﻿unit OpenThreadTimer;

interface

uses
  SysUtils, Classes, Dialogs;


type
  TThreadTimer = class(TThread)
  private
    { Private declarations }
   procedure OnEvent;
  public
   FSender: TObject;
   FEvent: TNotifyEvent;
   FInterval: Integer;
  protected
    procedure Execute; override;
  end;


type
  TOpenThreadTimer = class(TObject)
  private
    { Private declarations }
    FEnabled: Boolean;
    FInterval: Integer;
    FOnThreadTimer: TNotifyEvent;
    ThreadTimer: TThreadTimer;
    procedure SetEnabled(Value: Boolean);
    procedure SetOnThreadTimer(Value: TNotifyEvent);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
  published
    { Published declarations }
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Interval: Integer read FInterval write FInterval;
    property OnThreadTimer: TNotifyEvent read FOnThreadTimer write SetOnThreadTimer;
  end;

implementation

{ TThreadTimer }

procedure TThreadTimer.Execute;
begin
  inherited;

  while not Terminated do begin
   Sleep(FInterval);
   // ** 싱크로 나이즈 ** //
   Synchronize(OnEvent);
  end;

end;

procedure TThreadTimer.OnEvent;
begin
 if Assigned(FEvent) = True then
  // ** OpenThreadTimer 에 할당 된 이벤트를 실행 시킴 ** //
  FEvent(FSender);
end;

{ TOpenThreadTimer }

constructor TOpenThreadTimer.Create;
begin
  inherited;

  FInterval := 30;
  FEnabled := False;
  ThreadTimer := nil;
  FOnThreadTimer := nil;
end;

destructor TOpenThreadTimer.Destroy;
begin
  FOnThreadTimer := nil;

  if FEnabled = True then
   SetEnabled(False);

  inherited;
end;

procedure TOpenThreadTimer.SetEnabled(Value: Boolean);
begin
 if Value = True then
  if (ThreadTimer = nil) and (Assigned(FOnThreadTimer) = True) then begin
   ThreadTimer := TThreadTimer.Create(True);
   ThreadTimer.FSender := Self;
   ThreadTimer.FEvent := FOnThreadTimer;
   ThreadTimer.FInterval := FInterval;
   ThreadTimer.Resume;

   FEnabled := True;
   Exit;
  end;

 if Value = False then
  if ThreadTimer <> nil then begin
   ThreadTimer.Terminate;
   ThreadTimer.WaitFor;
   ThreadTimer := nil;

   FEnabled := False;
  end;

end;

procedure TOpenThreadTimer.SetOnThreadTimer(Value: TNotifyEvent);
begin
 FOnThreadTimer := Value;
end;

end.
 