﻿// ** 오픈 캡쳐에서 사용되는 함수의 집합들이다 ** //

unit OpenInterface;

interface

uses Windows, Classes, Graphics, SysUtils, Forms, Dialogs, math,
  IniFiles, ComCtrls, StdCtrls, Menus, SHDocVw, Controls,
  Buttons, Spin;

procedure SetBigBitmapSize(var Bitmap: TBitmap; Width, Height: Integer);

//
// 데스크탑 모니터 좌표를 원점 좌표로 변경하거나 원점 좌표를 데스크탑 좌표로 변경하는 함수들입니다.
// 듀얼 모니터 때문에 추가된 함수입니다.
//
function DesktopCoordinateToOriginCoordinate(DesktopCoordinate: TRect): TRect;
  overload;
function OriginCoordinateToDesktopCoordinate(OriginCoordinate: TRect): TRect;
  overload;

procedure DesktopCoordinateToOriginCoordinate(DesktopCoordinate: HRGN);
  overload;
procedure OriginCoordinateToDesktopCoordinate(OriginCoordinate: HRGN); overload;

function DesktopCoordinateToOriginCoordinate(DesktopCoordinate: TPoint)
  : TPoint; overload;
function OriginCoordinateToDesktopCoordinate(OriginCoordinate: TPoint): TPoint;
  overload;
//

// N개의 모니터에 사각형 좌표들을 모두 합한 리전을 리턴합니다.
// 이 함수를 호출 후 반드시 리전을 DeleteObject로 해제하여야 합니다.
function GetDesktopRgn(): HRGN;
// GetDesktopRgn과 성격이 거의 비슷하나 전체 좌표를 가져오는게 아니라
// 작업 영역에 좌표를 가져옵니다. ( 작업 표시줄을 제외한 좌표 )
function GetDesktopWorkRgn(): HRGN;

// UAC와 관련하여 관리자 권한이 없는 프로세스가 관리자 권한이 있는
// 프로세스에 ChildWindowFromPoint 를 요청할 경우 차일드 객체를 가져올 수 없는
// 문제점이 있습니다. ( 그러나, WindowsFromPoint 는 다 가져올 수 있음 )
// 이러한 문제를 해결하기 위해서 자체적으로 ChildWindowFromPoint를 구현하였습니다.
function MyChildWindowFromPoint(WindowHandle: THandle; Point: TPoint): THandle;

// 윈도우 핸들로부터 윈도우 리전을 가져옵니다.
// AutoCreate = True일 경우 Rgn을 새로 생성하여 리턴합니다.
// ( 이 함수는 대상 윈도우 핸들 좌표영역이 데스크탑 영역밖으로 나갈 경우
// 가려진 부분에 영역을 제외해서 리전을 가져옵니다. )
function GetWindowRealRgn(WindowHandle: HWND; var Rgn: HRGN;
  AutoCreate: Boolean): Boolean;
// 위에 함수와 거의 동일하며 차일드 윈도우에 좌표를 가져옵니다.
function GetChildWindowRealRgn(TopHandle, ChildHandle: HWND; var Rgn: HRGN;
  AutoCreate: Boolean): Boolean;

// 이 윈도우에 클라이언트 좌표를 가져옵니다.
function GetWorkRect(WindowHandle: HWND; var lpRect: TRect): Boolean;

procedure NormalizePoint(var FirstPt, EndPt: TPoint);
procedure MinMaxInteger(Const Min, Max: Integer; var AInteger: Integer);
procedure MinMaxRect(Const MinX, MaxX, MinY, MaxY: Integer; var lpRect: TRect);

function IsScrollnBar(TopHandle, WindowHandle: HWND; nBar: Integer): Boolean;
function IsScrollBar(WindowHandle: HWND): Boolean;

// 파이어 폭스, 넷스케이프, 오페라 스크롤 바인지 체크합니다.
function IsMozillaScrollBar(WindowHandle: HWND): Boolean;
// IE 스크롤 바인지 체크합니다.
function IsIEScrollBar(WindowHandle: HWND): Boolean;
// 글 2007 스크롤 바인지 체크합니다.
function IsHangul2007ScrollBar(WindowHandle: HWND): Boolean;
// MS Office 2007 Word 스크롤 바인지 체크합니다.
function IsMSOfficeWord2007ScrollBar(WindowHandle: HWND): Boolean;
// MS Office 2007 Excel 스크롤 바인지 체크합니다.
function IsMSOfficeExcel2007ScrollBar(WindowHandle: HWND): Boolean;
// GoogleChrome 스크롤 바인지 체크합니다.
function IsGoogleChromeScrollBar(WindowHandle: HWND): Boolean;

// 웹 브라우저의 스크롤 바인지 체크합니다.
function IsWebScrollBar(WindowHandle: HWND): Boolean;
// 오피스 스크롤 바인지 체크합니다.
function IsOfficeScrollBar(WindowHandle: HWND): Boolean;

function GetScrollBarHandle(WindowHandle: HWND): HWND;

// 인자로 넘어온 사각형 좌표를 데스크탑 영역에 벗어나지 않는 사각형 좌표로 변경합니다.
// 모니터가 N개일 경우 ( 단, N은 1이상 ) 이 함수는 되도록이면 쓰지 않는 것이 좋습니다.
// 왜냐하면 각각의 모니터들은 다른 해상도를 가질 수 있으며 이러한 좌표들을 합치면
// 리전형태로 관리되어야 하기 때문입니다.
procedure SetRectInDesktopRect(var ARect: TRect);
// SetRectInDesktopRect함수와 비슷하나 작업영역을 기준으로 한다는 점에서 다릅니다.
// procedure SetRectInDesktopWorkRect(var ARect: TRect);
procedure SetRectInParent(WindowHandle: HWND; var ARect: TRect);

procedure SetFontAntiAlias(Font: TFont);
procedure FreeFontAntiAlias(Font: TFont);

function GetStringsWidth(Canvas: TCanvas; Strings: TStrings): Integer;
function GetStringsHeight(Canvas: TCanvas; Strings: TStrings): Integer;

function EqualPoint(Const Pt, Pt1: TPoint): Boolean;
function EqualPointXY(Const Pt, Pt1: TPoint): Boolean;

function SetFileName(Const FileName: String): String;

procedure ShortCutToHotKey(ShortCut: TShortCut; var HotKey: Uint;
  var Key: Word);
function SetWindowHotKey(WindowHandle: HWND; ID: Integer; ShortCut: TShortCut)
  : Boolean;

function DrawTriangle(Canvas: TCanvas; L, T, R, B: Integer): Boolean;

procedure LoadINI(AForm: TForm; ComponentStates: Boolean; Const SectionName,
  FileName: String);
procedure SaveINI(AForm: TForm; ComponentStates: Boolean; Const SectionName,
  FileName: String);

procedure LoadCheckMenuItem(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
procedure SaveCheckMenuItem(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
procedure LoadCheckMenuItems(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
procedure SaveCheckMenuItems(MenuItem: TMenuItem; Const SectionName,
  FileName: String);

procedure InfoMessage(Caption, Text: String); Overload;
procedure InfoMessage(Text: String); Overload;

procedure BeginCursor;
procedure EndCursor;

implementation

uses accessIE, OpenColorCbx, GR32_RangeBars, OpenPalette;

procedure SetBigBitmapSize(var Bitmap: TBitmap; Width, Height: Integer);
var
  p4: Pointer;
  BitmapInfo: TBitmapInfo;
begin
  BitmapInfo.bmiHeader.biSize := sizeof(TBitmapInfo);
  BitmapInfo.bmiHeader.biWidth := Width;
  BitmapInfo.bmiHeader.biHeight := Height;
  BitmapInfo.bmiHeader.biPlanes := 1;
  BitmapInfo.bmiHeader.biBitCount := 32;
  BitmapInfo.bmiHeader.biCompression := BI_RGB;
  Bitmap.Handle := CreateDIBSection(GetDC(0), BitmapInfo, DIB_RGB_COLORS, p4,
    0, 0);
  ZeroMemory(p4, Width * Height * 4);
end;

function DesktopCoordinateToOriginCoordinate(DesktopCoordinate: TRect): TRect;
var
  OriginCoordinate: TRect;
begin
  OriginCoordinate := DesktopCoordinate;

  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
  begin
    Inc(OriginCoordinate.Left, -Screen.DesktopLeft);
    Inc(OriginCoordinate.Top, -Screen.DesktopTop);
    Inc(OriginCoordinate.Right, -Screen.DesktopLeft);
    Inc(OriginCoordinate.Bottom, -Screen.DesktopTop);
  end;

  Result := OriginCoordinate;
end;

function OriginCoordinateToDesktopCoordinate(OriginCoordinate: TRect): TRect;
var
  DesktopCoordinate: TRect;
begin
  DesktopCoordinate := OriginCoordinate;

  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
  begin
    Inc(DesktopCoordinate.Left, Screen.DesktopLeft);
    Inc(DesktopCoordinate.Top, Screen.DesktopTop);
    Inc(DesktopCoordinate.Right, Screen.DesktopLeft);
    Inc(DesktopCoordinate.Bottom, Screen.DesktopTop);
  end;

  Result := DesktopCoordinate;
end;

procedure DesktopCoordinateToOriginCoordinate(DesktopCoordinate: HRGN);
begin
  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
    OffsetRgn(DesktopCoordinate, -Screen.DesktopLeft, -Screen.DesktopTop);
end;

procedure OriginCoordinateToDesktopCoordinate(OriginCoordinate: HRGN);
begin
  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
    OffsetRgn(OriginCoordinate, Screen.DesktopLeft, Screen.DesktopTop);
end;

function DesktopCoordinateToOriginCoordinate(DesktopCoordinate: TPoint): TPoint;
var
  OriginCoordinate: TPoint;
begin
  OriginCoordinate := DesktopCoordinate;

  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
  begin
    Inc(OriginCoordinate.X, -Screen.DesktopLeft);
    Inc(OriginCoordinate.Y, -Screen.DesktopTop);
  end;

  Result := OriginCoordinate;
end;

function OriginCoordinateToDesktopCoordinate(OriginCoordinate: TPoint): TPoint;
var
  DesktopCoordinate: TPoint;
begin
  DesktopCoordinate := OriginCoordinate;

  if (Screen.DesktopLeft < 0) or (Screen.DesktopTop < 0) then
  begin
    Inc(DesktopCoordinate.X, Screen.DesktopLeft);
    Inc(DesktopCoordinate.Y, Screen.DesktopTop);
  end;

  Result := DesktopCoordinate;
end;

function GetDesktopRgn: HRGN;
var
  i: Integer;

  DesktopRgn: HRGN;
  TempRgn: HRGN;
begin
  DesktopRgn := 0;

  for i := 0 to Screen.MonitorCount - 1 do
  begin
    if DesktopRgn = 0 then
      DesktopRgn := CreateRectRgnIndirect(Screen.Monitors[i].BoundsRect)
    else
    begin
      TempRgn := CreateRectRgnIndirect(Screen.Monitors[i].BoundsRect);

      CombineRgn(DesktopRgn, DesktopRgn, TempRgn, RGN_OR);

      DeleteObject(TempRgn);
    end;
  end;

  Result := DesktopRgn;
end;

function GetDesktopWorkRgn(): HRGN;
var
  i: Integer;

  DesktopWorkRgn: HRGN;
  TempRgn: HRGN;
begin
  DesktopWorkRgn := 0;

  for i := 0 to Screen.MonitorCount - 1 do
  begin
    if DesktopWorkRgn = 0 then
      DesktopWorkRgn := CreateRectRgnIndirect(Screen.Monitors[i].WorkareaRect)
    else
    begin
      TempRgn := CreateRectRgnIndirect(Screen.Monitors[i].WorkareaRect);

      CombineRgn(DesktopWorkRgn, DesktopWorkRgn, TempRgn, RGN_OR);

      DeleteObject(TempRgn);
    end;
  end;

  Result := DesktopWorkRgn;
end;

function PointInWindow(WindowHandle: THandle; Point: TPoint): Boolean;
var
  WindowRgn: HRGN;
begin
  GetWindowRealRgn(WindowHandle, WindowRgn, True);
  Result := PtInRegion(WindowRgn, Point.X, Point.Y);

  DeleteObject(WindowRgn);
end;


function MyChildWindowFromPoint(WindowHandle: THandle; Point: TPoint): THandle;
begin
  Result := WindowHandle;

  // 해당 윈도우에 Z순서가 가장 최상단에 있는 ( 가장 맨 앞에 있는 컨트롤 )
  // 첫 번째 차일드 윈도우를 가져옵니다.
  WindowHandle := GetWindow(WindowHandle, GW_CHILD);
  while True do
  begin
    if WindowHandle = 0 then
      break;

    // GetWindow(WindowHandle, GW_HWNDNEXT);
    // 를 통해서 다음 윈도우를 찾을 때 다음 윈도우는
    // 현재 윈도우보다 Z 순서 크기가 작습니다.
    // 즉, 현재 윈도우보다 뒤에 있습니다.

    if IsWindowVisible(WindowHandle) = False then
    begin
      // 이 윈도우가 숨겨져 있습니다.
      // 다음 윈도우를 찾습니다.
      WindowHandle := GetWindow(WindowHandle, GW_HWNDNEXT);

      Continue;
    end;

    // 이 윈도우가 현재 마우스 좌표에 포함되는 영역에 있는지 확인합니다.
    if PointInWindow(WindowHandle, Point) = False then
    begin
      // 포함되지 않는 영역에 있습니다.
      // 다음 윈도우를 찾습니다.
      WindowHandle := GetWindow(WindowHandle, GW_HWNDNEXT);

      Continue;
    end;

    // 현재 마우스 좌표에 포함되는 영역이므로 여기서 루프를 마치며 해당 윈도우에
    // 차일드 윈도우에 대해서 검사합니다.
    // 즉, 모든 윈도우를 순회하게 됩니다.
    Result := MyChildWindowFromPoint(WindowHandle, Point);

    break;
  end;
end;

function GetWindowRealRgn(WindowHandle: HWND; var Rgn: HRGN;
  AutoCreate: Boolean): Boolean;
var
  DesktopRgn: HRGN;
  WindowRect: TRect;
begin
  Result := False;
  if AutoCreate = True then
    Rgn := CreateRectRgn(0, 0, 0, 0);

  if GetWindowRect(WindowHandle, WindowRect) = False then
    Exit;

  DesktopRgn := GetDesktopRgn;

  if GetWindowRgn(WindowHandle, Rgn) <> ERROR then
  begin
    OffsetRgn(Rgn, WindowRect.Left, WindowRect.Top);

    Result := True;
  end
  else
  begin
    DeleteObject(Rgn);
    Rgn := CreateRectRgnIndirect(WindowRect);

    Result := True;
  end;
  CombineRgn(Rgn, DesktopRgn, Rgn, RGN_AND);
  DeleteObject(DesktopRgn);
end;

function GetChildWindowRealRgn(TopHandle, ChildHandle: HWND; var Rgn: HRGN;
  AutoCreate: Boolean): Boolean;
var
  TopRect, ChildRect: TRect;
  TopRgn, ChildRgn, DesktopRgn: HRGN;
begin

  GetWindowRect(TopHandle, TopRect);
  GetWindowRect(ChildHandle, ChildRect);

  TopRgn := CreateRectRgnIndirect(TopRect);
  ChildRgn := CreateRectRgnIndirect(ChildRect);

  if GetWindowRgn(TopHandle, TopRgn) <> ERROR then
    OffsetRgn(TopRgn, TopRect.Left, TopRect.Top)
  else
  begin
    // SetRectInDesktopRect(TopRect);
    TopRgn := CreateRectRgnIndirect(TopRect);
  end;

  if GetWindowRgn(ChildHandle, ChildRgn) <> ERROR then
    OffsetRgn(ChildRgn, ChildRect.Left, ChildRect.Top)
  else
  begin
    // SetRectInDesktopRect(ChildRect);
    ChildRgn := CreateRectRgnIndirect(ChildRect);
  end;

  if AutoCreate = True then
    Rgn := CreateRectRgn(0, 0, 0, 0);

  CombineRgn(Rgn, TopRgn, ChildRgn, RGN_AND);

  DesktopRgn := GetDesktopRgn;

  CombineRgn(Rgn, DesktopRgn, Rgn, RGN_AND);

  DeleteObject(TopRgn);
  DeleteObject(ChildRgn);
  DeleteObject(DesktopRgn);

  Result := Rgn <> 0;
end;

// 해당 윈도우의 작업 영역을 구합니다.
function GetWorkRect(WindowHandle: HWND; var lpRect: TRect): Boolean;
var
  WindowInfo: TWindowInfo;
begin
  WindowInfo.cbSize := sizeof(TWindowInfo);
  GetWindowInfo(WindowHandle, WindowInfo);

  lpRect := WindowInfo.rcClient;

  Result := True;
end;

// 사용자 캡쳐 시 오른쪽 아래로 드래그로 캡쳐하지 않고 그 외에 다른 방향으로 캡쳐했을 때
// 오른쪽 아래로 드래그한 것과 같은 포인터로 돌려주는함수입니다.
procedure NormalizePoint(var FirstPt, EndPt: TPoint);
var
  Pt, Pt1: TPoint;
begin

  if FirstPt.X - EndPt.X > 0 then
  begin
    Pt.X := FirstPt.X;
    Pt1.X := EndPt.X;
    FirstPt.X := Pt1.X;
    EndPt.X := Pt.X;
  end;

  if FirstPt.Y - EndPt.Y > 0 then
  begin
    Pt.Y := FirstPt.Y;
    Pt1.Y := EndPt.Y;
    FirstPt.Y := Pt1.Y;
    EndPt.Y := Pt.Y;
  end;

end;

procedure MinMaxInteger(Const Min, Max: Integer; var AInteger: Integer);
begin
  if AInteger < Min then
    AInteger := 0;
  if AInteger > Max then
    AInteger := Max;
end;

procedure MinMaxRect(Const MinX, MaxX, MinY, MaxY: Integer; var lpRect: TRect);
begin
  MinMaxInteger(MinX, MaxX, lpRect.Left);
  MinMaxInteger(MinY, MaxY, lpRect.Top);
  MinMaxInteger(MinX, MaxX, lpRect.Right);
  MinMaxInteger(MinY, MaxY, lpRect.Bottom);
end;

function IsScrollnBar(TopHandle, WindowHandle: HWND; nBar: Integer): Boolean;
var
  ScrollInfo: TScrollInfo;
  ClassName: Array [0 .. 255] of Char;
  rcClient: TRect;
  ScrollBarHandle: HWND;
begin
  Result := False;

  ZeroMemory(@ScrollInfo, sizeof(ScrollInfo));
  ScrollInfo.cbSize := sizeof(ScrollInfo);
  ScrollInfo.fMask := SIF_TRACKPOS;

  if nBar = SB_CTL then
  begin

    GetWorkRect(WindowHandle, rcClient);

    if TopHandle = 0 then
      ScrollBarHandle := WindowFromPoint
        (Point(rcClient.Right + 3, rcClient.Top + 80))
    else
      ScrollBarHandle := MyChildWindowFromPoint
        (TopHandle, Point(rcClient.Right + 3, rcClient.Top + 80));

    if ScrollBarHandle = 0 then
      Exit;

    // MyChildWindowFromPoint 는 0을 리턴하지 않고 실패 시
    // 인자로 넘어온 핸들을 리턴합니다.
    if ScrollBarHandle = TopHandle then
      Exit;

    GetClassName(ScrollBarHandle, ClassName, sizeof(ClassName));

    // ** 반드시 클래스 이름이 ScrollBar 일때만 접근한다 ** //
    if (ClassName = 'ScrollBar') then
      if GetScrollInfo(ScrollBarHandle, nBar, ScrollInfo) = True then
      begin
        Result := True;
        Exit;
      end;
  end
  else
  begin
    // ** 스크롤바를 가져오는데 성공했고
    if GetScrollInfo(WindowHandle, nBar, ScrollInfo) and
    // 세로 스크롤바가 실제로 존재할 떄 .. ** //
      ((GetWindowLong(WindowHandle, GWL_STYLE) and WS_VSCROLL) <> 0) then
      Result := True;
  end;

end;

// ** 해당 윈도우의 스크롤바가 포함되어 있는가.. ** //
function IsScrollBar(WindowHandle: HWND): Boolean;
begin
  Result := False;

  if IsScrollnBar(0, WindowHandle, SB_VERT) = True then
  begin
    Result := True;
    Exit;
  end;

  if IsScrollnBar(0, WindowHandle, SB_CTL) = True then
  begin
    Result := True;
    Exit;
  end;

end;

function IsMozillaScrollBar(WindowHandle: HWND): Boolean;
Const
  MinMargin = 13;
  Margin = 10;
var
  ClassName: Array [0 .. 255] of Char;
begin
  Result := False;

  GetClassName(WindowHandle, ClassName, sizeof(ClassName));

  if (ClassName = 'MozillaWindowClass') or (ClassName = 'NS_AE_WindowClass') or
    (ClassName = 'OperaWindowClass') then
  begin
    if GetParent(WindowHandle) <> 0 then
    begin
      { GetWindowRect(WindowHandle ,lpRect);
        GetWindowRect(GetParent(WindowHandle) ,ParentRect);

        ScrollSize := GetSystemMetrics(SM_CXVSCROLL);
        //Inc(ScrollSize ,Margin);
        Inc(ScrollSize ,lpRect.Left-ParentRect.Left);

        // ** 9x ~ NT ** //
        if ParentRect.Right - lpRect.Right in [MinMargin..ScrollSize + Margin] then
        }
      Result := True
      // else Result := False;
    end;
  end
  else
  begin // else Result := False;
    // if ClassName = 'WebViewWindowClass' then
    // Result := True;
  end;
end;

function IsIEScrollBar(WindowHandle: HWND): Boolean;
var
  IE: IWebbrowser2;
begin
  Result := GetIEFromHandle(WindowHandle, IE) = S_OK;
end;

function IsWebScrollBar(WindowHandle: HWND): Boolean;
begin
  Result := IsMozillaScrollBar(WindowHandle);
  if Result = True then
    Exit;

  Result := IsGoogleChromeScrollBar(WindowHandle);
  if Result = True then
    Exit;

  Result := IsIEScrollBar(WindowHandle);
end;

function IsOfficeScrollBar(WindowHandle: HWND): Boolean;
begin
  Result := (IsHangul2007ScrollBar(WindowHandle) or IsMSOfficeWord2007ScrollBar
      (WindowHandle) or IsMSOfficeExcel2007ScrollBar(WindowHandle));
end;

function IsHangul2007ScrollBar(WindowHandle: HWND): Boolean;
var
  ClassName: Array [0 .. 255] of Char;
begin
  Result := False;

  GetClassName(WindowHandle, ClassName, sizeof(ClassName));
  if ClassName = 'HwpMainEditWnd' then
    Result := True;
end;

function IsMSOfficeWord2007ScrollBar(WindowHandle: HWND): Boolean;
var
  ClassName: Array [0 .. 255] of Char;
begin
  Result := False;

  GetClassName(WindowHandle, ClassName, sizeof(ClassName));
  if ClassName = '_WwG' then
    Result := True;

end;

function IsMSOfficeExcel2007ScrollBar(WindowHandle: HWND): Boolean;
var
  ClassName: Array [0 .. 255] of Char;
begin
  Result := False;

  GetClassName(WindowHandle, ClassName, sizeof(ClassName));
  if ClassName = 'EXCEL7' then
    Result := True;

end;

function IsGoogleChromeScrollBar(WindowHandle: HWND): Boolean;
var
  ClassName: Array [0 .. 255] of Char;
begin
  Result := False;

  GetClassName(WindowHandle, ClassName, sizeof(ClassName));
  if ClassName = 'Chrome_RenderWidgetHostHWND' then
    Result := True;

end;

function GetScrollBarHandle(WindowHandle: HWND): HWND;
var
  lpRect: TRect;
  ClassName: Array [0 .. 255] of Char;
begin
  GetWorkRect(WindowHandle, lpRect);
  Result := WindowFromPoint(Point(lpRect.Right + 2, lpRect.Bottom div 2));
  GetClassName(Result, ClassName, sizeof(ClassName));
  if ClassName <> 'ScrollBar' then
    Result := 0;
end;

procedure SetRectInDesktopRect(var ARect: TRect);
var
  Rgn, DesktopRgn: HRGN;
begin
  DesktopRgn := GetDesktopRgn;
  Rgn := CreateRectRgnIndirect(ARect);

  if Rgn = 0 then
    Exit;

  if CombineRgn(Rgn, DesktopRgn, Rgn, RGN_AND) <> ERROR then
    GetRgnBox(Rgn, ARect);

  DeleteObject(Rgn);
  DeleteObject(DesktopRgn);
end;

procedure SetRectInParent(WindowHandle: HWND; var ARect: TRect);
var
  ParentRect: TRect;
  ChildRect: TRect;
  ParentHandle: HWND;
begin
  ChildRect := ARect;
  if GetParent(WindowHandle) = 0 then
    Exit;

  ParentHandle := GetParent(WindowHandle);
  GetWindowRect(ParentHandle, ParentRect);

  if ChildRect.Left < ParentRect.Left then
    ChildRect.Left := ParentRect.Left;
  if ChildRect.Top < ParentRect.Top then
    ChildRect.Top := ParentRect.Top;
  if ChildRect.Right > ParentRect.Right then
    ChildRect.Right := ParentRect.Right;
  if ChildRect.Bottom > ParentRect.Bottom then
    ChildRect.Bottom := ParentRect.Bottom;

  ARect := ChildRect;

end;

procedure SetFontAntiAlias(Font: TFont);
var
  LogFont: TLogFont;
begin
  GetObject(Font.Handle, sizeof(LogFont), Addr(LogFont));
  LogFont.lfQuality := ANTIALIASED_QUALITY;
  Font.Handle := CreateFontIndirect(LogFont);
end;

procedure FreeFontAntiAlias(Font: TFont);
var
  LogFont: TLogFont;
begin
  GetObject(Font.Handle, sizeof(LogFont), Addr(LogFont));
  LogFont.lfQuality := DEFAULT_QUALITY;
  Font.Handle := CreateFontIndirect(LogFont);
end;

function GetStringsWidth(Canvas: TCanvas; Strings: TStrings): Integer;
var
  i: Integer;
  AWidth: Integer;
begin
  Result := -1;

  for i := 0 to Strings.Count - 1 do
  begin
    AWidth := Canvas.TextWidth(Strings.Strings[i]);
    if AWidth > Result then
      Result := AWidth;
  end;

end;

function GetStringsHeight(Canvas: TCanvas; Strings: TStrings): Integer;
var
  i: Integer;
begin

  if Strings.Count = 0 then
    Result := -1
  else
    Result := 0;

  for i := 0 to Strings.Count - 1 do
  begin

    if Strings.Strings[i] = '' then
      Inc(Result, Canvas.TextHeight(' '))
    else
      Inc(Result, Canvas.TextHeight(Strings.Strings[i]));

  end;

end;

function EqualPoint(Const Pt, Pt1: TPoint): Boolean;
begin
  Result := (Pt.X = Pt1.X) and (Pt.Y = Pt1.Y);
end;

function EqualPointXY(Const Pt, Pt1: TPoint): Boolean;
begin
  Result := (Pt.X = Pt1.X) or (Pt.Y = Pt1.Y);
end;

function SetFileName(Const FileName: String): String;
Const
  dName: Array [0 .. 8] of Char = ('\', '/', ':', '*', '"', '<', '>', '|', '?');
var
  i: Integer;
begin
  Result := FileName;

  for i := 0 to 8 do
    Result := StringReplace(Result, dName[i], '', [rfreplaceAll, rfIgnoreCase]);

  Result := StringReplace(Result, #13 + #10, ' ', [rfreplaceAll, rfIgnoreCase]);

end;

procedure ShortCutToHotKey(ShortCut: TShortCut; var HotKey: Uint;
  var Key: Word);
var
  MenuShift: TShiftState;
begin
  ShortCutToKey(ShortCut, Key, MenuShift);

  HotKey := 0;
  if ssShift in MenuShift then
    HotKey := Mod_Shift;
  if ssCtrl in MenuShift then
    HotKey := HotKey or Mod_Control;
  if ssAlt in MenuShift then
    HotKey := HotKey or Mod_Alt;
end;

function SetWindowHotKey(WindowHandle: HWND; ID: Integer; ShortCut: TShortCut)
  : Boolean;
var
  MenuKey: Word;
  HotKey: Uint;
begin
  Result := True;
  ShortCutToHotKey(ShortCut, HotKey, MenuKey);
  if not RegisterHotkey(WindowHandle, ID, HotKey, MenuKey) then
    Result := False;
end;

function DrawTriangle(Canvas: TCanvas; L, T, R, B: Integer): Boolean;
var
  Triangle: Array [0 .. 3] of TPoint;
begin

  Result := not((Canvas = nil) or (Canvas.Handle = 0));
  if Result = False then
    Exit
  else
    Result := True;

  if B > T then
  begin
    Triangle[0].X := L + ((R - L) div 2);
    Triangle[0].Y := T;
    Triangle[1].X := L;
    Triangle[1].Y := B;
    Triangle[2].X := R;
    Triangle[2].Y := B;
    Triangle[3].X := L + ((R - L) div 2);
    Triangle[3].Y := T;
    Canvas.Polygon(Triangle);
  end;

  if B < T then
  begin
    Triangle[0].X := R + ((L - R) div 2);
    Triangle[0].Y := B;
    Triangle[1].X := R;
    Triangle[1].Y := T;
    Triangle[2].X := L;
    Triangle[2].Y := T;
    Triangle[3].X := R + ((L - R) div 2);
    Triangle[3].Y := B;
    Canvas.Polygon(Triangle);
  end;

end;

procedure LoadINI(AForm: TForm; ComponentStates: Boolean; Const SectionName,
  FileName: String);
var
  i: Integer;
  INIFile: TINIFile;
begin

  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try

      with AForm do
      begin
        if ComponentStates = True then
          for i := 0 to ComponentCount - 1 do
          begin

            if Components[i] is TPageControl = True then
              with TPageControl(Components[i]) do
                ActivePageIndex := INIFile.ReadInteger
                  (SectionName, Name, ActivePageIndex);

            if (Components[i] is TComboBox = True) or
              (Components[i] is TOpenColorCbx = True) then
              with TComboBox(Components[i]) do
              begin
                if AForm.Components[i] is TOpenColorCbx = True then
                begin
                  TOpenColorCbx(AForm.Components[i]).CustomColor :=
                    INIFile.ReadInteger(SectionName, Name + '_CustomColor',
                    clWhite);
                  // ** 프로그램에 설정 정보가 저장되어 있지 않을 경우 ** //
                  if INIFile.ReadInteger(SectionName, Name, -1) = -1 then
                    ItemIndex := 16
                  else
                    ItemIndex := INIFile.ReadInteger(SectionName, Name, 0);
                end
                else
                  ItemIndex := INIFile.ReadInteger
                    (SectionName, Name, ItemIndex);

              end;

            if (Components[i] is TUpDown) = True then
              with TUpDown(Components[i]) do
              begin

                Position := INIFile.ReadInteger(SectionName, Name, Position);
                // ** Position 으로 하면 안됨 ** //
                TEdit(Associate).Text := IntToStr
                  (INIFile.ReadInteger(SectionName, Name, 0));

              end;

            if (Components[i] is TCheckBox) = True then
              with TCheckBox(Components[i]) do
                Checked := INIFile.ReadBool(SectionName, Name, Checked);

            if (Components[i] is TEdit) = True then
              with TEdit(Components[i]) do
                Text := INIFile.ReadString(SectionName, Name, Text);

            if (Components[i] is TSpinEdit) = True then
              with TSpinEdit(Components[i]) do
                Value := INIFile.ReadInteger(SectionName, Name, Value);

            if (Components[i] is TScrollBar) = True then
              with TScrollBar(Components[i]) do
              begin
                Position := INIFile.ReadInteger(SectionName, Name, Position);
                OnChange(nil);
              end;

            if (Components[i] is TSpeedButton) = True then
              with TSpeedButton(Components[i]) do
                Down := INIFile.ReadBool(SectionName, Name, Down);

            if (Components[i] is TOpenDialog) = True then
              with TOpenDialog(Components[i]) do
              begin
                InitialDir := INIFile.ReadString(SectionName, Name, '');
                FilterIndex := INIFile.ReadInteger
                  (SectionName, Name + '_FilterIndex', FilterIndex);
              end;

            if (Components[i] is TSaveDialog) = True then
              with TSaveDialog(Components[i]) do
              begin
                InitialDir := INIFile.ReadString(SectionName, Name, '');
                FilterIndex := INIFile.ReadInteger
                  (SectionName, Name + '_FilterIndex', FilterIndex);
              end;

            if (Components[i] is TOpenPalette) = True then
              with TOpenPalette(Components[i]) do
                Color := INIFile.ReadInteger(SectionName, Name, Color);

          end;

        if AForm.BorderStyle = bsDialog then
        begin
          Left := (Screen.Width - Width) div 2;
          Top := (Screen.Height - Height) div 2;
        end
        else
        begin
          if INIFile.ValueExists(SectionName, 'Left') = True then
          begin
            Left := INIFile.ReadInteger(SectionName, 'Left', Left);
            Top := INIFile.ReadInteger(SectionName, 'Top', Top);
            Width := INIFile.ReadInteger(SectionName, 'Width', Width);
            Height := INIFile.ReadInteger(SectionName, 'Height', Height);
          end
          else
          begin
            Left := (Screen.Width - Width) div 2;
            Top := (Screen.Height - Height) div 2;
          end;
        end;
      end;

    finally
      Free;
    end;
end;

procedure SaveINI(AForm: TForm; ComponentStates: Boolean; Const SectionName,
  FileName: String);
var
  i: Integer;
  INIFile: TINIFile;
begin
  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try

      with AForm do
      begin
        if ComponentStates = True then
          for i := 0 to ComponentCount - 1 do
          begin

            if Components[i] is TPageControl = True then
              with TPageControl(Components[i]) do
                INIFile.WriteInteger(SectionName, Name, ActivePageIndex);

            if (Components[i] is TComboBox = True) or
              (Components[i] is TOpenColorCbx = True) then
              with TComboBox(Components[i]) do
              begin
                INIFile.WriteInteger(SectionName, Name, ItemIndex);
                if AForm.Components[i] is TOpenColorCbx = True then
                  INIFile.WriteInteger(SectionName, Name + '_CustomColor',
                    TOpenColorCbx(AForm.Components[i]).CustomColor);
              end;

            if (Components[i] is TUpDown) = True then
              with TUpDown(Components[i]) do
                INIFile.WriteInteger(SectionName, Name, Position);

            if (Components[i] is TCheckBox) = True then
              with TCheckBox(Components[i]) do
                INIFile.WriteBool(SectionName, Name, Checked);

            if (Components[i] is TSpinEdit) = True then
              with TSpinEdit(Components[i]) do
                INIFile.WriteInteger(SectionName, Name, Value);

            if (Components[i] is TEdit) = True then
              with TEdit(Components[i]) do
                INIFile.WriteString(SectionName, Name, Text);

            if (Components[i] is TScrollBar) = True then
              with TScrollBar(Components[i]) do
                INIFile.WriteInteger(SectionName, Name, Position);

            if (Components[i] is TSpeedButton) = True then
              with TSpeedButton(Components[i]) do
                INIFile.WriteBool(SectionName, Name, Down);

            if (Components[i] is TOpenDialog) = True then
              with TOpenDialog(Components[i]) do
              begin
                INIFile.WriteString(SectionName, Name, InitialDir);
                INIFile.WriteInteger(SectionName, Name + '_FilterIndex',
                  FilterIndex);
              end;

            if (Components[i] is TSaveDialog) = True then
              with TSaveDialog(Components[i]) do
              begin
                INIFile.WriteString(SectionName, Name, InitialDir);
                INIFile.WriteInteger(SectionName, Name + '_FilterIndex',
                  FilterIndex);
              end;

            if (Components[i] is TOpenPalette) = True then
              with TOpenPalette(Components[i]) do
                INIFile.WriteInteger(SectionName, Name, Color);

          end;

        if (AForm.WindowState = wsNormal) and (AForm.BorderStyle <> bsDialog)
          then
        begin
          INIFile.WriteInteger(SectionName, 'Left', Left);
          INIFile.WriteInteger(SectionName, 'Top', Top);
          INIFile.WriteInteger(SectionName, 'Width', Width);
          INIFile.WriteInteger(SectionName, 'Height', Height);
        end;

      end;

    finally
      Free;
    end;

end;

procedure LoadCheckMenuItem(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
var
  INIFile: TINIFile;
begin

  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try
      MenuItem.Checked := INIFile.ReadBool(SectionName, MenuItem.Name,
        MenuItem.Checked);
    finally
      Free;
    end;
end;

procedure SaveCheckMenuItem(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
var
  INIFile: TINIFile;
begin

  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try
      INIFile.WriteBool(SectionName, MenuItem.Name, MenuItem.Checked);
    finally
      Free;
    end;
end;

procedure LoadCheckMenuItems(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
var
  i: Integer;
  INIFile: TINIFile;
begin

  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try
      for i := 0 to MenuItem.Count - 1 do
        with MenuItem.Items[i] do
          Checked := INIFile.ReadBool(SectionName, Name, Checked);

    finally
      Free;
    end;

end;

procedure SaveCheckMenuItems(MenuItem: TMenuItem; Const SectionName,
  FileName: String);
var
  i: Integer;
  INIFile: TINIFile;
begin

  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) + FileName);
  with INIFile do
    try
      for i := 0 to MenuItem.Count - 1 do
        with MenuItem.Items[i] do
          INIFile.WriteBool(SectionName, Name, Checked);

    finally
      Free;
    end;

end;

procedure InfoMessage(Caption, Text: String);
begin
  application.MessageBox(PChar(Caption), PChar(Text),
    MB_OK + MB_ICONASTERISK + MB_DEFBUTTON1 + MB_APPLMODAL);
end;

procedure InfoMessage(Text: String);
begin
  InfoMessage(Text, application.Title);
end;

procedure BeginCursor;
begin
  Screen.Cursor := crHourGlass;
end;

procedure EndCursor;
begin
  Screen.Cursor := crDefault;
end;

end.
