﻿unit libImageEffect;

interface

uses Windows , Graphics , SysUtils , Types;


type
	TRGB = record
	 B, G, R : byte;
	end;

	PRGBArray = ^TRGBArray;
	TRGBArray = array[0..0] of TRGB;

procedure GrayScale(var ABitmap: TBitmap);
procedure Invert(var ABitmap: TBitmap);
procedure AddColorNoise(var clip: tbitmap; Amount: Integer);
procedure AddMonoNoise(var clip: tbitmap; Amount: Integer);
procedure AddLightness(var clip: tbitmap; Amount: Integer);
procedure AddDarkness(var src: tbitmap; Amount: integer);
procedure AddMosaic(var Bm:TBitmap;size:Integer);
function IsGIF(var FileName:String):boolean;
procedure AddSaturation(var clip: tbitmap; Amount: Integer);
procedure GetGIFSize(const sGIFFile: string; var wWidth, wHeight: Word);

implementation

 uses uMain;

function IntToByte(i:Integer):Byte;
begin
  if      i>255 then Result:=255
  else if i<0   then Result:=0
  else               Result:=i;
end;

procedure GrayScale(var ABitmap: TBitmap);
var
 p0: PByteArray;
 Gray, X, Y: Integer;
begin
  ABitmap.PixelFormat := pf24Bit;
  for Y:=0 to ABitmap.Height-1 do
  begin
    p0 := ABitmap.ScanLine[y];
    for X := 0 to ABitmap.Width-1 do
    begin
      Gray := Round(p0[X*3]*0.3+p0[X*3+1]*0.59+p0[X*3+2]*0.11);
      p0[X*3] := Gray;
      p0[X*3+1] := Gray;
      p0[X*3+2] := Gray;
    end;
  end;
end;

procedure Invert(var ABitmap: TBitmap);
var
 W, H, X, Y: Integer;
 p: PByteArray;
begin
 W := ABitmap.Width;
 H := ABitmap.Height;
 ABitmap.PixelFormat :=pf24bit;

 for Y := 0 to H - 1 do begin
  p := ABitmap.ScanLine[Y];

  for x:=0 to w-1 do begin
   p[X*3] := not p[X*3];
   p[X*3+1] := not p[X*3+1];
   p[X*3+2] := not p[X*3+2];
  end;

 end;

end;

procedure AddSaturation(var clip: tbitmap; Amount: Integer);
var
 p0: PByteArray;
 Gray, r, g, b, x, y: Integer;
begin
 clip.PixelFormat:=pf24bit;

  for y:=0 to clip.Height-1 do begin
    p0:=clip.scanline[y];
    for x:=0 to clip.Width-1 do
    begin
      r:=p0[x*3];
      g:=p0[x*3+1];
      b:=p0[x*3+2];
      Gray:=(r+g+b)div 3;
      p0[x*3]:=IntToByte(Gray+(((r-Gray)*Amount)div 255));
      p0[x*3+1]:=IntToByte(Gray+(((g-Gray)*Amount)div 255));
      p0[x*3+2]:=IntToByte(Gray+(((b-Gray)*Amount)div 255));
    end;
  end;
end;

// 이미지의 모자이크를 추가하는 함수이다.
procedure AddMosaic(var Bm:TBitmap;size:Integer);
var
   x,y,i,j:integer;
   p1,p2:PByteArray;
   r,g,b:byte;
begin
bm.PixelFormat:=pf24Bit;
  y:=0;
  repeat
    p1:=bm.scanline[y];
    x:=0;
    repeat
      j:=1;
      repeat
      p2:=bm.scanline[y];
      x:=0;
      repeat
        r:=p1[x*3];
        g:=p1[x*3+1];
        b:=p1[x*3+2];
        i:=1;
       repeat
       p2[x*3]:=r;
       p2[x*3+1]:=g;
       p2[x*3+2]:=b;
       inc(x);
       inc(i);
       until (x>=bm.width) or (i>size);
      until x>=bm.width;
      inc(j);
      inc(y);
      until (y>=bm.height) or (j>size);
    until (y>=bm.height) or (x>=bm.width);
  until y>=bm.height;
end;

/// 이미지를 어둡게 하는 함수이다.
procedure AddDarkness(var src: tbitmap; Amount: integer);
var
p0:PByteArray;
r,g,b,x,y: Integer;
begin
  src.pixelformat:=pf24Bit;
  for y:=0 to src.Height-1 do begin
    p0:=src.scanline[y];
    for x:=0 to src.Width-1 do
    begin
      r:=p0[x*3];
      g:=p0[x*3+1];
      b:=p0[x*3+2];
      p0[x*3]:=IntToByte(r-((r)*Amount)div 255);
      p0[x*3+1]:=IntToByte(g-((g)*Amount)div 255);
      p0[x*3+2]:=IntToByte(b-((b)*Amount)div 255);
   end;
  end;
end;

/// 이미지를 밝게 하는 함수이다.
procedure AddLightness(var clip: tbitmap; Amount: Integer);
var
p0:PByteArray;
r,g,b,x,y: Integer;
begin
clip.PixelFormat:=pf24Bit;
  for y:=0 to clip.Height-1 do begin
    p0:=clip.scanline[y];
    for x:=0 to clip.Width-1 do
    begin
      r:=p0[x*3];
      g:=p0[x*3+1];
      b:=p0[x*3+2];
      p0[x*3]:=IntToByte(r+((255-r)*Amount)div 255);
      p0[x*3+1]:=IntToByte(g+((255-g)*Amount)div 255);
      p0[x*3+2]:=IntToByte(b+((255-b)*Amount)div 255);
    end;
  end;
end;

// 이미지의 모노 노이즈를 추가하는 함수이다.
procedure AddMonoNoise(var clip: tbitmap; Amount: Integer);
var
p0:PByteArray;
x,y,a,r,g,b: Integer;
begin
clip.PixelFormat:=pf24bit;
  for y:=0 to clip.Height-1 do
  begin
    p0:=clip.scanline[y];
    for x:=0 to clip.Width-1 do
    begin
      a:=Random(Amount)-(Amount shr 1);
      r:=p0[x*3]+a;
      g:=p0[x*3+1]+a;
      b:=p0[x*3+2]+a;
      p0[x*3]:=IntToByte(r);
      p0[x*3+1]:=IntToByte(g);
      p0[x*3+2]:=IntToByte(b);
    end;
  end;
end;

// 이미지의 컬러 노이즈를 추가하는 함수이다//
procedure AddColorNoise(var clip: tbitmap; Amount: Integer);
var
p0:PByteArray;
x,y,r,g,b: Integer;
begin
clip.PixelFormat:=pf24Bit;
  for y:=0 to clip.Height-1 do
  begin
    p0:=clip.ScanLine [y];
    for x:=0 to clip.Width-1 do
    begin
      r:=p0[x*3]+(Random(Amount)-(Amount shr 1));
      g:=p0[x*3+1]+(Random(Amount)-(Amount shr 1));
      b:=p0[x*3+2]+(Random(Amount)-(Amount shr 1));
      p0[x*3]:=IntToByte(r);
      p0[x*3+1]:=IntToByte(g);
      p0[x*3+2]:=IntToByte(b);
    end;
  end;
end;

procedure GetGIFSize(const sGIFFile: string; var wWidth, wHeight: Word);
type
  TGIFHeader = record
    Sig: array[0..5] of char;
    ScreenWidth, ScreenHeight: Word;
    Flags, Background, Aspect: Byte;
  end;

  TGIFImageBlock = record
    Left, Top, Width, Height: Word;
    Flags: Byte;
  end; 
var
  f: file; 
  Header: TGifHeader;
  ImageBlock: TGifImageBlock; 
  nResult: integer;
  x: integer;
  c: char; 
  DimensionsFound: boolean; 
begin
  wWidth  := 0; 
  wHeight := 0; 

  if sGifFile = '' then
    Exit; 

  {$I-} 
  FileMode := 0;   { read-only }
  AssignFile(f, sGifFile); 
  reset(f, 1);
  if IOResult <> 0 then 
    { Could not open file }
    Exit; 

  { Read header and ensure valid file. }
  BlockRead(f, Header, SizeOf(TGifHeader), nResult);
  if (nResult <> SizeOf(TGifHeader)) or (IOResult <> 0) or 
    (StrLComp('GIF', Header.Sig, 3) <> 0) then
  begin 
    { Image file invalid }
    Close(f); 
    Exit;
  end;

  { Skip color map, if there is one } 
  if (Header.Flags and $80) > 0 then
  begin 
    x := 3 * (1 shl ((Header.Flags and 7) + 1));
    Seek(f, x); 
    if IOResult <> 0 then
    begin 
      { Color map thrashed }
      Close(f);
      Exit;
    end; 
  end;

  DimensionsFound := False;
  FillChar(ImageBlock, SizeOf(TGIFImageBlock), #0); 
  { Step through blocks. }
  BlockRead(f, c, 1, nResult); 
  while (not EOF(f)) and (not DimensionsFound) do
  begin 
    case c of
      ',': { Found image } 
        begin
          BlockRead(f, ImageBlock, SizeOf(TGIFImageBlock), nResult); 
          if nResult <> SizeOf(TGIFImageBlock) then
          begin 
            { Invalid image block encountered }
            Close(f);
            Exit;
          end; 
          wWidth := ImageBlock.Width;
          wHeight := ImageBlock.Height; 
          DimensionsFound := True;
        end; 
      'y': { Skip }
        begin
          { NOP }
        end;
      { nothing else.  just ignore }
    end;
    BlockRead(f, c, 1, nResult);
  end;
  Close(f);
  {$I+}
end;

function IsGIF(var FileName: String):boolean;
var
 GIFW, GIFH: WORD;
begin
 Result := False;
 GetGIFSize(FileName, GIFW, GIFH);
 
 if (GIFW<>0) or (GIFH<>0) then
  Result := True;
end;

end.
