﻿unit uAddText;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, OpenColorCbx, Buttons, ImgList, ComCtrls,
  ToolWin, TypInfo;

type
  TfrmAddText = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    gbxAddText: TGroupBox;
    mmoText: TMemo;
    FontDlg: TFontDialog;
    chkFrame: TCheckBox;
    chkRender: TCheckBox;
    cboFontList: TComboBox;
    cboFontSize: TComboBox;
    cboCharset: TComboBox;
    FontStyleImages: TImageList;
    tbFontStyles: TToolBar;
    btnBold: TToolButton;
    btnItalic: TToolButton;
    btnUnderline: TToolButton;
    tbSeparator: TToolButton;
    gbxText: TGroupBox;
    Label1: TLabel;
    btnAlignLeft: TToolButton;
    btnAlignCenter: TToolButton;
    btnAlignRight: TToolButton;
    Label2: TLabel;
    Label3: TLabel;
    cboFrameSize: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure cboFontListChange(Sender: TObject);
    procedure cboFontSizeChange(Sender: TObject);
    procedure cboCharsetChange(Sender: TObject);
    procedure btnBoldClick(Sender: TObject);
    procedure btnItalicClick(Sender: TObject);
    procedure btnUnderlineClick(Sender: TObject);
    procedure cboFontColorChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure chkRenderClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mmoTextKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    procedure UpdateFontStyles(Sender: TObject; FontStyle: TFontStyle);
    procedure SaveINI;
    procedure LoadINI;
  public
    { Public declarations }
    FcboFrameColor: TOpenColorCbx;
    FcboBKColor: TOpenColorCbx;
    FcboFontColor: TOpenColorCbx;
  end;

var
  frmAddText: TfrmAddText;

implementation

uses uMain, OpenInterface, iniFiles;

{$R *.dfm}

procedure TfrmAddText.SaveINI;
const
  Section = '이미지의 텍스트 추가';
var
  i: Integer;
  INIFile: TINIFile;
begin
  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) +
    'OpenSetup.ini');
  with INIFile do
  try

    for i := 0 to ComponentCount - 1 do
    begin

      if (Components[i] is TComboBox) or (Components[i] is TOpenColorCbx) then
        with TComboBox(Components[i]) do
        begin
          INIFile.WriteInteger(Section, Name, ItemIndex);
          if Self.Components[i] is TOpenColorCbx then
            INIFile.WriteInteger(Section, Name + '_CustomColor',
              TOpenColorCbx(Self.Components[i]).CustomColor);
        end;

      if (Components[i] is TToolButton) = True then
        with TToolButton(Components[i]) do
          INIFile.WriteBool(Section, Name, Down);

      if (Components[i] is TCheckBox) = True then
        with TCheckBox(Components[i]) do
          INIFile.WriteBool(Section, Name, Checked);

      if (Components[i] is TMemo) = True then
        with TMemo(Components[i]) do
          INIFile.WriteString(Section, Name, StringReplace(Text, #13 + #10,
            '\n',
            [rfreplaceAll, rfIgnoreCase]));

    end;

  finally
    Free;
  end;

end;

procedure TfrmAddText.LoadINI;
const
  Section = '이미지의 텍스트 추가';
  DefaultFont = '굴림';
var
  i, j: Integer;
  INIFile: TINIFile;
begin
  INIFile := TINIFile.Create(ExtractFilePath(application.ExeName) +
    'OpenSetup.ini');
  with INIFile do
  try

    for i := 0 to ComponentCount - 1 do
    begin

      if (Components[i] is TComboBox) or (Components[i] is TOpenColorCbx) then
        with TComboBox(Components[i]) do
        begin
          if Self.Components[i] is TOpenColorCbx then
            TOpenColorCbx(Self.Components[i]).CustomColor :=
              INIFile.ReadInteger(Section, Name + '_CustomColor', clWhite);
          if INIFile.ReadInteger(Section, Name, 0) > 0 then
            ItemIndex := INIFile.ReadInteger(Section, Name, 0)
          else
            ItemIndex := 0;

          if Name = 'cboFontSize' then
            if INIFile.ReadInteger(Section, Name, -1) = -1 then
              ItemIndex := 8;

          if Name = 'cboBKColor' then
            if INIFile.ReadInteger(Section, Name, -1) = -1 then
              ItemIndex := 16;

          if (Name = 'cboFrameColor') or (Name = 'cboFontColor')
            or (Name = 'cboFrameSiaze') then
            if INIFile.ReadInteger(Section, Name, -1) = -1 then
              ItemIndex := 1;

          if Name = 'cboFontList' then
            if INIFile.ReadInteger(Section, Name, -1) = -1 then
              for j := 0 to Items.Count - 1 do
                if Items[j] = DefaultFont then
                begin
                  ItemIndex := j;
                  Break;
                end;

        end;

      if (Components[i] is TToolButton) = True then
        with TToolButton(Components[i]) do
        begin
          Down := INIFile.ReadBool(Section, Name, False);
          // OnClick(Components[i]);
        end;

      if (Components[i] is TCheckBox) = True then
        with TCheckBox(Components[i]) do
          Checked := INIFile.ReadBool(Section, Name, True);

      if (Components[i] is TMemo) = True then
        with TMemo(Components[i]) do
          Text := StringReplace(INIFile.ReadString(Section, Name, ''), '\n', #13
            + #10, [rfreplaceAll, rfIgnoreCase]);

    end;

  finally
    Free;
  end;

end;

procedure TfrmAddText.mmoTextKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // Ctrl + a 키로 전체선택되도록 설정
  if (Shift = [ssCtrl]) and (Key = 65) then
    TMemo(Sender).SelectAll;
end;

procedure TfrmAddText.UpdateFontStyles(Sender: TObject; FontStyle: TFontStyle);
var
  FontStyles: TFontStyles;
begin

  case TToolButton(Sender).Down of
    False:
      begin
        FontStyles := mmoText.Font.Style;
        ExClude(FontStyles, FontStyle);
        mmoText.Font.Style := FontStyles;
        TToolButton(Sender).Down := False;
      end;
    True:
      begin
        FontStyles := mmoText.Font.Style;
        InClude(FontStyles, FontStyle);
        mmoText.Font.Style := FontStyles;
        TToolButton(Sender).Down := True;
      end;
  end;

  // mmoText.Font.Style:=tpAddText.FontStyles;

end;

procedure TfrmAddText.FormCreate(Sender: TObject);
const
  Max_Width = 200;
var
  i: Integer;
begin
  if Self.ClassName = Sender.ClassName then
  begin
    // FormCreate가 MDI에 의해서 불릴 수도 있습니다.
    // 실제로 객체가 생성되는 경우에만 이 초기화 루틴을 실행합니다.

    FcboFrameColor := TOpenColorCbx.Create(Self);
    FcboBKColor := TOpenColorCbx.Create(Self);
    FcboFontColor := TOpenColorCbx.Create(Self);

    FcboFrameColor.Name := 'cboFrameColor';
    FcboBKColor.Name := 'cboBKColor';
    FcboFontColor.Name := 'cboFontColor';

    FcboFrameColor.Parent := gbxText;
    FcboBKColor.Parent := gbxText;
    FcboFontColor.Parent := gbxAddText;

    FcboFrameColor.Left := 88;
    FcboFrameColor.Top := 16;
    FcboFrameColor.Width := 130;
    FcboFrameColor.Height := 22;

    FcboBKColor.Left := 88;
    FcboBKColor.Top := 40;
    FcboBKColor.Width := 130;
    FcboBKColor.Height := 22;

    FcboFontColor.Left := 173;
    FcboFontColor.Top := 53;
    FcboFontColor.Width := 130;
    FcboFontColor.Height := 22;

    FcboFontColor.OnChange := cboFontColorChange;
  end;

  for i := 1 to 100 do
    cboFontSize.Items.Add(IntToStr(i));

  for i := 0 to Screen.Fonts.Count - 1 do
    cboFontList.Items.Add(Screen.Fonts[i]);

  for i := 1 to 10 do
    cboFrameSize.Items.Add(IntToStr(i));

  LoadINI;

  cboFontList.OnChange(Sender);

  SendMessage(cboFontList.Handle, CB_SETDROPPEDWIDTH, Max_Width, 0);

  cboCharset.OnChange(Sender);
  cboFontSize.OnChange(Sender);
  FcboFontColor.OnChange(Sender);

  btnBold.Click;
  btnItalic.Click;
  btnUnderline.Click;

  chkRender.OnClick(Sender);

end;

procedure TfrmAddText.FormDestroy(Sender: TObject);
begin
  FcboFontColor.Free;
  FcboBKColor.Free;
  FcboFrameColor.Free;
end;

procedure TfrmAddText.cboFontListChange(Sender: TObject);
begin
  mmoText.Font.Name := cboFontList.Text;
end;

procedure TfrmAddText.cboFontSizeChange(Sender: TObject);
var
  i: Integer;
begin

  if (TryStrToInt(cboFontSize.Text, i) = True) then
    mmoText.Font.Size := StrToInt(cboFontSize.Text);

end;

procedure TfrmAddText.cboCharsetChange(Sender: TObject);
begin

  case cboCharset.ItemIndex of
    0: mmoText.Font.Charset := HANGEUL_CHARSET;
    1: mmoText.Font.Charset := ANSI_CHARSET;
  end;

end;

procedure TfrmAddText.btnBoldClick(Sender: TObject);
begin
  UpdateFontStyles(Sender, fsBold);
end;

procedure TfrmAddText.btnItalicClick(Sender: TObject);
begin
  UpdateFontStyles(Sender, fsItalic);
end;

procedure TfrmAddText.btnUnderlineClick(Sender: TObject);
begin
  UpdateFontStyles(Sender, fsUnderline);
end;

procedure TfrmAddText.cboFontColorChange(Sender: TObject);
begin
  mmoText.Font.Color := FcboFontColor.SelectColor;
end;

procedure TfrmAddText.btnOKClick(Sender: TObject);
begin
  SaveINI;
end;

procedure TfrmAddText.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // ESC키로 창 닫기
  if Key = VK_ESCAPE then
    Close;
end;

procedure TfrmAddText.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_RETURN) and (GetFocus <> mmoText.Handle) then
  begin
    SaveINI;
    ModalResult := mrOK;
  end;

end;

procedure TfrmAddText.chkRenderClick(Sender: TObject);
begin
  if chkRender.Checked = True then
  begin
    btnAlignLeft.Enabled := False;
    btnAlignCenter.Enabled := False;
    btnAlignRight.Enabled := False;
  end
  else
  begin
    btnAlignLeft.Enabled := True;
    btnAlignCenter.Enabled := True;
    btnAlignRight.Enabled := True;
  end;

end;

end.