﻿unit uAnyAngle;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GR32_Image, GR32_RangeBars, GR32, StdCtrls, ComCtrls,
  ufrmInherited, OpenColorCbx;

const
  bWidth = 200;
  bHeight = 150;

type
  TfrmAnyAngle = class(TfrmNormal)
    btnOk: TButton;
    btnCancel: TButton;
    edtAngle: TEdit;
    udAngle: TUpDown;
    chkLinear: TCheckBox;
    Label1: TLabel;
    gbAngle: TScrollBar;
    procedure FormDestroy(Sender: TObject);
    procedure gbAngleChange(Sender: TObject);
    procedure udAngleChanging(Sender: TObject; var AllowChange: Boolean);
    procedure edtAngleChange(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnOkClick(Sender: TObject);
    procedure chkLinearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cboBackColorChange(Sender: TObject);
  private
    { Private declarations }
    FImgDst: TImage32;
    FcboBackColor: TOpenColorCbx;

    BackBuffer: TBitmap32;
    procedure ScaleRot(SrcBitmap: TBitmap32; var DstBitmap: TBitmap32;
      Alpha: Double; SoftDraw: Boolean; BackColor: TColor32);
  public
    { Public declarations }
  end;

var
  frmAnyAngle: TfrmAnyAngle;

implementation

uses uMain, uMDI, GR32_Transforms, Gr32_Resamplers, Math;

{$R *.dfm}

procedure TfrmAnyAngle.ScaleRot(SrcBitmap: TBitmap32; var DstBitmap: TBitmap32;
  Alpha: Double; SoftDraw: Boolean; BackColor: TColor32);
var
  Angle: Extended;
  CosTheta: Extended;
  SinTheta: Extended;
  SrcR: Integer;
  SrcB: Integer;
  T: TAffineTransformation;
  Sx, Sy: Single;
begin
  Angle := Alpha;

  SrcBitmap.CombineMode := cmBlend;
  SrcBitmap.DrawMode := dmBlend;
  SrcBitmap.OuterColor := clTrWhite32;
  SrcBitmap.ResetAlpha;
  SrcR := SrcBitmap.Width - 1;
  SrcB := SrcBitmap.Height - 1;

  if SoftDraw then begin
    TKernelResampler.Create(SrcBitmap);
    TKernelResampler(SrcBitmap.Resampler).Kernel := TLinearKernel.Create
  end else
    TNearestResampler.Create(SrcBitmap);

  T := TAffineTransformation.Create;
  T.SrcRect := FloatRect(0, 0, SrcR + 1, SrcB + 1);
  try
    T.Clear;

    T.Translate(-SrcR / 2, -SrcB / 2);
    T.Rotate(0, 0, Alpha);
    Alpha := Alpha * 3.14159265358979 / 180;

    Sx := Abs(SrcR * Cos(Alpha)) + Abs(SrcB * Sin(Alpha));
    Sy := Abs(SrcR * Sin(Alpha)) + Abs(SrcB * Cos(Alpha));

    Sx := SrcBitmap.Width / Sx;
    Sy := SrcBitmap.Height / Sy;

    while Angle >= 360 do
      Angle := Angle - 360;
    while Angle < 0 do
      Angle := Angle + 360;

    SinCos(Angle * Pi / 180, SinTheta, CosTheta);

    if (SinTheta * CosTheta) < 0 then
    begin
      DstBitmap.Width := Round(Abs(SrcBitmap.Width * CosTheta - SrcBitmap.Height
        * SinTheta));
      DstBitmap.Height := Round(Abs(SrcBitmap.Width * SinTheta - SrcBitmap.Height
        * CosTheta));
    end
    else
    begin
      DstBitmap.Width := Round(Abs(SrcBitmap.Width * CosTheta + SrcBitmap.Height
        * SinTheta));
      DstBitmap.Height := Round(Abs(SrcBitmap.Width * SinTheta + SrcBitmap.Height
        * CosTheta));
    end;

    T.Translate(DstBitmap.Width / 2, DstBitmap.Height / 2);

    DstBitmap.Clear(BackColor);
    Transform(DstBitmap, SrcBitmap, T);

  finally
    T.Free;
  end;

end;

procedure TfrmAnyAngle.FormDestroy(Sender: TObject);
begin
  BackBuffer.Free;

  FcboBackColor.Free;
  FImgDst.Free;
end;

procedure TfrmAnyAngle.gbAngleChange(Sender: TObject);
var
  DstBitmap: TBitmap32;
begin
  // ** 상속 폼에서 Creat 보다 이 이벤트가 먼저 실행되므로 ** //
  if BackBuffer = nil then
    Exit;

  edtAngle.Text := IntToStr(gbAngle.Position);

  FImgDst.BeginUpdate;
  try
    DstBitmap := FImgDst.Bitmap;

    ScaleRot(BackBuffer, DstBitmap,
      -gbAngle.Position, chkLinear.Checked,
      Color32(FcboBackColor.SelectColor));

    with FImgDst.Bitmap.Canvas do
    begin
      Pen.Width := 1;
      Pen.Color := clGray;
      Pen.Style := psDot;
      Brush.Style := bsClear;
      Rectangle(0, 0, FimgDst.Bitmap.Width, FimgDst.Bitmap.Height);
    end;

  finally
    FimgDst.EndUpdate;
    FimgDst.Repaint;
  end;

end;

procedure TfrmAnyAngle.udAngleChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  gbAngle.Position := udAngle.Position;
end;

procedure TfrmAnyAngle.edtAngleChange(Sender: TObject);
begin
  gbAngle.Position := udAngle.Position;
end;

procedure TfrmAnyAngle.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
    btnOK.OnClick(Sender);
end;

procedure TfrmAnyAngle.btnOkClick(Sender: TObject);
var
  SrcBitmap, DstBitmap: TBitmap32;
begin
  SrcBitmap := TBitmap32.Create;
  DstBitmap := TBitmap32.Create;
  try
    with TfrmMDI(frmMain.ActiveMDIChild) do
    begin
      if FRBLayer = nil then
        SrcBitmap.Assign(FAImage.Bitmap)
      else
        SrcBitmap.Assign(FBitmapLayer.Bitmap);

      SrcBitmap.ResetAlpha;

      ScaleRot(SrcBitmap, DstBitmap,
        -gbAngle.Position, chkLinear.Checked,
        Color32(FcboBackColor.SelectColor));

      if FRBLayer = nil then
      begin
        SaveUndoImage(FAImage.Bitmap);
        FAImage.Bitmap.Assign(DstBitmap)
      end
      else
        with FRBLayer.Location do
        begin
          FlattenLayer(True, False);

          FBitmapLayer.Bitmap.Width := DstBitmap.Width;
          FBitmapLayer.Bitmap.Height := DstBitmap.Height;

          FBitmapLayer.Bitmap.Draw(0, 0, DstBitmap);

          FRBLayer.Location := FloatRect(Left, Top, Left + DstBitmap.Width, Top
            + DstBitmap.Height);
        end;
    end;
  finally
    SrcBitmap.Free;
    DstBitmap.Free;

    frmMain.cboZoom.OnChange(Sender);
  end;

end;

procedure TfrmAnyAngle.chkLinearClick(Sender: TObject);
begin
  gbAngle.OnChange(Sender);
end;

procedure TfrmAnyAngle.FormCreate(Sender: TObject);
var
  SrcBitmap: TBitmap32;
  DstRect: TRect;
  SrcRect: TRect;
begin
  FImgDst := TImage32.Create(Self);
  FcboBackColor := TOpenColorCbx.Create(Self);

  FImgDst.Parent := Self;
  FcboBackColor.Parent := Self;

  FImgDst.SetBounds(16, 16, 305, 257);
  FcboBackColor.SetBounds(66, 332, 145, 22);

  FcboBackColor.OnChange := cboBackColorChange;

  BackBuffer := TBitmap32.Create;
  BackBuffer.SetSize(bWidth, bHeight);
  BackBuffer.Clear(clWhite32);

  with TfrmMDI(frmMain.ActiveMDIChild) do
  begin
    DstRect := BackBuffer.BoundsRect;

    if FRBLayer = nil then
      SrcRect := FAImage.Bitmap.BoundsRect
    else
      with FRBLayer.Location do
        SrcRect := Rect(0, 0, Round(Right - Left), Round(Bottom - Top));

    IntersectRect(DstRect, SrcRect, DstRect);

    BackBuffer.SetSize(DstRect.Right, DstRect.Bottom);

    if FRBLayer = nil then
      SrcBitmap := FAImage.Bitmap
    else
      SrcBitmap := FBitmapLayer.Bitmap;

    TKernelResampler.Create(SrcBitmap);
    TKernelResampler(SrcBitmap.Resampler).Kernel := TLinearKernel.Create;
    SrcBitmap.DrawTo(BackBuffer, DstRect);
  end;

  FImgDst.Bitmap.Assign(BackBuffer);

  gbAngle.OnChange(Sender);

end;

procedure TfrmAnyAngle.cboBackColorChange(Sender: TObject);
begin
  gbAngle.OnChange(nil);
end;

end.