﻿unit uColorCapture;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls , ufrmInherited, GR32_RangeBars, ComCtrls,
  GR32_Image, Buttons;

type
  TfrmColorCapture = class(TForm)
    btnOK: TButton;
    ColorDlg: TColorDialog;
    gbxRGB: TGroupBox;
    lblRed: TLabel;
    edtRed: TEdit;
    edtGreen: TEdit;
    lblGreen: TLabel;
    edtBlue: TEdit;
    lblBlue: TLabel;
    udRed: TUpDown;
    udGreen: TUpDown;
    udBlue: TUpDown;
    pnlColor: TPanel;
    shpCaptureColor: TShape;
    shpCustomColor: TShape;
    gbxHSL: TGroupBox;
    lblH: TLabel;
    lblS: TLabel;
    lblL: TLabel;
    edtH: TEdit;
    edtS: TEdit;
    edtL: TEdit;
    udH: TUpDown;
    udS: TUpDown;
    udL: TUpDown;
    lblDelphi: TLabel;
    lblWeb: TLabel;
    lblRGB: TLabel;
    edtRGB: TEdit;
    edtWeb: TEdit;
    edtDelphi: TEdit;
    shpEdge: TShape;
    btnCPlusCopy: TSpeedButton;
    btnWebCopy: TSpeedButton;
    btnDelphiCopy: TSpeedButton;
    gbBlue: TScrollBar;
    gbGreen: TScrollBar;
    gbRed: TScrollBar;
    gbH: TScrollBar;
    gbS: TScrollBar;
    gbL: TScrollBar;
    procedure shpCaptureColorMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure gbRedChange(Sender: TObject);
    procedure gbGreenChange(Sender: TObject);
    procedure gbBlueChange(Sender: TObject);
    procedure edtRedChange(Sender: TObject);
    procedure edtGreenChange(Sender: TObject);
    procedure edtBlueChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure gbHChange(Sender: TObject);
    procedure gbSChange(Sender: TObject);
    procedure gbLChange(Sender: TObject);
    procedure edtHChange(Sender: TObject);
    procedure edtSChange(Sender: TObject);
    procedure edtLChange(Sender: TObject);
    procedure btnCPlusCopyClick(Sender: TObject);
    procedure btnWebCopyClick(Sender: TObject);
    procedure btnDelphiCopyClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FImg32: TImage32;

    FCreate: Boolean;
    FChanging: Boolean;
  public
    { Public declarations }
    procedure UpdateColor(AColor: TColor; gbRGB, gbHSL: Boolean);
    procedure ChangeColor(IsRGB: Boolean);
  end;

var
  frmColorCapture: TfrmColorCapture;

implementation

 uses GR32, OpenInterface, OpenDrawShape, HSLUnit;

{$R *.dfm}

procedure TfrmColorCapture.UpdateColor(AColor: TColor; gbRGB, gbHSL: Boolean);
var
 HSLColor: THSLColor;
 RGB: DWORD;
 R, G, B: Byte;
begin
 FChanging := True;

 shpCustomColor.Brush.Color := AColor;

 RGB := ColorToRGB(AColor);

 R := GetRValue(RGB);
 G := GetGValue(RGB);
 B := GetBValue(RGB);

 edtWeb.Text :=
 Format('#%.2x%.2x%.2x', [R, G, B]);

 edtRGB.Text :=
 Format('0x%x', [AColor]);

 edtDelphi.Text := ColorToString(AColor);

 if gbRGB = True then begin
  gbRed.Position := R;
  edtRed.Text := IntToStr(gbRed.Position);
  gbGreen.Position := G;
  edtGreen.Text := IntToStr(gbGreen.Position);
  gbBlue.Position := B;
  edtBlue.Text := IntToStr(gbBlue.Position);
 end;

 if gbHSL = True then begin
  HSLColor := ColorToHSL(AColor);

  gbH.Position := HSLColor.H;
  edtH.Text := IntToStr(gbH.Position);
  gbS.Position := HSLColor.S;
  edtS.Text := IntToStr(gbS.Position);
  gbL.Position := HSLColor.L;
  edtL.Text := IntToStr(gbL.Position);
 end;

 FChanging := False;
end;

procedure TfrmColorCapture.ChangeColor(IsRGB: Boolean);
var
 AColor: TColor;
 HSLColor: THSLColor;
 R, G, B: Byte;
 H, S, L: Integer;
 gbRGB, gbHSL: Boolean;
begin
 FChanging := True;

 if IsRGB = True then begin
  R := gbRed.Position;
  G := gbGreen.Position;
  B := gbBlue.Position;

  AColor := RGB(R, G, B);

  gbRGB := False;
  gbHSL := True;
  end else begin
            H := gbH.Position;
            S := gbS.Position;
            L := gbL.Position;

            HSLColor := HSL(H, S, L);
            AColor := HSLToColor(HSLColor);

            gbRGB := True;
            gbHSL := False;
           end;

 UpdateColor(AColor, gbRGB, gbHSL);

 FChanging := False;
end;

procedure TfrmColorCapture.shpCaptureColorMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 ColorDlg.Color := shpCaptureColor.Brush.Color;
 
 if ColorDlg.Execute = True then
  UpdateColor(ColorDlg.Color, True, True);
end;

procedure TfrmColorCapture.FormCreate(Sender: TObject);
var
 i, Y: Integer;
begin
 FImg32 := TImage32.Create(Self);

 FImg32.Name := 'Img32';

 FImg32.Parent := Self;
 FImg32.SetBounds(68, 26, 41, 41);

 Font.Assign(Screen.MenuFont);

 for i := 0 to ComponentCount-1 do
  if Components[i] is TLabel then
   SetFontAntiAlias(TLabel(Components[i]).Font)
    else if Components[i] is TEdit then
          SetFontAntiAlias(TEdit(Components[i]).Font);

 FCreate := True;                     

 pnlColor.DoubleBuffered := True;

 FImg32.Bitmap.SetSize(FImg32.Width, FImg32.Height);
 FImg32.Bitmap.Clear(clWhite32);

 FImg32.Bitmap.Canvas.Pen.Width := 5;

 Y := FImg32.Height div 2;

 open_Arrow(FImg32.Bitmap, Point(5, Y), Point(FImg32.Width - 5, Y), 255, True);
end;

procedure TfrmColorCapture.FormDestroy(Sender: TObject);
begin
 FImg32.Free;
end;

procedure TfrmColorCapture.gbRedChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtRed.Text := IntToStr(gbRed.Position);
 ChangeColor(True);
end;

procedure TfrmColorCapture.gbGreenChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtGreen.Text := IntToStr(gbGreen.Position);
 ChangeColor(True);
end;

procedure TfrmColorCapture.gbBlueChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtBlue.Text := IntToStr(gbBlue.Position);
 ChangeColor(True);
end;

procedure TfrmColorCapture.edtRedChange(Sender: TObject);
begin
 gbRed.Position := udRed.Position;
end;

procedure TfrmColorCapture.edtGreenChange(Sender: TObject);
begin
 gbGreen.Position := udGreen.Position;
end;

procedure TfrmColorCapture.edtBlueChange(Sender: TObject);
begin
 gbBlue.Position := udBlue.Position;
end;

procedure TfrmColorCapture.FormActivate(Sender: TObject);
begin
 if FCreate = True then begin
  FChanging := False;

  UpdateColor(shpCaptureColor.Brush.Color, True, True);

  FCreate := False;
 end;

end;

procedure TfrmColorCapture.gbHChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtH.Text := IntToStr(gbH.Position);
 ChangeColor(False);
end;

procedure TfrmColorCapture.gbSChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtS.Text := IntToStr(gbS.Position);
 ChangeColor(False);
end;

procedure TfrmColorCapture.gbLChange(Sender: TObject);
begin
 if FChanging = True then
  Exit;

 edtL.Text := IntToStr(gbL.Position);
 ChangeColor(False);
end;

procedure TfrmColorCapture.edtHChange(Sender: TObject);
begin
 gbH.Position := udH.Position;
end;

procedure TfrmColorCapture.edtSChange(Sender: TObject);
begin
 gbS.Position := udS.Position;
end;

procedure TfrmColorCapture.edtLChange(Sender: TObject);
begin
 gbL.Position := udL.Position;
end;

procedure TfrmColorCapture.btnCPlusCopyClick(Sender: TObject);
begin
 edtRGB.SelectAll;
 edtRGB.CopyToClipboard;
 edtRGB.SelStart := 0;
end;

procedure TfrmColorCapture.btnWebCopyClick(Sender: TObject);
begin
 edtWeb.SelectAll;
 edtWeb.CopyToClipboard;
 edtWeb.SelStart := 0;
end;

procedure TfrmColorCapture.btnDelphiCopyClick(Sender: TObject);
begin
 edtDelphi.SelectAll;
 edtDelphi.CopyToClipboard;
 edtDelphi.SelStart := 0;
end;

end.
