﻿unit uCommonDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, GR32_RangeBars;

const
  CM_Effect = WM_USER + $1247;
  ModeColorNoise = 0;
  ModeMonoNoise = 1;
  ModeMosaic = 2;
  ModeBrightness = 3;
  ModeSaturation = 4;

type
  TfrmCommonDlg = class(TForm)
    grpbx: TGroupBox;
    imgDst: TImage;
    Bevel1: TBevel;
    lblUnit: TLabel;
    edtPosition: TEdit;
    btnOk: TButton;
    btnCancel: TButton;
    TrackBar: TScrollBar;
    procedure FormCreate(Sender: TObject);
    procedure TrackBarChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
  private
    { Private declarations }
    EffectMode: Byte;
    ABitmap: TBitmap;
    procedure ColorNoise(var Message: TMessage); message CM_Effect;
  public
    { Public declarations }
  end;

var
  frmCommonDlg: TfrmCommonDlg;
implementation

uses uMDI, uMain, GR32, GR32_Transforms, GR32_Resamplers, libImageEffect,
  OpenInterface;

{$R *.dfm}

procedure TfrmCommonDlg.ColorNoise(var Message: TMessage);

  procedure SetimgDstSize(ScrBitmap: TBitmap32);
  begin
    if ScrBitmap.BoundsRect.Right < imgDst.ClientWidth then
      imgDst.Width := ScrBitmap.BoundsRect.Right;
    if ScrBitmap.BoundsRect.Bottom < imgDst.ClientHeight then
      imgDst.Height := ScrBitmap.BoundsRect.Bottom;
  end;

var
  BufferBitmap: TBitmap32;
begin
  Message.Result := Integer(True);

  case Message.LParam of
    ModeColorNoise:
      begin
        Caption := '이미지의 노이즈 추가(Color)';
        grpbx.Caption := '이미지의 노이즈를 추가합니다. (Color)';
      end;
    ModeMonoNoise:
      begin
        Caption := '이미지의 노이즈 추가(Mono)';
        grpbx.Caption := '이미지의 노이즈를 추가합니다. (Mono)';
      end;
    ModeMosaic:
      begin
        Caption := '이미지의 모자이크 추가';
        grpbx.Caption := '이미지의 모자이크 추가를 추가합니다.';
      end;
    ModeBrightness:
      begin
        Caption := '이미지의 밝기 조절';
        grpbx.Caption := '이미지의 밝기를 조절 할 수 있습니다.';
        //lblUnit.Caption:='&+';
        TrackBar.Min := -250;
        TrackBar.Max := 250;
        edtPosition.Text := IntToStr(0);
        edtPosition.Left := edtPosition.Left + 10;
        lblUnit.Visible := False;
      end;

    ModeSaturation:
      begin
        Caption := '이미지의 채도 조절';
        grpbx.Caption := '이미지의 채도를 조절 할 수 있습니다.';
        //lblUnit.Caption:='&+';
        TrackBar.Min := -100;
        TrackBar.Max := 100;
        edtPosition.Text := IntToStr(0);
        edtPosition.Left := edtPosition.Left + 10;
        lblUnit.Visible := False;
      end;

  end;

  with TfrmMDI(frmMain.ActiveMDIChild) do
  try

    BufferBitmap := TBitmap32.Create;
    try
      with imgDst do
        BufferBitmap.SetSize(ClientWidth, ClientHeight);

      with imgDst do
        if FRBLayer = nil then
        begin
          SetimgDstSize(FAImage.Bitmap);

          TKernelResampler.Create(FAImage.Bitmap);
          TKernelResampler(FAImage.Bitmap.Resampler).Kernel := TLinearKernel.Create;
          FAImage.Bitmap.DrawTo(BufferBitmap, BufferBitmap.BoundsRect);
        end
        else
        begin
          SetimgDstSize(FBitmapLayer.Bitmap);

          TKernelResampler.Create(FBitmapLayer.Bitmap);
          TKernelResampler(FBitmapLayer.Bitmap.Resampler).Kernel := TLinearKernel.Create;
          FBitmapLayer.Bitmap.DrawTo(BufferBitmap, BufferBitmap.BoundsRect);
        end;

      ABitmap.Assign(BufferBitmap);
    finally
      BufferBitmap.Free;
    end;

    imgDst.Picture.Bitmap.Assign(ABitmap);

    if Message.LParam = ModeBrightness then
      TrackBar.Position := 0;

    EffectMode := Message.LParam;

  except
    Message.Result := Integer(False);
    Application.MessageBox(PChar(Caption), PChar('실패'), MB_OK +
      MB_ICONASTERISK + MB_DEFBUTTON1 + MB_APPLMODAL);
  end;

end;

procedure TfrmCommonDlg.FormCreate(Sender: TObject);
begin
  // ** 시스템 폰트 ** //
  Font.Assign(Screen.MenuFont);

  with imgDst do
  begin
    Picture.Bitmap.Width := imgDst.Width;
    Picture.Bitmap.Height := imgDst.Height;
  end;

  ABitmap := TBitmap.Create;

end;

procedure TfrmCommonDlg.TrackBarChange(Sender: TObject);
var
  DelphiBitmap: TBitmap;
begin
  DelphiBitmap := TBitmap.Create;
  try

    DelphiBitmap.Assign(ABitmap);

    case EffectMode of
      ModeColorNoise: AddColorNoise(DelphiBitmap, TrackBar.Position);
      ModeMonoNoise: AddMonoNoise(DelphiBitmap, TrackBar.Position);
      ModeMosaic: AddMosaic(DelphiBitmap, TrackBar.Position);
      ModeBrightness:
        begin
          if TrackBar.Position < 0 then
            AddDarkness(DelphiBitmap, -TrackBar.Position)
          else if TrackBar.Position > 0 then
            AddLightness(DelphiBitmap, TrackBar.Position);
        end;
      ModeSaturation: AddSaturation(DelphiBitmap, TrackBar.Position);
    end;

    imgDst.Picture.Bitmap.Assign(DelphiBitmap);

  finally
    DelphiBitmap.Free;
  end;

  edtPosition.Text := IntToStr(TrackBar.Position);

end;

procedure TfrmCommonDlg.FormDestroy(Sender: TObject);
begin
  ABitmap.Free;
end;

procedure TfrmCommonDlg.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmCommonDlg.btnOkClick(Sender: TObject);
var
  DelphiBitmap: TBitmap;
begin
  if frmMain.MDIChildCount > 0 then
    with TfrmMDI(frmMain.ActiveMDIChild) do
    try
      BeginCursor;
      DelphiBitmap := TBitmap.Create;
      try
        if FRBLayer = nil then
        begin
          DelphiBitmap.Assign(FAImage.Bitmap);
          SaveUndoImage(FAImage.Bitmap);
        end
        else
        begin
          FlattenLayer(True, False);
          DelphiBitmap.Assign(FBitmapLayer.Bitmap);
        end;

        case EffectMode of
          ModeColorNoise: AddColorNoise(DelphiBitmap, TrackBar.Position);
          ModeMonoNoise: AddMonoNoise(DelphiBitmap, TrackBar.Position);
          ModeMosaic: AddMosaic(DelphiBitmap, TrackBar.Position);
          ModeBrightness:
            begin
              if TrackBar.Position < 0 then
                AddDarkness(DelphiBitmap, -TrackBar.Position)
              else if TrackBar.Position > 0 then
                AddLightness(DelphiBitmap, TrackBar.Position);
            end;
          ModeSaturation: AddSaturation(DelphiBitmap, TrackBar.Position);

        end;

        if FRBLayer = nil then
          FAImage.Bitmap.Assign(DelphiBitmap)
        else
          FBitmapLayer.Bitmap.Assign(DelphiBitmap);

      finally
        DelphiBitmap.Free;
        // ** with TfrmMDI(frmMain.ActiveMDIChild) do 이므로 그냥 Close 하면
        // MDI 폼이 닫힌다 ** //
        Self.Close;
        EndCursor;
      end;
    except
      Application.MessageBox(PChar(Caption), PChar('실패'), MB_OK +
        MB_ICONASTERISK + MB_DEFBUTTON1 + MB_APPLMODAL);
      frmMain.mnuUndo.OnClick(Sender);
    end;

end;

end.