﻿unit uMDI;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus, ImgList, Math, Buttons,
  iniFiles, uBitmapList, GR32_Image, GR32_Layers, GR32_LowLevel, GR32,
  GR32_RangeBars, GR32_Polygons, uAddText, XPMenu,
  ufrmInherited;

const
  CM_ResizeImage = WM_USER + $1000;
  CM_CaptureStart = WM_USER + $1001;

  stPen = 1;
  stLine = stPen;
  stRectangle = 3;
  stEllipse = 4;
  stRoundRect = 5;
  stArrow = 6;

  Layer_Normal = 0;
  Layer_DeleteRgn = 1;

  CUR_DRAG = 14;
  CUR_DRAG_MOVE = 15;

type
  TfrmMDI = class(TfrmNormal)
    tmrBrush: TTimer;
    tmrAutoScroll: TTimer;
    PopupMenu: TPopupMenu;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure CloseWndClick(Sender: TObject);
    procedure UndoPopupMenuClick(Sender: TObject);
    procedure RedoPopupMenuClick(Sender: TObject);
    procedure tmrBrushTimer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ScrollBoxClick(Sender: TObject);
    procedure tmrAutoScrollTimer(Sender: TObject);
    procedure AImagePaintStage(Sender: TObject; Buffer: TBitmap32;
      StageNum: Cardinal);
    procedure AImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure AImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure AImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer; Layer: TCustomLayer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure AImageDblClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
  private
    { Private declarations }
    FXPMenu: TXPMenu;

    procedure CaptureStart(var Message: TMessage); message CM_CaptureStart;
    function UpdateTabRefresh(MDIIndex: Integer; MDICaption: string): boolean;
    function SetZoomPoint(ZoomPoint: TPoint): TPoint;
    procedure UpdateINIFile(bReadINI: Boolean);
    procedure AddMDITab(TabName: string; var flags: Integer);
    procedure ADrawText(OldPt, nPt: TPoint);
    procedure ADrawBufferText(Render: Boolean = True);
    procedure ImageAutoScroll(Bool: Boolean);
    // ********************** //
    procedure DrawShape(TooSel: Byte; OldPt, nPt: TPoint; UseWinGDI: Boolean;
      Antialiased: Boolean = False);
    procedure DeleteShape(TooSel: Byte; OldPt, nPt: TPoint);
    // ********************** //
  public
    { Public declarations }
    FAImage: TImgView32;
    FfrmAddText: TfrmAddText;

    ToolSel: 0..13;
    FRBLayer: TRubberbandLayer;
    FBitmapLayer: TBitmapLayer;

    OldPt, nPt: TPoint;
    FDown: Boolean;
    FDraw: Boolean;
    FDoubleClick: Boolean;
    FLayerDown: Boolean;
    FLayerMove: Boolean;
    FLayerBuffer: TBitmap32;
    EraserBitmap: TBitmap32;

    // ** Undo, Redo ** //
    uBitmapList: TBitmapList;
    rBitmapList: TBitmapList;
    // ***************************************** //
    // ** SaveUndoImage ** //
    // **이미지 되돌리기를 위해서 변경되기 전 미리저장한다. ** //
    // ** frmMain 에서도 사용하므로 private 에 놓지 않았다. ** //
    procedure SaveUndoImage(ABitmap: TBitmap32);
    procedure EditingImage;
    // ***************************************** //
    function GetVertScroll: TCustomRangeBar;
    function GetHorzScroll: TCustomRangeBar;
    procedure FlattenLayer(Save, DeleteLayer: Boolean);
    procedure RBResizing(Sender: TObject;
      const OldLocation: TFloatRect; var NewLocation: TFloatRect;
      DragState: TDragState; Shift: TShiftState);
    function CreateBitmapLayer: TBitmapLayer;
    function IsDraw: Boolean;
  end;

var
  frmMDI: TfrmMDI;

implementation

uses uMain, uOption, OpenDrawShape, OpenInterface;

{$R *.dfm}

type
  TXX = class(TCustomImgView32);

function TfrmMDI.GetVertScroll: TCustomRangeBar;
begin
  Result := TXX(FAImage).VScroll;
end;

function TfrmMDI.GetHorzScroll: TCustomRangeBar;
begin
  Result := TXX(FAImage).HScroll;
end;

procedure TfrmMDI.DrawShape(TooSel: Byte; OldPt, nPt: TPoint; UseWinGDI:
  Boolean; Antialiased: Boolean = False);
var
  Alpha: Byte;
begin
  Alpha := Round((255 / 100) * frmMain.gbShapeAlpha.Position);

  case TooSel of
    tsPen, tsEraser, tsLine: open_Line(FAImage.Bitmap, OldPt, nPt, Alpha,
        Antialiased);
    tsRect: open_Rectangle(FAImage.Bitmap,
        OldPt, nPt,
        Antialiased,
        Alpha,
        uBitmapList.Bitmaps[uBitmapList.GetCount - 1]);
    tsEllipse: open_Ellipse(FAImage.Bitmap,
        OldPt, nPt,
        Antialiased,
        Alpha,
        uBitmapList.Bitmaps[uBitmapList.GetCount - 1]);
    tsRoundRect: open_RoundRectangle(FAImage.Bitmap,
        OldPt, nPt,
        Antialiased,
        Alpha,
        uBitmapList.Bitmaps[uBitmapList.GetCount - 1]);
    tsArrow: open_Arrow(FAImage.Bitmap, OldPt, nPt, Alpha, Antialiased);
  end;

end;

procedure TfrmMDI.DeleteShape(TooSel: Byte; OldPt, nPt: TPoint);
var
  DstRect: TRect;
  Margin: Integer;
begin
  if (EqualPoint(OldPt, nPt) = False) then
  begin

    NormalizePoint(OldPt, nPt);

    if ToolSel = tsArrow then
      Margin := 65
    else
      Margin := 15;

    Dec(OldPt.X, Margin);
    Dec(OldPt.Y, Margin);
    Inc(nPt.X, Margin);
    Inc(nPt.Y, Margin);

    DstRect := Rect(OldPt.X, OldPt.Y, nPt.X, nPt.Y);

    with uBitmapList.Bitmaps[uBitmapList.GetCount - 1] do
    begin
      MinMaxRect(0, Width, 0, Height, DstRect);
      uBitmapList.Bitmaps[uBitmapList.GetCount - 1].DrawTo(FAImage.Bitmap,
        DstRect, DstRect);
    end;

  end;

end;

procedure TfrmMDI.ImageAutoScroll(Bool: Boolean);
const
  SIZE_GRIP: array[Boolean] of TSizeGripStyle = (sgAuto, sgNone);
begin
  if (ToolSel in [tsPen, tsAirBrush, tsEraser, tsLine, tsEllipse, tsRect,
    tsRoundRect, tsText, tsArrow, tsSelect]) then
  begin
    tmrAutoScroll.Enabled := Bool;
    FAImage.SizeGrip := SIZE_GRIP[Bool];
  end;
end;

function TfrmMDI.CreateBitmapLayer: TBitmapLayer;
begin
  with frmMain do
  begin
    mnuCut.Enabled := True;
    mnuCrop.Enabled := True;
    mnuDelete.Enabled := True;
    btnSelect.Down := True;
    ChangePaintTool(tsSelect);
  end;

  FBitmapLayer := TBitmapLayer.Create(FAImage.Layers);

  with FBitmapLayer do
  begin
    Scaled := True;
    Bitmap.DrawMode := dmBlend;
    Bitmap.MasterAlpha := Round((255 / 100) * frmMain.gbLayerAlpha.Position);
    frmMain.lblLayerAlpha.Caption := '투명도 : ' +
      IntToStr(frmMain.gbLayerAlpha.Position) + '%';
    Tag := Layer_Normal;
  end;

  FRBLayer := TRubberbandLayer.Create(FAImage.Layers);

  with FRBLayer do
  begin
    MinWidth := 1;
    MinHeight := 1;
    ChildLayer := FBitmapLayer;
    OnResizing := RBResizing;
  end;

  frmMain.UpdatePaintPanel;

  Result := FBitmapLayer;
end;

function TfrmMDI.IsDraw: Boolean;
begin
  // ** 현재 그리기 중인지 ** //
  Result := FDown;
end;

procedure TfrmMDI.RBResizing(Sender: TObject;
  const OldLocation: TFloatRect; var NewLocation: TFloatRect;
  DragState: TDragState; Shift: TShiftState);

  procedure LayerResizing(const OldLocation: TFloatRect; var NewLocation:
    TFloatRect; DragState: TDragState);
  begin
    case DragState of
      dsMove:
        begin
          NewLocation.Left := Round(NewLocation.Left);
          NewLocation.Top := Round(NewLocation.Top);
          NewLocation.Right := Round(NewLocation.Left + (OldLocation.Right -
            OldLocation.Left));
          NewLocation.Bottom := Round(NewLocation.Top + (OldLocation.Bottom -
            OldLocation.Top));
        end;
      dsSizeL: NewLocation.Left := Round(NewLocation.Left);
      dsSizeT: NewLocation.Top := Round(NewLocation.Top);
      dsSizeR: NewLocation.Right := Round(NewLocation.Right);
      dsSizeB: NewLocation.Bottom := Round(NewLocation.Bottom);
      dsSizeTL:
        begin
          NewLocation.Top := Round(NewLocation.Top);
          NewLocation.Left := Round(NewLocation.Left);
        end;
      dsSizeTR:
        begin
          NewLocation.Top := Round(NewLocation.Top);
          NewLocation.Right := Round(NewLocation.Right);
        end;
      dsSizeBL:
        begin
          NewLocation.Left := Round(NewLocation.Left);
          NewLocation.Bottom := Round(NewLocation.Bottom);
        end;
      dsSizeBR:
        begin
          NewLocation.Right := Round(NewLocation.Right);
          NewLocation.Bottom := Round(NewLocation.Bottom);
        end;
    end;
  end;

begin

  LayerResizing(OldLocation, NewLocation, DragState);

  case Ord(DragState) of
    Ord(dsMove):
      begin
        if FLayerMove = False then
          FLayerMove := True;
        case ToolSel of
          tsSelect: if FBitmapLayer.Tag = Layer_DeleteRgn then
              with FAImage.Bitmap.Canvas do
              begin
                FlattenLayer(True, False);
                Brush.Style := bsSolid;
                Brush.Color := frmMain.FPaletteBrush.Color;
                with OldLocation do
                  FillRect(Rect(Round(Left), Round(Top), Round(Right),
                    Round(Bottom)));
                FBitmapLayer.Tag := Layer_Normal;
              end;
          tsText:
            begin
              if FLayerDown = False then
                FLayerDown := True;

              FRBLayer.Location := NewLocation;
              ADrawBufferText(False);
            end;
        end; // case ToolSel of
        with FRBLayer.Location do
          frmMain.StatusBar.Panels[1].Text :=
            Format('레이어 좌표 : (%d ,  %d ,  %d ,  %d)',
            [Round(Left), Round(Top), Round(Right), Round(Bottom)]);
      end;
    Ord(dsSizeL)..
      Ord(dsSizeBR):
      begin
        case ToolSel of
          tsSelect: if FLayerDown = False then
              with FBitmapLayer do
              begin
                FLayerBuffer := TBitmap32.Create;
                FAImage.Bitmap.Draw(Round(OldLocation.Left),
                  Round(OldLocation.Top), Bitmap);
                FLayerBuffer.Assign(FAImage.Bitmap);
                Bitmap.Assign(nil);
                FLayerDown := True;
              end;
          tsText:
            begin
              if FLayerDown = False then
                FLayerDown := True;

              FRBLayer.Location := NewLocation;
              ADrawBufferText(False);
            end;
        end; // case ToolSel of

        with FRBLayer.Location do
          frmMain.StatusBar.Panels[1].Text :=
            Format('레이어 좌표 : (%d ,  %d ,  %d ,  %d) ,  레이어 크기 : %d x %d',
            [Round(Left), Round(Top), Round(Right), Round(Bottom), Round(Right -
              Left), Round(Bottom - Top)]);
      end;
  end;

end;

procedure TfrmMDI.FlattenLayer(Save, DeleteLayer: Boolean);
var
  BufferBitmap: TBitmap32;
begin
  if FfrmAddText <> nil then
  begin
    FreeAndNil(FfrmAddText);

    if Save = True then
    begin
      frmMain.ChangePaintTool(tsText);
      frmMain.btnSelect.Down := True;
      ToolSel := tsSelect;
    end;

  end;

  if FRBLayer = nil then
  begin
    if Save = True then
      SaveUndoImage(FAImage.Bitmap);

  end
  else
  begin // == if RBLayer <> nil then
    BufferBitmap := TBitmap32.Create;
    FAImage.BeginUpdate;
    try
      BufferBitmap.SetSize(FAImage.Bitmap.Width, FAImage.Bitmap.Height);
      BufferBitmap.Draw(0, 0, FAImage.Bitmap);
      with FRBLayer.Location do
        BufferBitmap.Draw(Round(Left), Round(Top), FBitmapLayer.Bitmap);

      if Save = True then
        SaveUndoImage(BufferBitmap);

      if DeleteLayer = True then
      begin
        FRBLayer.ChildLayer := nil;
        // ** 자동으로 RBLayer ...등 모두 Free 해준다 ** //
        FAImage.Layers.Clear;
        FRBLayer := nil;

        FAImage.Bitmap.Assign(BufferBitmap);
        with frmMain do
        begin
          pnlBitmapLayer.Visible := False;
          UpdateMenu(nil);
        end;
      end;

    finally
      BufferBitmap.Free;
      FAImage.EndUpdate;

      FAImage.Repaint;
    end;
  end;

end;

function TfrmMDI.SetZoomPoint(ZoomPoint: TPoint): TPoint;
begin

  Inc(ZoomPoint.X, Round(GetHorzScroll.Position));
  Inc(ZoomPoint.Y, Round(GetVertScroll.Position));

  if FAImage.GetBitmapRect.Left > 0 then
    Dec(ZoomPoint.X, FAImage.GetBitmapRect.Left);
  if FAImage.GetBitmapRect.Top > 0 then
    Dec(ZoomPoint.Y, FAImage.GetBitmapRect.Top);

  if ZoomPoint.X <> 0 then
    ZoomPoint.X := Round(ZoomPoint.X / FAImage.Scale);
  if ZoomPoint.Y <> 0 then
    ZoomPoint.Y := Round(ZoomPoint.Y / FAImage.Scale);

  Result := ZoomPoint;

end;

procedure TfrmMDI.UpdateINIFile(bReadINI: Boolean);
var
  OpenSetup: TiniFile;
begin
  try
    OpenSetup := TiniFile.Create(ExtractFilePath(Application.ExeName) +
      INIPath);
    try
      OpenSetup.WriteString('오픈 캡쳐', '업데이트 날짜', IntToStr(UpdateDate));
      if FileExists(ExtractFilePath(application.ExeName) + INIPath) then
      begin

        case bReadINI of
          True:
            begin
              if OpenSetup.ReadBool('윈도우', 'MDI 창', False) = True then
                frmMain.MDIChildren[0].WindowState := wsMaximized;
            end;
          False: OpenSetup.WriteBool('윈도우', 'MDI 창',
              frmMain.MDIChildren[0].WindowState = wsMaximized);
        end;

      end;
    finally
      OpenSetup.Free;
    end;
  except
    // 에러 메세지를 보여주지 않습니다.
    // 만약 이 부분에러 에러가 발생하였다면 종료할 때도 같은 에러가 발생하므로
    // 종료할 때 에러 메세지를 보여줍니다.
  end;
end;

procedure TfrmMDI.EditingImage;
var
  Str: string;
begin
  if uBitmapList.Tag = uBitmapList.GetCount then
  begin
    Str := Copy(Caption, 0, Length(Caption) - 1);
    Caption := Str;
    frmMain.MDITab.Tabs[Tag] := ExtractFileName(Str);
  end
  else
  begin
    Str := (Copy(Caption, Length(Caption), 1));
    if Str <> '*' then
    begin
      Caption := Caption + '*';
      frmMain.MDITab.Tabs[Tag] := ExtractFileName(frmMain.MDITab.Tabs[Tag]) +
        '*';
    end;
  end;

end;

procedure TfrmMDI.ADrawBufferText(Render: Boolean = True);

  procedure ARenderText(Bitmap32: TBitmap32; Strings: TStrings; Level: Byte;
    ClipRect: TRect);
  var
    i, TextY: Integer;
  begin
    TextY := 2 + (ClipRect.Top - 2);

    Bitmap32.ClipRect := ClipRect;

    for i := 0 to Strings.Count - 1 do
    begin
      Bitmap32.RenderText(ClipRect.Left, TextY, Strings.Strings[i], Level,
        Color32(Bitmap32.Font.Color));
      Inc(TextY, Bitmap32.TextHeight('안녕'));
    end;

    Bitmap32.ClipRect := Rect(0, 0, Bitmap32.Width, Bitmap32.Height);
  end;

var
  DrawRect: TRect;
  DT_Align: Byte;
  Margin: Integer;
begin

  if (FBitmapLayer <> nil) and (FfrmAddText <> nil) then
    with FfrmAddText do
      with FBitmapLayer do
      begin

        FAImage.BeginUpdate;

        try
          Margin := 0;

          with FRBLayer.Location do
            DrawRect := Rect(Round(Left), Round(Top), Round(Right),
              Round(Bottom));

          Bitmap.Width := DrawRect.Right - DrawRect.Left;
          Bitmap.Height := DrawRect.Bottom - DrawRect.Top;

          if chkFrame.Checked = True then
          begin
            with Bitmap.Canvas do
            begin
              Pen.Mode := pmCopy;
              Pen.Style := psSolid;
              Pen.Color := FcboFrameColor.SelectColor;
              Margin := StrToInt(cboFrameSize.Items[cboFrameSize.ItemIndex]);
              Pen.Width := Margin;

              Brush.Style := bsSolid;
              Brush.Color := FcboBKColor.SelectColor;
            end;

            // ** 테두리 크기가 짝수일때와 홀수 일 때 사각형의 테두리가 다르게 그려짐 ** //
            // ** 이에 대한 수정을 한다. ** //
            // ** 짝수라면 ** //
            if Margin = 1 then
              DrawRect := Rect(0, 0, Bitmap.Width - 1, Bitmap.Height - 1)
            else
              DrawRect := Rect(1, 1, Bitmap.Width - 1, Bitmap.Height - 1);

            open_Rectangle(Bitmap, Point(DrawRect.Left, DrawRect.Top),
              Point(DrawRect.Right, DrawRect.Bottom),
              True,
              255);

            with DrawRect do
            begin
              Inc(Left, 2 + Margin);
              Inc(Top, 2 + Margin);
              Dec(Right, 2 + Margin);
              Dec(Bottom, 2 + Margin);
            end;

          end
          else
          begin
            Bitmap.Draw(0, 0, DrawRect, FAImage.Bitmap);

            DrawRect := Rect(0, 0, Bitmap.Width, Bitmap.Height);

            Bitmap.Canvas.Brush.Style := bsClear;
          end;

          if chkRender.Checked = True then
            Bitmap.Font.Assign(mmoText.Font)
          else
            Bitmap.Canvas.Font.Assign(mmoText.Font);

          DT_Align := DT_LEFT;

          if btnAlignLeft.Down = True then
            DT_Align := DT_LEFT
          else if btnAlignCenter.Down = True then
            DT_Align := DT_CENTER
          else if btnAlignRight.Down = True then
            DT_Align := DT_Right;

          if chkRender.Checked = True then
          begin
            if Render = True then
              ARenderText(Bitmap, mmoText.Lines, 2, DrawRect)
            else
              ARenderText(Bitmap, mmoText.Lines, 0, DrawRect);
          end
          else
            DrawText(Bitmap.Canvas.Handle, PChar(mmoText.Text), -1,
              DrawRect, DT_Align or DT_WORDBREAK or DT_EDITCONTROL);

          Bitmap.ResetAlpha;

          if chkFrame.Checked = False then
            with Bitmap.Canvas do
            begin
              Pen.Width := 0;
              Brush.Color := clWhite;
              Brush.Style := bsSolid;

              with FRBLayer.Location do
              begin
                if Left < 0 then
                  Rectangle(0, 0, -Round(Left), Round(Bottom - Top));
                if Top < 0 then
                  Rectangle(0, 0, Round(Right - Left), -Round(Top));
                if Right > FAImage.Bitmap.Width then
                  Rectangle(Round(Right - Left) - (Round(Right) -
                    FAImage.Bitmap.Width), 0,
                    Round(Right - Left), Round(Bottom - Top));
                if Bottom > FAImage.Bitmap.Height then
                  Rectangle(0, Round(Bottom - Top) - (Round(Bottom) -
                    FAImage.Bitmap.Height),
                    Round(Right - Left), Round(Bottom - Top));

              end;

            end;

        finally
          FAImage.EndUpdate;

          FAImage.Repaint;
        end;

      end;

end;

procedure TfrmMDI.ADrawText(OldPt, nPt: TPoint);
begin

  NormalizePoint(OldPt, nPt);

  FfrmAddText := TfrmAddText.Create(Application);

  with FfrmAddText do
    case ShowModal of
      mrOK:
        begin

          SaveUndoImage(FAImage.Bitmap);

          // ** 중간에 레이어가 삭제되는 것을 방지 ** //
          ToolSel := tsSelect;
          with CreateBitmapLayer do
          begin

            FRBLayer.Location := FloatRect(Rect(OldPt, nPt));

            ADrawBufferText;

            LockWindowUpdate(frmMain.pnlPaintTool.Handle);

            frmMain.pnlBitmapLayer.Visible := True;
            ToolSel := tsText;
            FAImage.Cursor := tsText;
            frmMain.btnText.Down := True;

            LockWindowUpdate(0);

          end;

        end;
      mrCancel:
        begin
          FfrmAddText.Free;
          FfrmAddText := nil;
        end;

    end; //Case ShowModal of

end;

function TfrmMDI.UpdateTabRefresh(MDIIndex: Integer; MDICaption: string):
  Boolean;
var
  i: Integer;
  s: string;
begin
  s := MDICaption;
  MDICaption := (Copy(MDICaption, Length(MDICaption), 1));
  Result := False;

  if frmOption.chkUseSave.Checked = False then
    Exit;

  for i := 0 to frmMain.MDIChildCount - 1 do
    if frmMain.MDIChildRen[i].Tag = MDIIndex then
    begin
      frmMain.MDIChildRen[i].Show;
      break;
    end;

  if MDICaption = '*' then
  begin
    MDICaption := (Copy(S, 0, Length(S) - 1));
    S := MDICaption + ' 를(을) 저장하시겠습니까?';

    case Application.MessageBox(Pchar(S), '오픈 캡쳐', MB_YESNOCANCEL +
      MB_ICONQUESTION + MB_DEFBUTTON1 + MB_APPLMODAL) of
      ID_YES:
        begin
          frmMain.mnuSaveFile.Click;

          // ** 사용자가 닫기 단추를 누르고 파일 저장 대화상자가 뜬 후 ** //
          // ** 취소 버튼을 눌렀다면 프로그램을 종료하지 않는다. ** //
          if (Copy(Self.Caption, Length(Self.Caption), 1)) = '*' then
            Result := True;
        end;
      ID_CANCEL: Result := True;
    end;

  end;

end;

procedure TfrmMDI.AddMDITab(TabName: string; var flags: Integer);
var
  IsAsterisk: string;
begin
  inc(flags);

  case frmMain.FCapture of
    False: IsAsterisk := '';
  else
    IsAsterisk := '*';
  end;

  if frmMain.MDITab.Tabs[0] = '캡쳐 작업 없음' then
    frmMain.MDITab.Tabs[0] := TabName + ' ' + IntToStr(flags) + IsAsterisk
  else if frmMain.MDITab.Tabs[0] <> '캡쳐 작업 없음' then
  begin
    frmMain.MDITab.Tabs.Add(TabName + ' ' + IntToStr(flags) + IsAsterisk);
    frmMain.MDITab.TabIndex := frmMain.MDITab.Tabs.Count - 1;
  end;

  Tag := frmMain.MDITab.Tabs.Count - 1;
  Caption := TabName + ' ' + IntToStr(flags) + IsAsterisk;
end;

procedure TfrmMDI.CaptureStart(var Message: TMessage);
begin
  with frmMain do
    case Message.LParam of
      0: AddMDITab('새 이미지', FCaptureCount.NewImage);
      1: AddMDITab('사용자 지정', FCaptureCount.CaptureSel);
      2: AddMDITab('윈도우', FCaptureCount.CaptureWnd);
      3: AddMDITab('윈도우, 컨트롤', FCaptureCount.CaptureCtrl);
      4: AddMDITab('활성화 윈도우', FCaptureCount.CaptureActiveWnd);
      5: AddMDITab('전체 화면', FCaptureCount.CaptureFull);
      6: AddMDITab('마지막 영역', FCaptureCount.CaptureLast);
      7: AddMDITab('아이콘', FCaptureCount.CaptureIcon);
      8: AddMDITab('고정된 영역', FCaptureCount.CaptureFixed);
      9: AddMDITab('메뉴', FCaptureCount.CaptureMenu);
      10: AddMDITab('마우스 포인터', FCaptureCount.CaptureCursor);
      11: AddMDITab('바탕화면', FCaptureCount.CaptureWallpaper);
      12: AddMDITab('오브젝트', FCaptureCount.CaptureObject);
    end;

end;

procedure TfrmMDI.SaveUndoImage(ABitmap: TBitmap32);
var
  Str: string;
begin
  // ** 이미지이름의 "*" 추가 관련 ** //
  Str := (Copy(Caption, Length(Caption), 1));

  rBitmapList.ClearBitmaps;

  // ** 더 이상 이미지를 추가하지 않고 맨 처음 것을 지운다. ** //
  //if uBitmapList.Items.Count = 5 then
  // uBitmapList.Items.Delete(0);

  if Str <> '*' then
  begin
    // ** 저장 된 이미지가 아니라면 .. ** //
    Caption := Caption + '*';
    frmMain.MDITab.Tabs[Tag] := frmMain.MDITab.Tabs[Tag] + '*';
    uBitmapList.AddBitmap(ABitmap);
    uBitmapList.Tag := uBitmapList.GetCount - 1;
  end
  else
  begin
    if uBitmapList.GetCount = 11 then
      uBitmapList.Tag := -1;

    uBitmapList.AddBitmap(ABitmap);
  end;

  rBitmapList.ClearBitmaps;
  frmMain.mnuRedo.Enabled := False;
  frmMain.mnuUndo.Enabled := True;

end;

procedure TfrmMDI.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  frmMain.MDIClosing(Tag);
end;

procedure TfrmMDI.FormActivate(Sender: TObject);
begin
  with frmMain do
  begin
    MDITab.TabIndex := Self.Tag;

    // ** 다른 창에서 현재 창으로 넘어왔을 떄.... ** //
    if FCapture = False then
      if FAImage.Scale = 0.25 then
        cboZoom.ItemIndex := 0
      else if FAImage.Scale = 0.5 then
        cboZoom.ItemIndex := 1
      else
        cboZoom.ItemIndex := Round(FAImage.Scale) + 1;

    mnuUndo.Enabled := uBitmapList.GetCount > 0;
    mnuRedo.Enabled := rBitmapList.GetCount > 0;

    // ** 다른 창에서 현재 창으로 넘어왔을 떄.... ** //
    if FCapture = False then
      Statusbar.Panels[0].Text := Format('가로 : %d x 세로 : %d',
        [FAImage.Bitmap.Width, FAImage.Bitmap.Height]);

    Statusbar.Panels[1].Text := '';
    StatusBar.Panels[2].Text := cboZoom.Text;
  end;

  frmMain.UpdatePaintTool(ToolSel);

end;

procedure TfrmMDI.FormCreate(Sender: TObject);
begin
  FAImage := TImgView32.Create(Self);
  EraserBitmap := TBitmap32.Create;

  FAImage.Name := 'AImage';
  FAImage.Parent := Self;

  FAImage.Align := alClient;
  FAImage.PopupMenu := PopupMenu;

  FAImage.OnDblClick := AImageDblClick;
  FAImage.OnMouseDown := AImageMouseDown;
  FAImage.OnMouseMove := AImageMouseMove;
  FAImage.OnMouseUp := AImageMouseUp;
  FAImage.OnPaintStage := AImagePaintStage;

  frmMain.XPMenuSetup(FXPMenu, Self);

  SendMessage(frmMain.Handle, CM_MDICursor, 0, 0);

  ToolSel := tsDrag;

  FAImage.Cursor := CUR_DRAG;

  FDown := False;
  FDoubleClick := False;
  FLayerDown := False;
  FLayerMove := False;

  UpdateiniFile(True);

  uBitmapList := TBitmapList.Create(nil);
  uBitmapList.Tag := -1;
  rBitmapList := TBitmapList.Create(nil);

  with FAImage.PaintStages[0]^ do
    if Stage = PST_CLEAR_BACKGND then
      Stage := PST_CUSTOM;

  FRBLayer := nil;
  FBitmapLayer := nil;
  FfrmAddText := nil;

  FDraw := False;

end;

procedure TfrmMDI.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := not UpdateTabRefresh(Tag, Caption);
end;

procedure TfrmMDI.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
const
  ScrollSize = 50;
begin
  if FDown = False then
    if WheelDelta >= 0 then
    begin
      if Shift = [ssShift] then
        GetHorzScroll.Position := GetHorzScroll.Position - ScrollSize
      else
        with frmMain.cboZoom do
          if Shift = [ssCtrl] then
          begin
            if ItemIndex > 0 then
            begin
              ItemIndex := ItemIndex - 1;
              OnChange(Sender);
            end;
          end
          else
            GetVertScroll.Position := GetVertScroll.Position - ScrollSize;
    end
    else
    begin
      if Shift = [ssShift] then
        GetHorzScroll.Position := GetHorzScroll.Position + ScrollSize
      else
        with frmMain.cboZoom do
          if Shift = [ssCtrl] then
          begin
            if ItemIndex < Items.Count - 1 then
            begin
              ItemIndex := ItemIndex + 1;
              OnChange(Sender);
            end;
          end
          else
            GetVertScroll.Position := GetVertScroll.Position + ScrollSize;
    end;

  TXX(FAImage).UpdateImage;
  Handled := True;
end;

procedure TfrmMDI.CloseWndClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMDI.UndoPopupMenuClick(Sender: TObject);
begin
  frmMain.mnuUndo.Click;
end;

procedure TfrmMDI.RedoPopupMenuClick(Sender: TObject);
begin
  frmMain.mnuRedo.Click;
end;

procedure TfrmMDI.tmrBrushTimer(Sender: TObject);
var
  i: Integer;
  theta: integer;
  r: integer;
  Pt: TPoint;
begin

  for i := 1 to 200 do
  begin
    r := Random(frmMain.udPenSize.Position + 15);
    theta := Random(360);

    Pt := nPt;
    Pt.X := Pt.X + Round((r * Cos(theta * 3.141592 / 180)));
    Pt.Y := Pt.Y + Round((r * Sin(theta * 3.141592 / 180)));

    FAImage.Bitmap.SetPixelTS(Pt.X, Pt.Y,
      SetAlpha(Color32(frmMain.FPalettePen.Color),
      Round((255 / 100) * frmMain.gbShapeAlpha.Position)));
  end;

  TXX(FAImage).UpdateImage;

end;

procedure TfrmMDI.FormDestroy(Sender: TObject);
var
  FindExHwnd: Hwnd;
begin
  FXPMenu.Free;
  FAImage.Free;
  EraserBitmap.Free;

  // ** 탭 관련 버그를 수정한다 ** ///////////
  FindExHwnd := FindWindowEx(frmMain.MDITab.Handle, 0, 'msctls_updown32', nil);
  if FindExHwnd <> 0 then
    if IsWindowVisible(FindExHwnd) then
      frmMain.MDITab.ScrollTabs(1);

  frmMain.MDITab.Repaint;

  uBitmapList.Free;
  rBitmapList.Free;

  UpdateiniFile(False);
end;

procedure TfrmMDI.ScrollBoxClick(Sender: TObject);
begin
  Windows.SetFocus(Handle);
end;

procedure TfrmMDI.tmrAutoScrollTimer(Sender: TObject);
var
  lpPoint: TPoint;
  ClientRect: TRect;
begin
  GetCursorPos(lpPoint);
  lpPoint := ScreenToClient(lpPoint);

  AutoScroll := False;

  ClientRect := GetClientRect;
  ClientRect.Right := ClientRect.Right - GetVertScroll.Width;
  ClientRect.Bottom := ClientRect.Bottom - GetHorzScroll.Height;

  if PtInRect(ClientRect, lpPoint) = False then
  begin

    if lpPoint.X < 1 then
      GetHorzScroll.Position := GetHorzScroll.Position + lpPoint.X;
    if lpPoint.Y < 1 then
      GetVertScroll.Position := GetVertScroll.Position + lpPoint.Y;
    if lpPoint.X > ClientWidth - GetVertScroll.Width then
      GetHorzScroll.Position := GetHorzScroll.Position + Abs(lpPoint.X -
        ClientWidth);
    if lpPoint.Y > ClientHeight - GetHorzScroll.Height then
      GetVertScroll.Position := GetVertScroll.Position + Abs(lpPoint.Y -
        ClientHeight);

    SendMessage(FAImage.Handle, WM_MOUSEMOVE, 0, lpPoint.X + lpPoint.Y shl 16);
    TXX(FAImage).UpdateImage;

  end;

end;

procedure TfrmMDI.AImagePaintStage(Sender: TObject; Buffer: TBitmap32;
  StageNum: Cardinal);
const
  Colors: array[0..1] of TColor32 = ($FFFFFFFF, $FFB0B0B0);
var
  W, I, J, Parity: Integer;
  Line1, Line2: TArrayOfColor32;
begin
  with FAImage.Buffer do
  begin
    W := Width;
    SetLength(Line1, W);
    SetLength(Line2, W);
    for I := 0 to W - 1 do
    begin
      Parity := I shr 3 and $1;
      Line1[I] := Colors[Parity];
      Line2[I] := Colors[1 - Parity];
    end;
    for J := 0 to Height - 1 do
    begin
      Parity := J shr 3 and $1;
      if Boolean(Parity) then
        MoveLongword(Line1[0], ScanLine[J]^, W)
      else
        MoveLongword(Line2[0], ScanLine[J]^, W);
    end;
  end;

end;

procedure TfrmMDI.AImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);

  procedure DrawingToolSetup;
  begin

    with frmMain do
      with FAImage.Bitmap.Canvas do
      begin
        Pen.Mode := pmCopy;
        Pen.Style := psSolid;
        Pen.Color := FPalettePen.Color;
        Pen.Width := udPenSize.Position;

        Brush.Color := FPaletteBrush.Color;

        case ToolSel of
          tsSelect, tsText:
            begin
              Pen.Mode := pmNotXor;
              Pen.Style := psDot;
              Pen.Color := clBlack;
              Pen.Width := 1;
              Brush.Style := bsClear;
            end;
          tsEraser:
            begin
              if btnSmall.Down then
                Pen.Width := 10
              else if btnNormal.Down then
                Pen.Width := 15
              else if btnBig.Down then
                Pen.Width := 20;

              Pen.Color := frmMain.FPaletteBrush.Color;
            end;
          tsRect, tsEllipse, tsRoundRect: if btnRect.Down then
              Brush.Style := bsClear
            else if btnSolid.Down then
              Brush.Style := bsSolid
            else
            begin
              Brush.Style := bsSolid;
              Pen.Style := psClear;
            end;

          tsArrow:
            begin
              Brush.Color := FPalettePen.Color;
              Brush.Style := bsSolid;
            end;

        end;

      end;

  end;

var
  lpPoint: TPoint;
begin
  Windows.SetFocus(Handle);

  if Button in [mbLeft] then
    ImageAutoScroll(True)
  else if (ToolSel <> tsZoom) then
  begin
    { ** 레이어, 텍스트 추가 시 사용자가 드래그 후
      마우스 업 할때 마우스 오른쪽 버튼을 동시에
      누르면 잔상이 남는 버그 수정 ** }
    GetCursorPos(lpPoint);
    lpPoint := FAImage.ScreenToClient(lpPoint);
    FAImage.OnMouseUp(Sender, Button, Shift, lpPoint.X,
      lpPoint.Y, Layer);
    Exit;
  end;

  if FRBLayer <> nil then
    if FRBLayer.HitTest(X, Y) = False then
      FlattenLayer(False, True)
    else
      Exit;

  if (Button in [mbLeft]) or (ToolSel = tsZoom) then
    FDown := True;

  if FDown = True then
  begin
    if (ToolSel in [tsDrag, tsSelColor, tsSelect, tsText, tsZoom]) = False then
      SaveUndoImage(FAImage.Bitmap);

    DrawingToolSetup;

    if ToolSel <> tsDrag then
    begin
      OldPt := SetZoomPoint(Point(X, Y));
      nPt := OldPt;
    end;

    case ToolSel of
      tsDrag:
        begin
          Screen.Cursor := CUR_DRAG_MOVE;
          FAImage.Cursor := CUR_DRAG_MOVE;
          Inc(X, Round(GetHorzScroll.Position));
          Inc(Y, Round(GetVertScroll.Position));
          OldPt := Point(X, Y);
        end;
      tsSelect, tsText: FAImage.Bitmap.Canvas.Rectangle(OldPt.X, OldPt.Y, nPt.X,
          nPt.Y);
      tsPen:
        begin
          FAImage.Bitmap.Canvas.MoveTo(OldPt.X, OldPt.Y);
          FAImage.Bitmap.Canvas.LineTo(nPt.X, nPt.Y);
        end;
      tsAirBrush:
        begin
          tmrBrush.OnTimer(Sender);
          tmrBrush.Enabled := True;
        end;
      tsLine, tsRect, tsEllipse,
        tsRoundRect, tsArrow:
        begin
          FDraw := True;
          DrawShape(ToolSel, OldPt, nPt, True);
        end;
    end; // case ToolSel of

  end; // if FDown = True then begin

end;

procedure TfrmMDI.AImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
var
  srcRect: TRect;
  FLeft, FTop: Integer;
begin

  ImageAutoScroll(False);

  if FDoubleClick = True then
    if (FBitmapLayer <> nil) and (FfrmAddText <> nil) then
      if FRBLayer.HitTest(X, Y) = True then
      begin
        case FfrmAddText.ShowModal of
          mrOK: ;
          mrCancel: FfrmAddText.OnCreate(Sender);
        end;

        ADrawBufferText;

      end;

  FDoubleClick := False;
  FLayerMove := False;

  if FLayerDown = True then
  begin

    case ToolSel of
      tsSelect: with FBitmapLayer do
        begin
          Bitmap.Width := Round(Location.Right - Location.Left);
          Bitmap.Height := Round(Location.Bottom - Location.Top);

          FAImage.BeginUpdate;
          try
            with Location do
              srcRect := Rect(Round(Left), Round(Top), Round(Right),
                Round(Bottom));

            Bitmap.Width := srcRect.Right - srcRect.Left;
            Bitmap.Height := srcRect.Bottom - srcRect.Top;

            MinMaxRect(0, FAImage.Bitmap.Width, 0, FAImage.Bitmap.Height,
              srcRect);

            FLeft := 0;
            FTop := 0;

            if Round(Location.Left) < 0 then
              FLeft := -Round(Location.Left);
            if Round(Location.Top) < 0 then
              FTop := -Round(Location.Top);

            FAImage.Bitmap.Draw(srcRect, srcRect, FLayerBuffer);

            FAImage.Bitmap.ResetAlpha;

            Bitmap.Draw(FLeft, FTop,
              srcRect,
              FAImage.Bitmap);

            FLayerBuffer.Free;

            FBitmapLayer.Tag := Layer_DeleteRgn;

          finally
            FAImage.EndUpdate;
          end;

        end;
      tsText:
        begin
          FLayerDown := False;
          ADrawBufferText;
        end;
    end;

    FLayerDown := False;

  end;

  if FDown = True then
  begin
    case ToolSel of
      tsDrag:
        begin
          FAImage.Cursor := CUR_DRAG;
          Screen.Cursor := crDefault;
        end;
      tsSelect:
        begin
          FAImage.Bitmap.Canvas.Rectangle(Rect(OldPt, nPt));

          nPt := SetZoomPoint(Point(X, Y));
          if EqualPointXY(OldPt, nPt) = False then
            with CreateBitmapLayer do
            begin
              NormalizePoint(OldPt, nPt);

              if OldPt.X < 0 then OldPt.X := 0;
              if OldPt.Y < 0 then OldPt.Y := 0;
              if nPt.X > FAImage.Bitmap.Width then nPt.X := FAImage.Bitmap.Width;
              if nPt.Y > FAImage.Bitmap.Height then nPt.Y := FAImage.Bitmap.Height;

              Bitmap.Width := nPt.X - OldPt.X;
              Bitmap.Height := nPt.Y - OldPt.Y;

              FAImage.Bitmap.ResetAlpha;

              Bitmap.Draw(0, 0, Rect(OldPt, nPt), FAImage.Bitmap);

              FRBLayer.Location := FloatRect(OldPt.X, OldPt.Y, nPt.X, nPt.Y);

              // ** 이 후 마우스로 드래그 시 영역 삭제 ** //
              Tag := 1;
            end;
        end;

      tsPen:
        begin
          OldPt := SetZoomPoint(Point(X, Y));

          FAImage.Bitmap.Canvas.MoveTo(OldPt.X, OldPt.Y);
          FAImage.Bitmap.Canvas.LineTo(nPt.X, nPt.Y);

        end;
      tsEraser:
        begin
          //OldPt := SetZoomPoint(Point(X, Y));
          EraserBitmap.DrawTo(FAImage.Bitmap, OldPt.X-Trunc(frmMain.EraserSize / 2), OldPt.Y-Trunc(frmMain.EraserSize / 2));
         // FAImage.Bitmap.Canvas.MoveTo(OldPt.X, OldPt.Y);
         // FAImage.Bitmap.Canvas.LineTo(nPt.X, nPt.Y);
        end;
      tsAirBrush: tmrBrush.Enabled := False;
      tsSelColor: with frmMain do
        begin
          nPt := SetZoomPoint(Point(X, Y));
          FPalettePen.Color := FAImage.Bitmap.Canvas.Pixels[nPt.X, nPt.Y];
          btnSelColor.Tag := -1;
          ChangePaintTool(FBeforeToolSel);
        end;
      tsFloodFill: with FAImage.Bitmap.Canvas do
        begin
          Brush.Color := frmMain.FPalettePen.Color;

          FloodFill(nPt.X, nPt.Y,
            Pixels[nPt.X, nPt.Y],
            fsSurFace);

          Brush.Color := frmMain.FPaletteBrush.Color;
        end;
      tsLine, tsRect, tsEllipse,
        tsRoundRect, tsArrow:
        begin
          DeleteShape(ToolSel, OldPt, nPt);

          nPt := SetZoomPoint(Point(X, Y));
          DrawShape(ToolSel, OldPt, nPt, False, True);

          FDraw := False;
        end;
      tsText: with FAImage.Bitmap.Canvas do
        begin
          Rectangle(Rect(OldPt, nPt));

          nPt := SetZoomPoint(Point(X, Y));
          NormalizePoint(OldPt, nPt);

          if OldPt.X < 0 then OldPt.X := 0;
          if OldPt.Y < 0 then OldPt.Y := 0;
          if nPt.X > FAImage.Bitmap.Width then nPt.X := FAImage.Bitmap.Width;
          if nPt.Y > FAImage.Bitmap.Height then nPt.Y := FAImage.Bitmap.Height;

          if EqualPointXY(OldPt, nPt) = False then
            ADrawText(OldPt, nPt);

        end;
    end; // case ToolSel of

    OldPt := Point(0, 0);
    nPt := Point(0, 0);
    FDown := False;

    Screen.Cursor := crDefault;

  end; // if FDown = True then begin

  if ToolSel = tsZoom then
    with frmMain.cboZoom do
    begin
      if (Button in [mbLeft]) and (ItemIndex < Items.Count - 1) then
        ItemIndex := ItemIndex + 1
      else if (Button in [mbRight]) and (ItemIndex > 0) then
        ItemIndex := ItemIndex - 1;

      OnChange(Sender);
    end;

end;

procedure TfrmMDI.AImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer; Layer: TCustomLayer);

  function RefreshCursor(X, Y: Integer): Boolean;
  begin
    Result := True;
    if FRBLayer <> nil then
      if FRBLayer.HitTest(X, Y) = True then
      begin
        FAImage.Cursor := crSizeAll;
        Result := False;
      end
      else
        FAImage.Cursor := ToolSel;
  end;

  procedure BrushLine (const xStart,yStart, xEnd,yEnd: Integer; tobitmap: TBitmap32);
  var
    a,b        :  Integer;  // displacements in x and y
    d          :  Integer;  // decision variable
    diag_inc   :  Integer;  // d's increment for diagonal steps
    dx_diag    :  Integer;  // diagonal x step for next pixel
    dx_nondiag :  Integer;  // nondiagonal x step for next pixel
    dy_diag    :  Integer;  // diagonal y step for next pixel
    dy_nondiag :  Integer;  // nondiagonal y step for next pixel
    i          :  Integer;  // loop index
    nondiag_inc:  Integer;  // d's increment for nondiagonal steps
    swap       :  Integer;  // temporary variable for swap
    x,y        :  Integer;  // current x and y coordinates
    BeginX, BeginY: Integer;
  begin {DrawLine}
    x := xStart;              // line starting point
    y := yStart;

    // Determine drawing direction and step to the next pixel.
    a := xEnd - xStart;       // difference in x dimension x방향 증가분
    b := yEnd - yStart;       // difference in y dimension y방향 증가분

    // Determine whether end point lies to right or left of start point.
    if   a < 0                // drawing towards smaller x values? 오른쪽에서 왼쪽으로 움직임
    then begin
      a := -a;                // make 'a' positive  //a를 양으로 만든다.
      dx_diag := -1           //오른쪽에서 왼쪽으로
    end
    else dx_diag := 1;  //마우스를 왼쪽에서 오른쪽으로 움직였다면 그방향으로 증가.

    // Determine whether end point lies above or below start point.
    if   b < 0                // drawing towards smaller x values? 아래에서 위로 움직임
    then begin
      b := -b;                // make 'a' positive
      dy_diag := -1           //아래에서 위로
    end
    else  dy_diag := 1; //마우스를 위에서 아래로 움직였다면 그방향으로 증가.

    // Identify octant containing end point.
    if   a < b  then//45도 보다 클 경우
    begin
      swap := a;  //a와 b를 바꾼다.
      a := b;
      b := swap;
      dx_nondiag := 0;       //x방향 증가치는 0
      dy_nondiag := dy_diag  //y방향 증가치는 1
    end
    else               //45도 보다 작을 경우
    begin
      dx_nondiag := dx_diag;  //x방향 증가치는 1
      dy_nondiag := 0        //y방향 증가치는 0
    end;

    d := b + b - a;            // initial value for d is 2*b - a
    nondiag_inc := b + b;      // set initial d increment values
    diag_inc    := b + b - a - a;

    for i := 0 TO a DO // draw the a+1 pixels
    begin

      EraserBitmap.DrawTo(tobitmap, x-Trunc(frmMain.EraserSize / 2), y-Trunc(frmMain.EraserSize / 2));

      //d 판별 d = b - a
      if  d < 0 then      // is midpoint above the line? 중점이 라인 아래쪽에 있다.
      begin               // step nondiagonally    대각선이 아니게 증가
        x := x + dx_nondiag;  //x 는 1만큼 증가
        y := y + dy_nondiag;  //y = y 로 변화 없다.
        d := d + nondiag_inc  // // update decision variable d = d + 2b;
      end

      else   // midpoint is above the line; b - a = d > 0 중점이 라인 위쪽에 있다.
      begin               // step diagonally 대각선으로 증가
        x := x + dx_diag;   // x = x 로 변화 없다.
        y := y + dy_diag;   // y 는 1만큼 증가
        d := d + diag_inc   // d = d + 2(b - a)
      end;

    end; //for
  end {Line};

var
  Pt: TPoint;
begin

  if RefreshCursor(X, Y) = False then
    Exit;

  if FDown = True then
  begin
    case ToolSel of
      tsDrag:
        begin
          Inc(X, Round(GetHorzScroll.Position));
          Inc(Y, Round(GetVertScroll.Position));

          GetHorzScroll.Position := GetHorzScroll.Position - (X - OldPt.X);
          GetVertScroll.Position := GetVertScroll.Position - (Y - OldPt.Y);
          TXX(FAImage).UpdateImage;
        end;
      tsPen:
        begin
          // ** OldPt 와 nPt 가 여기서만 뒤바뀜 ** //
          OldPt := SetZoomPoint(Point(X, Y));

          FAImage.Bitmap.Canvas.MoveTo(OldPt.X, OldPt.Y);
          FAImage.Bitmap.Canvas.LineTo(nPt.X, nPt.Y);

          nPt := OldPt;
        end;
      tsEraser:
        begin
          OldPt := SetZoomPoint(Point(X, Y));
          FAImage.BeginUpdate;
          BrushLine(OldPt.X, OldPt.Y, nPt.X, nPt.Y, FAImage.Bitmap);
          FAImage.EndUpdate;
          FAImage.Invalidate;
          FAImage.Refresh;
          nPt := OldPt;
        end;

      tsAirBrush: nPt := SetZoomPoint(Point(X, Y));
      tsLine, tsRect, tsEllipse,
        tsRoundRect, tsArrow: if FDraw = False then
        begin
          DeleteShape(ToolSel, OldPt, nPt);
          nPt := SetZoomPoint(Point(X, Y));
          DrawShape(ToolSel, OldPt, nPt, False, True);
          FDraw := True;
        end;
      tsSelect, tsText: with FAImage.Bitmap.Canvas do
        begin
          Rectangle(OldPt.X, OldPt.Y, nPt.X, nPt.Y);
          nPt := SetZoomPoint(Point(X, Y));
          Rectangle(OldPt.X, OldPt.Y, nPt.X, nPt.Y);
        end;
    end; // case ToolSel of

    if ToolSel <> tsDrag then
      if ToolSel in [tsPen, tsEraser] then
        frmMain.StatusBar.Panels[1].Text :=
          Format('마우스 좌표 : (%d ,  %d)', [nPt.X, nPt.Y])
      else if ((nPt.X - OldPt.X) = 0) and ((nPt.Y - OldPt.Y) = 0) = False then
        frmMain.StatusBar.Panels[1].Text :=
          Format('마우스 좌표 : (%d ,  %d) ,  크기 : %d x %d', [nPt.X,
          nPt.Y, Abs(nPt.X - OldPt.X), Abs(nPt.Y - OldPt.Y)])

        // if FDown = True then begin
  end
  else if FLayerDown = False then
  begin
    Pt := SetZoomPoint(Point(X, Y));
    frmMain.StatusBar.Panels[1].Text := Format('마우스 좌표 : (%d ,  %d)',
      [Pt.X, Pt.Y]);
  end;

end;

procedure TfrmMDI.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  lpPoint: TPoint;
begin

  case Key of
    VK_Left: GetHorzScroll.Position := GetHorzScroll.Position - FAImage.Scale;
    VK_RIGHT: GetHorzScroll.Position := GetHorzScroll.Position + FAImage.Scale;
    VK_UP: GetVertScroll.Position := GetVertScroll.Position - FAImage.Scale;
    VK_DOWN: GetVertScroll.Position := GetVertScroll.Position + FAImage.Scale;
  end;

  if Key in [VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN] then
  begin
    GetCursorPos(lpPoint);
    lpPoint := ScreenToClient(lpPoint);
    SendMessage(FAImage.Handle, WM_MOUSEMOVE, 0, lpPoint.X + lpPoint.Y shl 16);
    TXX(FAImage).UpdateImage;
  end;

end;

procedure TfrmMDI.AImageDblClick(Sender: TObject);
begin
  FDoubleClick := True;
end;

procedure TfrmMDI.PopupMenuPopup(Sender: TObject);
var
  i: Integer;
  MenuItem: TMenuItem;
begin
  ImageAutoScroll(False);
  PopupMenu.Items.Clear;

  if ToolSel <> tsZoom then
  begin
    with frmMain.MainMenu.Items[1] do
      for i := 0 to Count - 1 do
      begin
        MenuItem := TMenuItem.Create(Self);
        MenuItem.Caption := Items[i].Caption;
        MenuItem.Hint := Items[i].Hint;
        MenuItem.ImageIndex := Items[i].ImageIndex;
        MenuItem.Enabled := Items[i].Enabled;
        MenuItem.OnClick := Items[i].OnClick;
        PopupMenu.Items.Add(MenuItem);
      end;

    with FXPMenu do
    begin
      AutoDetect := False;
      Active := False;
      AutoDetect := True;
      Active := True;
    end;

  end;

end;

end.