﻿unit uOption;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, Buttons, FileCtrl, IniFiles,
  GR32_RangeBars, Spin;

Const
  INIProgramSetupSectionName = 'Program Setup';

type
  TfrmOption = class(TForm)
    btnOk: TButton;
    btnCancel: TButton;
    pcCaptureOption: TPageControl;
    tsCaptureOption: TTabSheet;
    gbxCapture: TGroupBox;
    chkCaptureMouse: TCheckBox;
    chkCaptureClipBoard: TCheckBox;
    chkCaptureMinimized: TCheckBox;
    gbxDelay: TGroupBox;
    Label5: TLabel;
    edtDelay: TEdit;
    udDelay: TUpDown;
    gbxFixed: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    edtFixedWidth: TEdit;
    udWidth: TUpDown;
    edtFixedHeight: TEdit;
    udHeight: TUpDown;
    tsAutoSave: TTabSheet;
    chkInvalidate: TCheckBox;
    Label1: TLabel;
    edtScrollDelay: TEdit;
    udScrollDelay: TUpDown;
    btnAdd: TButton;
    lblAutoSave: TLabel;
    chkAutoSave: TCheckBox;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    edtPath: TEdit;
    Label4: TLabel;
    cboFormat: TComboBox;
    btnSelectFolder: TBitBtn;
    Label6: TLabel;
    edtFileName: TEdit;
    tsJPEG: TTabSheet;
    lblJpeg: TLabel;
    edtJpeg: TEdit;
    udJpeg: TUpDown;
    gbxIsHide: TGroupBox;
    tsSaveImage: TTabSheet;
    chkUseSave: TCheckBox;
    chkActive: TCheckBox;
    chkTime: TCheckBox;
    tmrTime: TTimer;
    GroupBox2: TGroupBox;
    stTime: TStaticText;
    edtTime: TEdit;
    spdExTime1: TSpeedButton;
    spdExTime2: TSpeedButton;
    spdExTime3: TSpeedButton;
    spdExTime4: TSpeedButton;
    gbJpeg: TScrollBar;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    edtWavPath: TEdit;
    btnWav: TButton;
    OpenWavDialog: TOpenDialog;
    TabSheet1: TTabSheet;
    chkTrayIconUse: TCheckBox;
    GroupBox4: TGroupBox;
    chkTrayMinimizeHide: TCheckBox;
    chkTrayCloseHide: TCheckBox;
    lblTrayCloseHide: TLabel;
    GroupBox5: TGroupBox;
    Label9: TLabel;
    edtNumberType: TSpinEdit;
    chkCheckForUpdate: TCheckBox;
    chkTrayStartupMinimized: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnSelectFolderClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtDelayKeyPress(Sender: TObject; var Key: Char);
    procedure gbJpegChange(Sender: TObject);
    procedure udJpegChanging(Sender: TObject; var AllowChange: Boolean);
    procedure edtJpegChange(Sender: TObject);
    procedure tmrTimeTimer(Sender: TObject);
    procedure chkTimeClick(Sender: TObject);
    procedure spdExTime1Click(Sender: TObject);
    procedure spdExTime2Click(Sender: TObject);
    procedure spdExTime3Click(Sender: TObject);
    procedure spdExTime4Click(Sender: TObject);
    procedure btnWavClick(Sender: TObject);
    procedure chkTrayIconUseClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    AutoSaveCount: Integer;
    procedure CheckAutoSave(StartCount: Integer = 1);
  end;

var
  frmOption: TfrmOption;

implementation

uses uMain, OpenInterface;

{$R *.dfm}

procedure TfrmOption.FormCreate(Sender: TObject);
begin
  udWidth.Max := Screen.Width;
  udHeight.Max := Screen.Height;

  stTime.DoubleBuffered := True;

  LoadINI(Self, True, INIProgramSetupSectionName, INIPath);
  if cboFormat.ItemIndex = -1 then
    cboFormat.ItemIndex := 2;

  if chkTime.Checked = True then
    tmrTime.Enabled := True;

  CheckAutoSave;

  // 옵션 폼에 한해서만 Auto-Create 형태이기 때문에
  // 애플리케이션 시작 시에 폼이 생성됩니다.
  // 따라서 트레이 아이콘 초기화는 이곳에서 하게 됩니다.
  // 그리고 또 설정이 변경되었을 경우 옵션 폼이 다시 생성되게 되므로
  // 설정 된 변경사항도 자동으로 적용됩니다.
  if chkTrayIconUse.Checked = True then begin
    frmMain.TrayIcon.Visible := True
  end else begin
    chkTrayMinimizeHide.Enabled := False;

    chkTrayCloseHide.Checked := False;
    chkTrayCloseHide.Enabled := False;

    chkTrayMinimizeHide.Checked := False;
    chkTrayMinimizeHide.Enabled := False;

    lblTrayCloseHide.Enabled := False;

    chkTrayStartupMinimized.Enabled := False;

    frmMain.TrayIcon.Visible := False;
  end;

  if (chkTrayStartupMinimized.Checked = True) and (frmMain.Initialize = False) then begin
    Application.ShowMainForm := False;
    // 현재 메인폼이 활성화 될 때 초기화 루틴이 실행되므로 강제로 활성화 루틴을
    // 실행시킵니다. 구조상 좋지 못 한 부분인데 변경해야 합니다.
    frmMain.OnActivate(Sender);
  end;
end;

procedure TfrmOption.CheckAutoSave(StartCount: Integer = 1);

  procedure GetAutoSaveCount;
  var
    NumberType, FileName: string;
    Ext: string;
    Count, i : Integer;
  begin
    if cboFormat.ItemIndex = 0 then
      Ext := 'jpg'
    else
      Ext := Copy(cboFormat.Text, Length(cboFormat.Text) - 3, 3);

    // 자릿수에 맞춰 일련번호 표시
    NumberType := '';
    for i := 1 to edtNumberType.Value do
      NumberType := NumberType + '0';

    for Count := 1 to 10000 do
    begin
      FileName := edtPath.Text + edtFileName.Text + FormatFloat(NumberType, Count) + '.' + Ext;

      if FileExists(FileName) = False then
      begin
        AutoSaveCount := Count;
        Break;
      end;

    end;

  end;

begin
  if chkAutoSave.Checked = True then
    GetAutoSaveCount;

end;

procedure TfrmOption.btnOkClick(Sender: TObject);
begin
  SaveINI(Self, True, INIProgramSetupSectionName, INIPath);

  CheckAutoSave;

  if Sender = btnOk then
    ModalResult := mrOK;
end;

procedure TfrmOption.btnSelectFolderClick(Sender: TObject);
var
  DirPath, AToken: string;
begin
  if SelectDirectory('Select Directory. : ', '', DirPath) = True
    then
  begin

    AToken := Copy(Dirpath, Length(Dirpath), 1);

    if AToken <> '\' then
      edtPath.Text := DirPath + '\'
    else
      edtPath.Text := DirPath;

  end;

end;

procedure TfrmOption.btnWavClick(Sender: TObject);
begin
  if OpenWavDialog.Execute = True then
    edtWavPath.Text := OpenWavDialog.FileName;
end;

procedure TfrmOption.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
    btnOk.OnClick(nil);
end;

procedure TfrmOption.edtDelayKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
    Key := #0;
end;

procedure TfrmOption.gbJpegChange(Sender: TObject);
begin
  udJpeg.Position := gbJpeg.Position;
end;

procedure TfrmOption.udJpegChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  gbJpeg.Position := udJpeg.Position;
end;

procedure TfrmOption.edtJpegChange(Sender: TObject);
begin
  gbJpeg.Position := udJpeg.Position;
end;

procedure TfrmOption.tmrTimeTimer(Sender: TObject);
begin
  stTime.Caption := FormatDateTime(edtTime.Text, Now);
end;

procedure TfrmOption.chkTimeClick(Sender: TObject);
begin
  tmrTime.Enabled := chkTime.Checked;
end;

procedure TfrmOption.chkTrayIconUseClick(Sender: TObject);
begin
  if chkTrayIconUse.Checked = True then begin
    chkTrayMinimizeHide.Enabled := True;

    chkTrayCloseHide.Enabled := True;
    chkTrayMinimizeHide.Enabled := True;

    lblTrayCloseHide.Enabled := True;

    chkTrayStartupMinimized.Enabled := True;

    // 사용자의 편의를 위해서 기본적인 기능에 대해서 미리 체크를 해줍니다.
    chkTrayCloseHide.Checked := True;
    chkTrayStartupMinimized.Checked := True;
  end else begin
    chkTrayMinimizeHide.Enabled := False;

    chkTrayCloseHide.Checked := False;
    chkTrayCloseHide.Enabled := False;

    chkTrayMinimizeHide.Checked := False;
    chkTrayMinimizeHide.Enabled := False;

    lblTrayCloseHide.Enabled := False;

    chkTrayStartupMinimized.Checked := False;
    chkTrayStartupMinimized.Enabled := False;
  end;

end;

procedure TfrmOption.spdExTime1Click(Sender: TObject);
begin
  edtTime.Text := 'HH시 NN분 SS초';
end;

procedure TfrmOption.spdExTime2Click(Sender: TObject);
begin
  edtTime.Text := 'HH시 NN분 SS초 AM/PM';
end;

procedure TfrmOption.spdExTime3Click(Sender: TObject);
begin
  edtTime.Text := 'YYYY년 MM월 DD일';
end;

procedure TfrmOption.spdExTime4Click(Sender: TObject);
begin
  edtTime.Text := 'DDD요일 HH시 NN분';
end;

end.
