////////////////////////////////////////////////////////////////////////////////
//
// BMPLoader.pas - BMP wrapper
// ---------------------------
// Version:   2003-01-31
// Maintain:  Michael Vinther         mv@logicnet.dk
//
// Last changes:
//
unit BMPLoader;

interface

implementation

uses SysUtils, LinarBitmap, Streams, Graphics, DelphiStream;

type
  TBMPLoader = class(TBitmapLoader)
             public
               function CanLoad(const Ext: string): Boolean; override;
               function CanSave(const Ext: string): Boolean; override;

               function GetLoadFilter: string; override;

               procedure LoadFromStream(Stream: TSeekableStream; const Ext: string; Bitmap: TLinarBitmap); override;
               procedure SaveToStream(Stream: TSeekableStream; const Ext: string; LBitmap: TLinarBitmap); override;
             end;

function TBMPLoader.GetLoadFilter: string;
begin
  Result:='Windows bitmap (*.bmp)|*.bmp';
end;

function TBMPLoader.CanLoad(const Ext: string): Boolean;
begin
  Result:=Ext='BMP';
end;

function TBMPLoader.CanSave(const Ext: string): Boolean;
begin
  Result:=Ext='BMP';
end;

procedure TBMPLoader.LoadFromStream(Stream: TSeekableStream; const Ext: string; Bitmap: TLinarBitmap);
var
  BMP : TBitmap;
  DelphiStream : TSeekableDelphiStream;
begin
  BMP:=TBitmap.Create;
  try
    DelphiStream:=TSeekableDelphiStream.Create(Stream);
    try
      BMP.LoadFromStream(DelphiStream);
    finally
      DelphiStream.Free;
    end;
    if BMP.Pixelformat in [pf1bit,pf4bit] then BMP.Pixelformat:=pf8bit
    else if BMP.Pixelformat in [pfDevice, pf15bit, pf16bit, pf32bit, pfCustom] then BMP.Pixelformat:=pf24bit;
    Bitmap.Assign(BMP);
  finally
   BMP.Free;
  end;
end;

procedure TBMPLoader.SaveToStream(Stream: TSeekableStream; const Ext: string; LBitmap: TLinarBitmap);
var
  Bitmap : TBitmap;
  DelphiStream : TSeekableDelphiStream;
begin
  Bitmap:=TBitmap.Create;
  try
    LBitmap.AssignTo(Bitmap);
    DelphiStream:=TSeekableDelphiStream.Create(Stream);
    try
      Bitmap.SaveToStream(DelphiStream);
    finally
      DelphiStream.Free;
    end;
  finally
    Bitmap.Free;
  end;
end;

var
  Loader : TBMPLoader;

initialization
  Loader:=TBMPLoader.Create;
  LinarBitmap.AddLoader(Loader);
finalization
  Loader.Free;
end.

