/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ICOLoader.pas - Windows icon and cursor coding/decoding
// -------------------------------------------------------
// Version:   2002-12-31
// Maintain:  Michael Vinther    |     mv@logicnet.dk
//
// Last changes:
//   Delphi 6 compatibility
// ** ڰ Ϻκ  ģ ̴. ** //
unit userICOLoader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, IconList,
  LinarBitmap, Streams, StdCtrls, ExtCtrls, PanelFrame, MemUtils, Math;



 Type TICOLoader = class(TBitmapLoader)
                 public
                   TransparentColor : TColor;
                   AllwaysLoadFirst : Boolean; // Set to load first icon in a file and not show dialog
                   CursorHotSpot    : TPoint;

                   constructor Create;

                   function CanLoad(Ext: string): Boolean; override;
                   function CanSave(Ext: string): Boolean; override;

                   function GetLoadFilter: string; override;

                   procedure LoadFromStream(Stream: TSeekableStream; Ext: string; Bitmap: TLinarBitmap); override;
                   procedure SaveToStream(Stream: TSeekableStream; Ext: string; Bitmap: TLinarBitmap); override;
                 end;

var
  Default : TICOLoader;

implementation

uses uSaveCursor;


constructor TICOLoader.Create;
begin
  inherited Create;
  TransparentColor:=$ff00ff;
end;

function TICOLoader.CanLoad(Ext: string): Boolean;
begin
  Ext:=UpperCase(Ext);
  Result:=(Ext='ICO') or (Ext='CUR');
end;

function TICOLoader.CanSave(Ext: string): Boolean;
begin
  Ext:=UpperCase(Ext);
  Result:=(Ext='ICO') or (Ext='CUR');
end;

function TICOLoader.GetLoadFilter: string;
begin
  Result:='Windows icon (*.ico)|*.ico|Windows cursor (*.cur)|*.cur';
end;

procedure TICOLoader.LoadFromStream(Stream: TSeekableStream; Ext: string; Bitmap: TLinarBitmap);
var
  List : TIconList;
  IconIndex : Integer;
begin
  List:=TIconList.Create;
  try
    List.LoadFromStream(Stream);
    if List.Count=0 then raise ELinearBitmap.Create(' Ͽ  ϴ.')
    else if AllwaysLoadFirst then IconIndex:=0
    else {with TfrmIconSelection.Create(Application) do
    try
      IconList:=List;
      ColorDialog.Color:=Self.TransparentColor;
      if ShowModal<>mrOk then Abort;
      Self.TransparentColor:=ColorDialog.Color;
      IconIndex:=SelectBox.ItemIndex;
    finally
      Free;
    end;   }
    IconIndex:=0;
   if List.ListType=ltCursor then CursorHotSpot:=List.Icons[IconIndex].HotSpot;
    List.Icons[IconIndex].AssignTo(Bitmap,TransparentColor);
  finally
    List.Free;
  end;
end;

// **   Լ ** //

function IsIconSize(W,H:Integer):Boolean;
begin
Result:=False;
if w <> h then Exit;
if (w = 16) or (w = 24) or (w = 32) or (w = 48) or (w = 64)
or (w = 72) or (w = 96) or (w = 128) then Result:=True;
end;

procedure TICOLoader.SaveToStream(Stream: TSeekableStream; Ext: string; Bitmap: TLinarBitmap);
var
  List : TIconList;
  I, LastColor, ColorCount : Integer;
  X : DWord;
  TR, TG, TB : Byte;
  Pix8, MaskPtr, Dest : PByte;
  Pix24 : PRGBRec;
  FlipTransparentColor : TColor;
begin
  if not IsIconSize(Bitmap.Width,Bitmap.Height) then raise Exception.Create('̹     ϴ.');
  List:=TIconList.Create;
  try
    if UpperCase(Ext)='ICO' then List.ListType:=ltIcon
    else List.ListType:=ltCursor;

    if Bitmap.PixelFormat=pf24bit then
    begin
      //FlipTransparentColor:=RGB2BGR(TransparentColor);
      FlipTransparentColor:=RGB(255,0,255);
      List.AddIcon(Bitmap.Width,Bitmap.Height,24);
      with List.Icons[0] do
      begin
        ZeroMem(Mask^,MaskSize);
        for I:=0 to Height-1 do
        begin
          Pix24:=@Image^[I*BytesPerLine];
          Move(Bitmap.ScanLine[Height-1-I]^,Pix24^,Bitmap.BytesPerLine);
          MaskPtr:=@Mask^[I*MaskBytesPerLine];
          for X:=0 to Width-1 do
          begin
            if PColor(Pix24)^ and $ffffff=FlipTransparentColor then
            begin
              Pix24^:=BlackPix24;
              PByte(DWord(MaskPtr)+X shr 3)^:=PByte(DWord(MaskPtr)+X shr 3)^ or (128 shr (X and 7));
            end;
            Inc(Pix24);
          end;
        end;
      end;
    end
    else
    begin
      TR:=TransparentColor and $ff; TG:=(TransparentColor shr 8) and $ff; TB:=(TransparentColor shr 16) and $ff;

      // Find number of colors used
      LastColor:=1;
      with Bitmap do for I:=0 to Size-1 do if (Map^[I]>LastColor) then
        with Bitmap.Palette^[Map^[I]] do
          if (R<>TR) or (G<>TG) or (B<>TB) then LastColor:=Bitmap.Map^[I];
      ColorCount:=LastColor+1;

      {if ColorCount<=2 then
      begin
        List.AddIcon(Bitmap.Width,Bitmap.Height,1);
        ColorCount:=2;
      end
      else} if ColorCount<=16 then
      begin
        List.AddIcon(Bitmap.Width,Bitmap.Height,4);
        ColorCount:=16;
        with List.Icons[0] do
        begin
          ZeroMem(Mask^,MaskSize);
          ZeroMem(Image^,ImageSize);
          for I:=0 to Height-1 do
          begin
            Pix8:=Bitmap.ScanLine[Height-1-I];
            Dest:=@Image^[I*BytesPerLine];
            MaskPtr:=@Mask^[I*MaskBytesPerLine];
            for X:=0 to Width-1 do
            begin
              with Bitmap.Palette^[Pix8^] do if (R=TR) and (G=TG) and (B=TB) then
                PByte(DWord(MaskPtr)+X shr 3)^:=PByte(DWord(MaskPtr)+X shr 3)^ or (128 shr (X and 7))
              else
                Dest^:=Dest^ or (Pix8^ shl ((not X and 1) shl 2));

              if X and 1=1 then Inc(Dest);
              Inc(Pix8);
            end;
          end;
        end;
      end
      else // 256 colors
      begin
        List.AddIcon(Bitmap.Width,Bitmap.Height,8);
        ColorCount:=256;
        with List.Icons[0] do
        begin
          ZeroMem(Mask^,MaskSize);
          for I:=0 to Height-1 do
          begin
            Pix8:=@Image^[I*BytesPerLine];
            Move(Bitmap.ScanLine[Height-1-I]^,Pix8^,Bitmap.BytesPerLine);
            MaskPtr:=@Mask^[I*MaskBytesPerLine];
            for X:=0 to Width-1 do
            begin
              with Bitmap.Palette^[Pix8^] do if (R=TR) and (G=TG) and (B=TB) then
              begin
                Pix8^:=0;
                PByte(DWord(MaskPtr)+X shr 3)^:=PByte(DWord(MaskPtr)+X shr 3)^ or (128 shr (X and 7));
              end;
              Inc(Pix8);
            end;
          end;
        end;
      end;

      with List.Icons[0] do // Convert palette
      begin
        MakeLogPalette(Bitmap.Palette^,Palette^,ColorCount);
        for I:=0 to ColorCount-1 do with Palette^[I] do
          if (peRed=TR) and (peGreen=TG) and (peBlue=TB) then DWord(Palette^[I]):=0;
      end;
    end;

    {// ************  ߰ ҽ *************** //
    if Ext = 'cur' then
     With TfrmIconSelection.Create(Application) do
      try
      List.Icons[0].Draw(0,0,imgCursor.Canvas);
      if ShowModal <> mrOk then
       Exit; 
      finally
      Free;
      end;
    // ********************************************* //}
    List.Icons[0].HotSpot:=FHotSpot;
    List.SaveToStream(Stream);
  finally
    List.Free;
  end;
end;

initialization
  Default:=TICOLoader.Create;
  LinarBitmap.AddLoader(Default);
finalization
  Default.Free;
end.
